@@ -40,12 +40,6 @@ nsChromeProtocolHandler::GetScheme(nsACString& result) {
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsChromeProtocolHandler::GetDefaultPort(int32_t* result) {
-  *result = -1;  // no port for chrome: URLs
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsChromeProtocolHandler::AllowPort(int32_t port, const char* scheme,
                                    bool* _retval) {
@@ -54,12 +48,6 @@ nsChromeProtocolHandler::AllowPort(int32_t port, const char* scheme,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsChromeProtocolHandler::GetProtocolFlags(uint32_t* result) {
-  *result = URI_STD | URI_IS_UI_RESOURCE | URI_IS_LOCAL_RESOURCE;
-  return NS_OK;
-}
-
 /* static */ nsresult nsChromeProtocolHandler::CreateNewURI(
     const nsACString& aSpec, const char* aCharset, nsIURI* aBaseURI,
     nsIURI** result) {
@@ -82,7 +70,7 @@ nsChromeProtocolHandler::GetProtocolFlags(uint32_t* result) {
   // and "chrome://navigator/content/navigator.xul".
 
   rv = nsChromeRegistry::Canonify(surl);
-  if (NS_FAILED(rv)) return rv;
+  mozilla::Unused << NS_WARN_IF(NS_FAILED(rv));
 
   surl.forget(result);
   return NS_OK;
@@ -98,21 +86,9 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI, nsILoadInfo* aLoadInfo,
 
   MOZ_ASSERT(aResult, "Null out param");
 
-#ifdef DEBUG
-  // Check that the uri we got is already canonified
-  nsresult debug_rv;
   nsCOMPtr<nsIURI> debugURL = aURI;
-  debug_rv = nsChromeRegistry::Canonify(debugURL);
-  if (NS_SUCCEEDED(debug_rv)) {
-    bool same;
-    debug_rv = aURI->Equals(debugURL, &same);
-    if (NS_SUCCEEDED(debug_rv)) {
-      NS_ASSERTION(same,
-                   "Non-canonified chrome uri passed to "
-                   "nsChromeProtocolHandler::NewChannel!");
-    }
-  }
-#endif
+  rv = nsChromeRegistry::Canonify(debugURL);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIChannel> result;
 
@@ -166,12 +142,13 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI, nsILoadInfo* aLoadInfo,
   rv = result->SetOriginalURI(aURI);
   if (NS_FAILED(rv)) return rv;
 
-  // Get a system principal for content files and set the owner
-  // property of the result
-  nsCOMPtr<nsIURL> url = do_QueryInterface(aURI);
+  // Use a system principal for /content and /skin files.
+  // See bug 1855225 for discussion about whether to extend it more generally
+  // to other chrome:// URIs.
   nsAutoCString path;
-  rv = url->GetPathQueryRef(path);
-  if (StringBeginsWith(path, "/content/"_ns)) {
+  aURI->GetPathQueryRef(path);
+  if (StringBeginsWith(path, "/content/"_ns) ||
+      StringBeginsWith(path, "/skin/"_ns)) {
     result->SetOwner(nsContentUtils::GetSystemPrincipal());
   }
 
@@ -182,8 +159,7 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI, nsILoadInfo* aLoadInfo,
   // See bug 531886, bug 533038.
   result->SetContentCharset("UTF-8"_ns);
 
-  *aResult = result;
-  NS_ADDREF(*aResult);
+  result.forget(aResult);
   return NS_OK;
 }
 
