@@ -18,13 +18,10 @@
 #include "mozilla/IntegerPrintfMacros.h"
 
 #include "nsCOMPtr.h"
-#include "nsIPrefService.h"
-#include "nsIServiceManager.h"
-#include "nsMemory.h"
 #include "nsNativeCharsetUtils.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/SharedLibrary.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/SecurityManagerSslMetrics.h"
 
 #include "nsAuthGSSAPI.h"
 
@@ -77,22 +74,23 @@ static struct GSSFunction {
 static bool gssNativeImp = true;
 static PRLibrary* gssLibrary = nullptr;
 
-#define gss_display_status_ptr ((gss_display_status_type)*gssFuncs[0].func)
-#define gss_init_sec_context_ptr ((gss_init_sec_context_type)*gssFuncs[1].func)
-#define gss_indicate_mechs_ptr ((gss_indicate_mechs_type)*gssFuncs[2].func)
-#define gss_release_oid_set_ptr ((gss_release_oid_set_type)*gssFuncs[3].func)
+#define gss_display_status_ptr ((gss_display_status_type) * gssFuncs[0].func)
+#define gss_init_sec_context_ptr \
+  ((gss_init_sec_context_type) * gssFuncs[1].func)
+#define gss_indicate_mechs_ptr ((gss_indicate_mechs_type) * gssFuncs[2].func)
+#define gss_release_oid_set_ptr ((gss_release_oid_set_type) * gssFuncs[3].func)
 #define gss_delete_sec_context_ptr \
-  ((gss_delete_sec_context_type)*gssFuncs[4].func)
-#define gss_import_name_ptr ((gss_import_name_type)*gssFuncs[5].func)
-#define gss_release_buffer_ptr ((gss_release_buffer_type)*gssFuncs[6].func)
-#define gss_release_name_ptr ((gss_release_name_type)*gssFuncs[7].func)
-#define gss_wrap_ptr ((gss_wrap_type)*gssFuncs[8].func)
-#define gss_unwrap_ptr ((gss_unwrap_type)*gssFuncs[9].func)
+  ((gss_delete_sec_context_type) * gssFuncs[4].func)
+#define gss_import_name_ptr ((gss_import_name_type) * gssFuncs[5].func)
+#define gss_release_buffer_ptr ((gss_release_buffer_type) * gssFuncs[6].func)
+#define gss_release_name_ptr ((gss_release_name_type) * gssFuncs[7].func)
+#define gss_wrap_ptr ((gss_wrap_type) * gssFuncs[8].func)
+#define gss_unwrap_ptr ((gss_unwrap_type) * gssFuncs[9].func)
 
 #ifdef XP_MACOSX
 static PRFuncPtr KLCacheHasValidTicketsPtr;
 #  define KLCacheHasValidTickets_ptr \
-    ((KLCacheHasValidTickets_type)*KLCacheHasValidTicketsPtr)
+    ((KLCacheHasValidTickets_type) * KLCacheHasValidTicketsPtr)
 #endif
 
 static nsresult gssInit() {
@@ -119,9 +117,9 @@ static nsresult gssInit() {
   } else {
 #ifdef XP_WIN
 #  ifdef _WIN64
-    NS_NAMED_LITERAL_STRING(kLibName, "gssapi64.dll");
+    constexpr auto kLibName = u"gssapi64.dll"_ns;
 #  else
-    NS_NAMED_LITERAL_STRING(kLibName, "gssapi32.dll");
+    constexpr auto kLibName = u"gssapi32.dll"_ns;
 #  endif
 
     lib = LoadLibraryWithFlags(kLibName.get());
@@ -137,7 +135,7 @@ static nsresult gssInit() {
         "libcom_err.so", "libkrb5.so",   "libgssapi.so"};
 
     PRLibSpec libSpec;
-    for (size_t i = 0; i < ArrayLength(verLibNames); ++i) {
+    for (size_t i = 0; i < std::size(verLibNames); ++i) {
       libSpec.type = PR_LibSpec_Pathname;
       libSpec.value.pathname = verLibNames[i];
       lib = PR_LoadLibraryWithFlags(libSpec, PR_LD_GLOBAL);
@@ -153,7 +151,7 @@ static nsresult gssInit() {
         "libgssapi.so.1"       /* Heimdal - Suse9, CITI - FC, MDK, Suse10*/
     };
 
-    for (size_t i = 0; i < ArrayLength(verLibNames) && !lib; ++i) {
+    for (size_t i = 0; i < std::size(verLibNames) && !lib; ++i) {
       lib = PR_LoadLibrary(verLibNames[i]);
 
       /* The CITI libgssapi library calls exit() during
@@ -170,7 +168,7 @@ static nsresult gssInit() {
       }
     }
 
-    for (size_t i = 0; i < ArrayLength(libNames) && !lib; ++i) {
+    for (size_t i = 0; i < std::size(libNames) && !lib; ++i) {
       char* libName = PR_GetLibraryName(nullptr, libNames[i]);
       if (libName) {
         lib = PR_LoadLibrary(libName);
@@ -194,10 +192,10 @@ static nsresult gssInit() {
 
   LOG(("Attempting to load gss functions\n"));
 
-  for (size_t i = 0; i < ArrayLength(gssFuncs); ++i) {
-    gssFuncs[i].func = PR_FindFunctionSymbol(lib, gssFuncs[i].str);
-    if (!gssFuncs[i].func) {
-      LOG(("Fail to load %s function from gssapi library\n", gssFuncs[i].str));
+  for (auto& gssFunc : gssFuncs) {
+    gssFunc.func = PR_FindFunctionSymbol(lib, gssFunc.str);
+    if (!gssFunc.func) {
+      LOG(("Fail to load %s function from gssapi library\n", gssFunc.str));
       PR_UnloadLibrary(lib);
       return NS_ERROR_FAILURE;
     }
@@ -265,8 +263,6 @@ nsAuthGSSAPI::nsAuthGSSAPI(pType package) : mServiceFlags(REQ_DEFAULT) {
 
   LOG(("entering nsAuthGSSAPI::nsAuthGSSAPI()\n"));
 
-  mComplete = false;
-
   if (!gssLibrary && NS_FAILED(gssInit())) return;
 
   mCtx = GSS_C_NO_CONTEXT;
@@ -312,6 +308,8 @@ void nsAuthGSSAPI::Reset() {
   }
   mCtx = GSS_C_NO_CONTEXT;
   mComplete = false;
+  mDelegationRequested = false;
+  mDelegationSupported = false;
 }
 
 /* static */
@@ -326,14 +324,15 @@ void nsAuthGSSAPI::Shutdown() {
 NS_IMPL_ISUPPORTS(nsAuthGSSAPI, nsIAuthModule)
 
 NS_IMETHODIMP
-nsAuthGSSAPI::Init(const char* serviceName, uint32_t serviceFlags,
-                   const char16_t* domain, const char16_t* username,
-                   const char16_t* password) {
+nsAuthGSSAPI::Init(const nsACString& serviceName, uint32_t serviceFlags,
+                   const nsAString& domain, const nsAString& username,
+                   const nsAString& password) {
   // we don't expect to be passed any user credentials
-  NS_ASSERTION(!domain && !username && !password, "unexpected credentials");
+  NS_ASSERTION(domain.IsEmpty() && username.IsEmpty() && password.IsEmpty(),
+               "unexpected credentials");
 
   // it's critial that the caller supply a service name to be used
-  NS_ENSURE_TRUE(serviceName && *serviceName, NS_ERROR_INVALID_ARG);
+  NS_ENSURE_TRUE(!serviceName.IsEmpty(), NS_ERROR_INVALID_ARG);
 
   LOG(("entering nsAuthGSSAPI::Init()\n"));
 
@@ -344,10 +343,10 @@ nsAuthGSSAPI::Init(const char* serviceName, uint32_t serviceFlags,
 
   static bool sTelemetrySent = false;
   if (!sTelemetrySent) {
-    mozilla::Telemetry::Accumulate(mozilla::Telemetry::NTLM_MODULE_USED_2,
-                                   serviceFlags & nsIAuthModule::REQ_PROXY_AUTH
-                                       ? NTLM_MODULE_KERBEROS_PROXY
-                                       : NTLM_MODULE_KERBEROS_DIRECT);
+    mozilla::glean::security::ntlm_module_used.AccumulateSingleSample(
+        serviceFlags & nsIAuthModule::REQ_PROXY_AUTH
+            ? NTLM_MODULE_KERBEROS_PROXY
+            : NTLM_MODULE_KERBEROS_DIRECT);
     sTelemetrySent = true;
   }
 
@@ -359,6 +358,7 @@ nsAuthGSSAPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
                            void** outToken, uint32_t* outTokenLen) {
   OM_uint32 major_status, minor_status;
   OM_uint32 req_flags = 0;
+  OM_uint32 ret_flags = 0;
   gss_buffer_desc input_token = GSS_C_EMPTY_BUFFER;
   gss_buffer_desc output_token = GSS_C_EMPTY_BUFFER;
   gss_buffer_t in_token_ptr = GSS_C_NO_BUFFER;
@@ -373,7 +373,22 @@ nsAuthGSSAPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
   // If they've called us again after we're complete, reset to start afresh.
   if (mComplete) Reset();
 
-  if (mServiceFlags & REQ_DELEGATE) req_flags |= GSS_C_DELEG_FLAG;
+  // Two-phase delegation logic
+  // Phase 1: Try authentication without delegation first
+  // Phase 2: Only retry with delegation if server supports it (ret_flags)
+  bool delegationConfigured = (mServiceFlags & REQ_DELEGATE) != 0;
+
+  if (delegationConfigured) {
+    if (!mDelegationRequested) {
+      // First attempt: don't request delegation yet
+      LOG(("First auth attempt without delegation"));
+      mDelegationRequested = true;
+    } else if (mDelegationSupported) {
+      // Second attempt: server supports delegation, now request it
+      LOG(("Retrying auth with delegation - server supports it"));
+      req_flags |= GSS_C_DELEG_FLAG;
+    }
+  }
 
   if (mServiceFlags & REQ_MUTUAL_AUTH) req_flags |= GSS_C_MUTUAL_FLAG;
 
@@ -427,7 +442,7 @@ nsAuthGSSAPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
     major_status = gss_init_sec_context_ptr(
         &minor_status, GSS_C_NO_CREDENTIAL, &mCtx, server, mMechOID, req_flags,
         GSS_C_INDEFINITE, GSS_C_NO_CHANNEL_BINDINGS, in_token_ptr, nullptr,
-        &output_token, nullptr, nullptr);
+        &output_token, &ret_flags, nullptr);
 
   if (GSS_ERROR(major_status)) {
     LogGssError(major_status, minor_status, "gss_init_sec_context() failed");
@@ -435,6 +450,27 @@ nsAuthGSSAPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
     rv = NS_ERROR_FAILURE;
     goto end;
   }
+  // Check if server supports delegation (OK-AS-DELEGATE equivalent)
+  if (delegationConfigured && !mDelegationSupported &&
+      (ret_flags & GSS_C_DELEG_FLAG)) {
+    LOG(("Server supports delegation (GSS_C_DELEG_FLAG in ret_flags)"));
+
+    // If we completed without requesting delegation, but server supports it,
+    // we need to restart with delegation
+    if (major_status == GSS_S_COMPLETE && !(req_flags & GSS_C_DELEG_FLAG)) {
+      LOG(("Restarting authentication to request delegation"));
+      Reset();
+
+      // These flags get cleared by Reset().
+      // Set them again to make sure the next call sets GSS_C_DELEG_FLAG
+      mDelegationRequested = true;
+      mDelegationSupported = true;
+
+      gss_release_name_ptr(&minor_status, &server);
+      return GetNextToken(inToken, inTokenLen, outToken, outTokenLen);
+    }
+  }
+
   if (major_status == GSS_S_COMPLETE) {
     // Mark ourselves as being complete, so that if we're called again
     // we know to start afresh.
@@ -448,17 +484,19 @@ nsAuthGSSAPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
   }
 
   *outTokenLen = output_token.length;
-  if (output_token.length != 0)
+  if (output_token.length != 0) {
     *outToken = moz_xmemdup(output_token.value, output_token.length);
-  else
+  } else {
     *outToken = nullptr;
+  }
 
   gss_release_buffer_ptr(&minor_status, &output_token);
 
-  if (major_status == GSS_S_COMPLETE)
+  if (major_status == GSS_S_COMPLETE) {
     rv = NS_SUCCESS_AUTH_FINISHED;
-  else
+  } else {
     rv = NS_OK;
+  }
 
 end:
   gss_release_name_ptr(&minor_status, &server);
@@ -490,10 +528,11 @@ nsAuthGSSAPI::Unwrap(const void* inToken, uint32_t inTokenLen, void** outToken,
 
   *outTokenLen = output_token.length;
 
-  if (output_token.length)
+  if (output_token.length) {
     *outToken = moz_xmemdup(output_token.value, output_token.length);
-  else
+  } else {
     *outToken = nullptr;
+  }
 
   gss_release_buffer_ptr(&minor_status, &output_token);
 
