@@ -14,18 +14,23 @@
 //
 
 #include "nsAuthSSPI.h"
+#include "nsComponentManagerUtils.h"
 #include "nsDNSService2.h"
-#include "nsIServiceManager.h"
 #include "nsIDNSService.h"
 #include "nsIDNSRecord.h"
-#include "nsMemory.h"
 #include "nsNetCID.h"
+#include "nsServiceManagerUtils.h"
 #include "nsCOMPtr.h"
 #include "nsICryptoHash.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/SecurityManagerSslMetrics.h"
+#include "mozilla/StaticPrefs_network.h"
 
 #include <windows.h>
 
+// for safer certificate parsing
+#include "nss/mozpkix/pkixutil.h"
+#include "nss/mozpkix/pkixder.h"
+
 #define SEC_SUCCESS(Status) ((Status) >= 0)
 
 #ifndef KERB_WRAP_NO_ENCRYPT
@@ -89,7 +94,7 @@ static nsresult InitSSPI() {
 
 //-----------------------------------------------------------------------------
 
-nsresult nsAuthSSPI::MakeSN(const char* principal, nsCString& result) {
+nsresult nsAuthSSPI::MakeSN(const nsACString& principal, nsCString& result) {
   nsresult rv;
 
   nsAutoCString buf(principal);
@@ -120,11 +125,15 @@ nsresult nsAuthSSPI::MakeSN(const char* principal, nsCString& result) {
                                   nsIDNSService::RESOLVE_CANONICAL_NAME, attrs,
                                   getter_AddRefs(record));
   if (NS_FAILED(rv)) return rv;
+  nsCOMPtr<nsIDNSAddrRecord> rec = do_QueryInterface(record);
+  if (!rec) {
+    return NS_ERROR_UNEXPECTED;
+  }
 
   nsAutoCString cname;
-  rv = record->GetCanonicalName(cname);
+  rv = rec->GetCanonicalName(cname);
   if (NS_SUCCEEDED(rv)) {
-    result = StringHead(buf, index) + NS_LITERAL_CSTRING("/") + cname;
+    result = StringHead(buf, index) + "/"_ns + cname;
     LOG(("Using SPN of [%s]\n", result.get()));
   }
   return rv;
@@ -146,11 +155,7 @@ nsAuthSSPI::~nsAuthSSPI() {
   Reset();
 
   if (mCred.dwLower || mCred.dwUpper) {
-#ifdef __MINGW32__
     (sspi->FreeCredentialsHandle)(&mCred);
-#else
-    (sspi->FreeCredentialHandle)(&mCred);
-#endif
     memset(&mCred, 0, sizeof(mCred));
   }
 }
@@ -173,9 +178,9 @@ void nsAuthSSPI::Reset() {
 NS_IMPL_ISUPPORTS(nsAuthSSPI, nsIAuthModule)
 
 NS_IMETHODIMP
-nsAuthSSPI::Init(const char* serviceName, uint32_t serviceFlags,
-                 const char16_t* domain, const char16_t* username,
-                 const char16_t* password) {
+nsAuthSSPI::Init(const nsACString& aServiceName, uint32_t aServiceFlags,
+                 const nsAString& aDomain, const nsAString& aUsername,
+                 const nsAString& aPassword) {
   LOG(("  nsAuthSSPI::Init\n"));
 
   mIsFirst = true;
@@ -184,7 +189,7 @@ nsAuthSSPI::Init(const char* serviceName, uint32_t serviceFlags,
 
   // The caller must supply a service name to be used. (For why we now require
   // a service name for NTLM, see bug 487872.)
-  NS_ENSURE_TRUE(serviceName && *serviceName, NS_ERROR_INVALID_ARG);
+  NS_ENSURE_TRUE(!aServiceName.IsEmpty(), NS_ERROR_INVALID_ARG);
 
   nsresult rv;
 
@@ -202,25 +207,25 @@ nsAuthSSPI::Init(const char* serviceName, uint32_t serviceFlags,
     // lookups. The incoming serviceName is in the format: "protocol@hostname",
     // SSPI expects
     // "<service class>/<hostname>", so swap the '@' for a '/'.
-    mServiceName.Assign(serviceName);
+    mServiceName = aServiceName;
     int32_t index = mServiceName.FindChar('@');
     if (index == kNotFound) return NS_ERROR_UNEXPECTED;
     mServiceName.Replace(index, 1, '/');
   } else {
     // Kerberos requires the canonical host, MakeSN takes care of this through a
     // DNS lookup.
-    rv = MakeSN(serviceName, mServiceName);
+    rv = MakeSN(aServiceName, mServiceName);
     if (NS_FAILED(rv)) return rv;
   }
 
-  mServiceFlags = serviceFlags;
+  mServiceFlags = aServiceFlags;
 
   SECURITY_STATUS rc;
 
   PSecPkgInfoW pinfo;
   rc = (sspi->QuerySecurityPackageInfoW)(package, &pinfo);
   if (rc != SEC_E_OK) {
-    LOG(("%s package not found\n", package));
+    LOG(("%S package not found\n", package));
     return NS_ERROR_UNEXPECTED;
   }
   mMaxTokenLen = pinfo->cbMaxToken;
@@ -234,11 +239,11 @@ nsAuthSSPI::Init(const char* serviceName, uint32_t serviceFlags,
   // domain, username, and password will be null if nsHttpNTLMAuth's
   // ChallengeReceived returns false for identityInvalid. Use default
   // credentials in this case by passing null for pai.
-  if (username && password) {
+  if (!aUsername.IsEmpty() && !aPassword.IsEmpty()) {
     // Keep a copy of these strings for the duration
-    mUsername.Assign(username);
-    mPassword.Assign(password);
-    mDomain.Assign(domain);
+    mUsername = aUsername;
+    mPassword = aPassword;
+    mDomain = aDomain;
     ai.Domain = reinterpret_cast<unsigned short*>(mDomain.BeginWriting());
     ai.DomainLength = mDomain.Length();
     ai.User = reinterpret_cast<unsigned short*>(mUsername.BeginWriting());
@@ -256,10 +261,10 @@ nsAuthSSPI::Init(const char* serviceName, uint32_t serviceFlags,
 
   static bool sTelemetrySent = false;
   if (!sTelemetrySent) {
-    mozilla::Telemetry::Accumulate(mozilla::Telemetry::NTLM_MODULE_USED_2,
-                                   serviceFlags & nsIAuthModule::REQ_PROXY_AUTH
-                                       ? NTLM_MODULE_WIN_API_PROXY
-                                       : NTLM_MODULE_WIN_API_DIRECT);
+    mozilla::glean::security::ntlm_module_used.AccumulateSingleSample(
+        aServiceFlags & nsIAuthModule::REQ_PROXY_AUTH
+            ? NTLM_MODULE_WIN_API_PROXY
+            : NTLM_MODULE_WIN_API_DIRECT);
     sTelemetrySent = true;
   }
 
@@ -276,8 +281,6 @@ nsAuthSSPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
   // String for end-point bindings.
   const char end_point[] = "tls-server-end-point:";
   const int end_point_length = sizeof(end_point) - 1;
-  const int hash_size = 32;  // Size of a SHA256 hash.
-  const int cbt_size = hash_size + end_point_length;
 
   SECURITY_STATUS rc;
   MS_TimeStamp ignored;
@@ -332,7 +335,63 @@ nsAuthSSPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
       // If we have stored a certificate, the Channel Binding Token
       // needs to be generated and sent in the first input buffer.
       if (mCertDERLength > 0) {
-        // First we create a proper Endpoint Binding structure.
+        // Default to SHA256 for compatibility, but detect SHA384 and SHA512
+        uint32_t hashAlgorithm = nsICryptoHash::SHA256;
+        uint32_t hashSize = 32;  // SHA256 hash size
+
+        // Compute the hash size.
+        [&]() {
+          if (!mozilla::StaticPrefs::network_auth_sspi_detect_hash()) {
+            // This check only exists to make sure that the hash algorithm check
+            // doesn't break previous working behaviour.
+            return;
+          }
+          using namespace mozilla::pkix;
+          Input certDER;
+
+          mozilla::pkix::Result pkixResult = certDER.Init(
+              static_cast<const uint8_t*>(mCertDERData), mCertDERLength);
+          if (pkixResult != Success) {
+            return;
+          }
+
+          BackCert cert(certDER, EndEntityOrCA::MustBeEndEntity, nullptr);
+          pkixResult = cert.Init();
+          if (pkixResult != Success) {
+            return;
+          }
+
+          // Parse the signature algorithm from the signed data
+          der::PublicKeyAlgorithm publicKeyAlg;
+          DigestAlgorithm digestAlg;
+          Reader signatureAlgorithmReader(cert.GetSignedData().algorithm);
+          pkixResult = der::SignatureAlgorithmIdentifierValue(
+              signatureAlgorithmReader, publicKeyAlg, digestAlg);
+
+          if (pkixResult != Success) {
+            return;
+          }
+          // Map digest algorithms to hash algorithms for Extended Protection
+          switch (digestAlg) {
+            case DigestAlgorithm::sha384:
+              hashAlgorithm = nsICryptoHash::SHA384;
+              hashSize = 48;  // SHA384 hash size
+              break;
+            case DigestAlgorithm::sha512:
+              hashAlgorithm = nsICryptoHash::SHA512;
+              hashSize = 64;  // SHA512 hash size
+              break;
+            case DigestAlgorithm::sha256:
+            default:
+              // Use SHA256 as default for compatibility
+              hashAlgorithm = nsICryptoHash::SHA256;
+              hashSize = 32;
+              break;
+          }
+        }();
+
+        // Create Endpoint Binding structure with correct size
+        const int cbt_size = hashSize + end_point_length;
         pendpoint_binding.dwInitiatorAddrType = 0;
         pendpoint_binding.cbInitiatorLength = 0;
         pendpoint_binding.dwInitiatorOffset = 0;
@@ -363,17 +422,19 @@ nsAuthSSPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
         memcpy(sspi_cbt_ptr, end_point, end_point_length);
         sspi_cbt_ptr += end_point_length;
 
-        // Start hashing. We are always doing SHA256, but depending
-        // on the certificate, a different alogirthm might be needed.
         nsAutoCString hashString;
+        nsresult rv = NS_ERROR_FAILURE;
 
-        nsresult rv;
         nsCOMPtr<nsICryptoHash> crypto;
         crypto = do_CreateInstance(NS_CRYPTO_HASH_CONTRACTID, &rv);
-        if (NS_SUCCEEDED(rv)) rv = crypto->Init(nsICryptoHash::SHA256);
-        if (NS_SUCCEEDED(rv))
+        if (NS_SUCCEEDED(rv)) {
+          rv = crypto->Init(hashAlgorithm);
+        }
+        if (NS_SUCCEEDED(rv)) {
           rv = crypto->Update((unsigned char*)mCertDERData, mCertDERLength);
+        }
         if (NS_SUCCEEDED(rv)) rv = crypto->Finish(false, hashString);
+
         if (NS_FAILED(rv)) {
           free(mCertDERData);
           mCertDERData = nullptr;
@@ -382,10 +443,9 @@ nsAuthSSPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
           return rv;
         }
 
-        // Once the hash has been computed, we store it in memory right
-        // after the Endpoint structure and the "tls-server-end-point:"
-        // char array.
-        memcpy(sspi_cbt_ptr, hashString.get(), hash_size);
+        // Store the computed hash in memory right after the Endpoint
+        // structure and the "tls-server-end-point:" char array
+        memcpy(sspi_cbt_ptr, hashString.get(), hashSize);
 
         // Free memory used to store the server certificate
         free(mCertDERData);
@@ -447,7 +507,7 @@ nsAuthSSPI::GetNextToken(const void* inToken, uint32_t inTokenLen,
     return NS_OK;
   }
 
-  LOG(("InitializeSecurityContext failed [rc=%d:%s]\n", rc, MapErrorCode(rc)));
+  LOG(("InitializeSecurityContext failed [rc=%ld:%s]\n", rc, MapErrorCode(rc)));
   Reset();
   free(ob.pvBuffer);
   return NS_ERROR_FAILURE;
