@@ -1,11 +1,9 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "PlacesTestUtils",
-  "resource://testing-common/PlacesTestUtils.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  PlacesTestUtils: "resource://testing-common/PlacesTestUtils.sys.mjs",
+});
 
 var PERMISSIONS_FILE_NAME = "permissions.sqlite";
 
@@ -26,36 +24,10 @@ add_task(async function test() {
   Assert.equal(pm.all.length, 0, "No cookies");
 
   let db = Services.storage.openDatabase(GetPermissionsFile(profile));
-  db.schemaVersion = 7;
+  db.schemaVersion = 4;
   db.executeSimpleSQL("DROP TABLE moz_perms");
   db.executeSimpleSQL("DROP TABLE moz_hosts");
 
-  /*
-   * V5 table
-   */
-  db.executeSimpleSQL(
-    "CREATE TABLE moz_perms (" +
-      " id INTEGER PRIMARY KEY" +
-      ",origin TEXT" +
-      ",type TEXT" +
-      ",permission INTEGER" +
-      ",expireType INTEGER" +
-      ",expireTime INTEGER" +
-      ",modificationTime INTEGER" +
-      ")"
-  );
-
-  let stmt6Insert = db.createStatement(
-    "INSERT INTO moz_perms (" +
-      "id, origin, type, permission, expireType, expireTime, modificationTime" +
-      ") VALUES (" +
-      ":id, :origin, :type, :permission, :expireType, :expireTime, :modificationTime" +
-      ")"
-  );
-
-  /*
-   * V4 table
-   */
   db.executeSimpleSQL(
     "CREATE TABLE moz_hosts (" +
       " id INTEGER PRIMARY KEY" +
@@ -78,50 +50,8 @@ add_task(async function test() {
       ")"
   );
 
-  /*
-   * The v4 table is a backup
-   */
-  db.executeSimpleSQL(
-    "CREATE TABLE moz_hosts_is_backup (dummy INTEGER PRIMARY KEY)"
-  );
-
   let id = 0;
 
-  function insertOrigin(
-    origin,
-    type,
-    permission,
-    expireType,
-    expireTime,
-    modificationTime
-  ) {
-    let thisId = id++;
-
-    stmt6Insert.bindByName("id", thisId);
-    stmt6Insert.bindByName("origin", origin);
-    stmt6Insert.bindByName("type", type);
-    stmt6Insert.bindByName("permission", permission);
-    stmt6Insert.bindByName("expireType", expireType);
-    stmt6Insert.bindByName("expireTime", expireTime);
-    stmt6Insert.bindByName("modificationTime", modificationTime);
-
-    try {
-      stmt6Insert.execute();
-    } finally {
-      stmt6Insert.reset();
-    }
-
-    return {
-      id: thisId,
-      origin,
-      type,
-      permission,
-      expireType,
-      expireTime,
-      modificationTime,
-    };
-  }
-
   function insertHost(
     host,
     type,
@@ -162,13 +92,6 @@ add_task(async function test() {
       isInBrowserElement,
     };
   }
-  // eslint-disable-next-line no-unused-vars
-  let created7 = [
-    insertOrigin("https://foo.com", "A", 2, 0, 0, 0),
-    insertOrigin("http://foo.com", "A", 2, 0, 0, 0),
-    insertOrigin("http://foo.com^inBrowser=1", "A", 2, 0, 0, 0),
-    insertOrigin("https://192.0.2.235", "A", 2, 0, 0),
-  ];
 
   // Add some rows to the database
   // eslint-disable-next-line no-unused-vars
@@ -185,10 +108,6 @@ add_task(async function test() {
     insertHost("localhost", "A", 1, 0, 0, 0, 0, false),
     insertHost("127.0.0.1", "A", 1, 0, 0, 0, 0, false),
     insertHost("192.0.2.235", "A", 1, 0, 0, 0, 0, false),
-    // Although ipv6 addresses are written with [] around the IP address,
-    // the .host property doesn't contain these []s, which means that we
-    // write it like this
-    insertHost("2001:db8::ff00:42:8329", "C", 1, 0, 0, 0, 0, false),
     insertHost("file:///some/path/to/file.html", "A", 1, 0, 0, 0, 0, false),
     insertHost("file:///another/file.html", "A", 1, 0, 0, 0, 0, false),
     insertHost(
@@ -216,31 +135,54 @@ add_task(async function test() {
   ];
 
   // CLose the db connection
-  stmt6Insert.finalize();
   stmtInsert.finalize();
   db.close();
   stmtInsert = null;
   db = null;
 
   let expected = [
-    // We should have kept the previously migrated entries
-    ["https://foo.com", "A", 2, 0, 0, 0],
-    ["http://foo.com", "A", 2, 0, 0, 0],
-    ["http://foo.com^inBrowser=1", "A", 2, 0, 0, 0],
+    // The http:// entries under foo.com won't be inserted, as there are history entries for foo.com,
+    // and http://foo.com or a subdomain are never visited.
+    // ["http://foo.com", "A", 1, 0, 0],
+    //
+    // Because we search for port/scheme combinations under eTLD+1, we should not have http:// entries
+    // for subdomains of foo.com either
+    // ["http://sub.foo.com", "B", 1, 0, 0],
+    // ["http://subber.sub.foo.com", "B", 1, 0, 0],
+
+    ["https://foo.com", "A", 1, 0, 0],
+    ["https://foo.com", "C", 1, 0, 0],
+    ["https://sub.foo.com", "B", 1, 0, 0],
+    ["https://subber.sub.foo.com", "B", 1, 0, 0],
+
+    // bar.ca will have both http:// and https:// for all entries, because there are no associated history entries
+    ["http://bar.ca", "B", 1, 0, 0],
+    ["https://bar.ca", "B", 1, 0, 0],
+    ["http://bar.ca", "A", 1, 0, 0],
+    ["https://bar.ca", "A", 1, 0, 0],
+    ["file:///some/path/to/file.html", "A", 1, 0, 0],
+    ["file:///another/file.html", "A", 1, 0, 0],
+
+    // Because we put ftp://some.subdomain.of.foo.com:8000/some/subdirectory in the history, we should
+    // also have these entries
+    ["ftp://foo.com:8000", "A", 1, 0, 0],
+    ["ftp://foo.com:8000", "C", 1, 0, 0],
+
+    // In addition, because we search for port/scheme combinations under eTLD+1, we should have the
+    // following entries
+    ["ftp://sub.foo.com:8000", "B", 1, 0, 0],
+    ["ftp://subber.sub.foo.com:8000", "B", 1, 0, 0],
 
     // Make sure that we also support localhost, and IP addresses
-    ["https://localhost:8080", "A", 1, 0, 0],
-    ["ftp://127.0.0.1:8080", "A", 1, 0, 0],
-
-    ["http://[2001:db8::ff00:42:8329]", "C", 1, 0, 0],
-    ["https://[2001:db8::ff00:42:8329]", "C", 1, 0, 0],
+    ["http://localhost", "A", 1, 0, 0],
+    ["https://localhost", "A", 1, 0, 0],
+    ["http://127.0.0.1", "A", 1, 0, 0],
+    ["https://127.0.0.1", "A", 1, 0, 0],
     ["http://192.0.2.235", "A", 1, 0, 0],
-
-    // There should only be one entry of this type in the database
-    ["https://192.0.2.235", "A", 2, 0, 0],
+    ["https://192.0.2.235", "A", 1, 0, 0],
   ];
 
-  let found = expected.map(it => 0);
+  let found = expected.map(() => 0);
 
   // Add some places to the places database
   await PlacesTestUtils.addVisits(
@@ -249,8 +191,6 @@ add_task(async function test() {
   await PlacesTestUtils.addVisits(
     Services.io.newURI("ftp://some.subdomain.of.foo.com:8000/some/subdirectory")
   );
-  await PlacesTestUtils.addVisits(Services.io.newURI("ftp://127.0.0.1:8080"));
-  await PlacesTestUtils.addVisits(Services.io.newURI("https://localhost:8080"));
 
   // This will force the permission-manager to reload the data.
   Services.obs.notifyObservers(null, "testonly-reload-permissions-from-disk");
@@ -316,15 +256,6 @@ add_task(async function test() {
       mozHostsCount.finalize();
     }
 
-    // Check that there are the right number of values in the permissions database
-    let mozPermsCount = db.createStatement("SELECT count(*) FROM moz_perms");
-    try {
-      mozPermsCount.executeStep();
-      Assert.equal(mozPermsCount.getInt64(0), expected.length);
-    } finally {
-      mozPermsCount.finalize();
-    }
-
     db.close();
   }
 });
