@@ -4,6 +4,7 @@
 
 #include "mozilla/dom/NetDashboardBinding.h"
 #include "mozilla/dom/ToJSValue.h"
+#include "mozilla/Components.h"
 #include "mozilla/ErrorNames.h"
 #include "mozilla/net/Dashboard.h"
 #include "mozilla/net/HttpInfo.h"
@@ -69,6 +70,21 @@ class HttpData : public nsISupports {
 
 NS_IMPL_ISUPPORTS0(HttpData)
 
+class Http3ConnectionStatsData : public nsISupports {
+  virtual ~Http3ConnectionStatsData() = default;
+
+ public:
+  NS_DECL_THREADSAFE_ISUPPORTS
+
+  Http3ConnectionStatsData() = default;
+
+  nsTArray<Http3ConnectionStatsParams> mData;
+  nsMainThreadPtrHandle<nsINetDashboardCallback> mCallback;
+  nsIEventTarget* mEventTarget{nullptr};
+};
+
+NS_IMPL_ISUPPORTS0(Http3ConnectionStatsData)
+
 class WebSocketRequest : public nsISupports {
   virtual ~WebSocketRequest() = default;
 
@@ -306,7 +322,7 @@ nsresult LookupHelper::ConstructAnswer(LookupArgument* aArgument) {
     GetErrorString(mStatus, dict.mError);
   }
 
-  JS::RootedValue val(cx);
+  JS::Rooted<JS::Value> val(cx);
   if (!ToJSValue(cx, dict, &val)) {
     return NS_ERROR_FAILURE;
   }
@@ -470,7 +486,7 @@ nsresult LookupHelper::ConstructHTTPSRRAnswer(LookupArgument* aArgument) {
     GetErrorString(mStatus, dict.mError);
   }
 
-  JS::RootedValue val(cx);
+  JS::Rooted<JS::Value> val(cx);
   if (!ToJSValue(cx, dict, &val)) {
     return NS_ERROR_FAILURE;
   }
@@ -489,7 +505,7 @@ Dashboard::RequestSockets(nsINetDashboardCallback* aCallback) {
   RefPtr<SocketData> socketData = new SocketData();
   socketData->mCallback = new nsMainThreadPtrHolder<nsINetDashboardCallback>(
       "nsINetDashboardCallback", aCallback, true);
-  socketData->mEventTarget = GetCurrentEventTarget();
+  socketData->mEventTarget = GetCurrentSerialEventTarget();
 
   if (nsIOService::UseSocketProcess()) {
     if (!gIOService->SocketProcessReady()) {
@@ -497,7 +513,9 @@ Dashboard::RequestSockets(nsINetDashboardCallback* aCallback) {
     }
 
     RefPtr<Dashboard> self(this);
-    SocketProcessParent::GetSingleton()->SendGetSocketData()->Then(
+    RefPtr<SocketProcessParent> socketParent =
+        SocketProcessParent::GetSingleton();
+    socketParent->SendGetSocketData()->Then(
         GetMainThreadSerialEventTarget(), __func__,
         [self{std::move(self)},
          socketData{std::move(socketData)}](SocketDataArgs&& args) {
@@ -558,7 +576,7 @@ nsresult Dashboard::GetSockets(SocketData* aSocketData) {
     CopyASCIItoUTF16(socketData->mData[i].host, mSocket.mHost);
     mSocket.mPort = socketData->mData[i].port;
     mSocket.mActive = socketData->mData[i].active;
-    mSocket.mTcp = socketData->mData[i].tcp;
+    CopyASCIItoUTF16(socketData->mData[i].type, mSocket.mType);
     mSocket.mSent = (double)socketData->mData[i].sent;
     mSocket.mReceived = (double)socketData->mData[i].received;
     dict.mSent += socketData->mData[i].sent;
@@ -567,7 +585,7 @@ nsresult Dashboard::GetSockets(SocketData* aSocketData) {
 
   dict.mSent += socketData->mTotalSent;
   dict.mReceived += socketData->mTotalRecv;
-  JS::RootedValue val(cx);
+  JS::Rooted<JS::Value> val(cx);
   if (!ToJSValue(cx, dict, &val)) return NS_ERROR_FAILURE;
   socketData->mCallback->OnDashboardDataAvailable(val);
 
@@ -579,7 +597,7 @@ Dashboard::RequestHttpConnections(nsINetDashboardCallback* aCallback) {
   RefPtr<HttpData> httpData = new HttpData();
   httpData->mCallback = new nsMainThreadPtrHolder<nsINetDashboardCallback>(
       "nsINetDashboardCallback", aCallback, true);
-  httpData->mEventTarget = GetCurrentEventTarget();
+  httpData->mEventTarget = GetCurrentSerialEventTarget();
 
   if (nsIOService::UseSocketProcess()) {
     if (!gIOService->SocketProcessReady()) {
@@ -587,7 +605,9 @@ Dashboard::RequestHttpConnections(nsINetDashboardCallback* aCallback) {
     }
 
     RefPtr<Dashboard> self(this);
-    SocketProcessParent::GetSingleton()->SendGetHttpConnectionData()->Then(
+    RefPtr<SocketProcessParent> socketParent =
+        SocketProcessParent::GetSingleton();
+    socketParent->SendGetHttpConnectionData()->Then(
         GetMainThreadSerialEventTarget(), __func__,
         [self{std::move(self)}, httpData](nsTArray<HttpRetParams>&& params) {
           httpData->mData.Assign(std::move(params));
@@ -682,7 +702,7 @@ nsresult Dashboard::GetHttpConnections(HttpData* aHttpData) {
     }
   }
 
-  JS::RootedValue val(cx);
+  JS::Rooted<JS::Value> val(cx);
   if (!ToJSValue(cx, dict, &val)) {
     return NS_ERROR_FAILURE;
   }
@@ -692,6 +712,127 @@ nsresult Dashboard::GetHttpConnections(HttpData* aHttpData) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+Dashboard::RequestHttp3ConnectionStats(nsINetDashboardCallback* aCallback) {
+  RefPtr<Http3ConnectionStatsData> data = new Http3ConnectionStatsData();
+  data->mCallback = new nsMainThreadPtrHolder<nsINetDashboardCallback>(
+      "nsINetDashboardCallback", aCallback, true);
+  data->mEventTarget = GetCurrentSerialEventTarget();
+
+  if (nsIOService::UseSocketProcess()) {
+    if (!gIOService->SocketProcessReady()) {
+      return NS_ERROR_NOT_AVAILABLE;
+    }
+
+    RefPtr<Dashboard> self(this);
+    RefPtr<SocketProcessParent> socketParent =
+        SocketProcessParent::GetSingleton();
+    socketParent->SendGetHttp3ConnectionStatsData()->Then(
+        GetMainThreadSerialEventTarget(), __func__,
+        [self{std::move(self)},
+         data](nsTArray<Http3ConnectionStatsParams>&& params) {
+          data->mData.Assign(std::move(params));
+          self->GetHttp3ConnectionStats(data);
+          data->mEventTarget->Dispatch(
+              NewRunnableMethod<RefPtr<Http3ConnectionStatsData>>(
+                  "net::Dashboard::GetHttp3ConnectionStats", self,
+                  &Dashboard::GetHttp3ConnectionStats, data),
+              NS_DISPATCH_NORMAL);
+        },
+        [self](const mozilla::ipc::ResponseRejectReason) {});
+    return NS_OK;
+  }
+
+  gSocketTransportService->Dispatch(
+      NewRunnableMethod<RefPtr<Http3ConnectionStatsData>>(
+          "net::Dashboard::GetHttp3ConnectionStatsDispatch", this,
+          &Dashboard::GetHttp3ConnectionStatsDispatch, data),
+      NS_DISPATCH_NORMAL);
+  return NS_OK;
+}
+
+nsresult Dashboard::GetHttp3ConnectionStatsDispatch(
+    Http3ConnectionStatsData* aData) {
+  RefPtr<Http3ConnectionStatsData> data = aData;
+  HttpInfo::GetHttp3ConnectionStatsData(&data->mData);
+  data->mEventTarget->Dispatch(
+      NewRunnableMethod<RefPtr<Http3ConnectionStatsData>>(
+          "net::Dashboard::GetHttp3ConnectionStats", this,
+          &Dashboard::GetHttp3ConnectionStats, data),
+      NS_DISPATCH_NORMAL);
+  return NS_OK;
+}
+
+nsresult Dashboard::GetHttp3ConnectionStats(Http3ConnectionStatsData* aData) {
+  RefPtr<Http3ConnectionStatsData> data = aData;
+  AutoSafeJSContext cx;
+
+  mozilla::dom::Http3ConnStatsDict dict;
+  dict.mConnections.Construct();
+
+  using mozilla::dom::Http3ConnectionStatsElement;
+  using mozilla::dom::Http3ConnStats;
+  Sequence<Http3ConnectionStatsElement>& connections =
+      dict.mConnections.Value();
+
+  uint32_t length = data->mData.Length();
+  if (!connections.SetCapacity(length, fallible)) {
+    JS_ReportOutOfMemory(cx);
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  for (uint32_t i = 0; i < data->mData.Length(); i++) {
+    Http3ConnectionStatsElement& connection =
+        *connections.AppendElement(fallible);
+
+    CopyASCIItoUTF16(data->mData[i].host, connection.mHost);
+    connection.mPort = data->mData[i].port;
+
+    connection.mStats.Construct();
+
+    Sequence<Http3ConnStats>& stats = connection.mStats.Value();
+
+    if (!stats.SetCapacity(data->mData[i].stats.Length(), fallible)) {
+      JS_ReportOutOfMemory(cx);
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+
+    for (uint32_t j = 0; j < data->mData[i].stats.Length(); j++) {
+      Http3ConnStats& info = *stats.AppendElement(fallible);
+      info.mPacketsRx = data->mData[i].stats[j].packetsRx;
+      info.mDupsRx = data->mData[i].stats[j].dupsRx;
+      info.mDroppedRx = data->mData[i].stats[j].droppedRx;
+      info.mSavedDatagrams = data->mData[i].stats[j].savedDatagrams;
+      info.mPacketsTx = data->mData[i].stats[j].packetsTx;
+      info.mLost = data->mData[i].stats[j].lost;
+      info.mLateAck = data->mData[i].stats[j].lateAck;
+      info.mPtoAck = data->mData[i].stats[j].ptoAck;
+      info.mWouldBlockRx = data->mData[i].stats[j].wouldBlockRx;
+      info.mWouldBlockTx = data->mData[i].stats[j].wouldBlockTx;
+      info.mPtoCounts.Construct();
+      Sequence<uint64_t>& ptoCounts = info.mPtoCounts.Value();
+      if (!ptoCounts.SetCapacity(data->mData[i].stats[j].ptoCounts.Length(),
+                                 fallible)) {
+        JS_ReportOutOfMemory(cx);
+        return NS_ERROR_OUT_OF_MEMORY;
+      }
+      for (auto pto : data->mData[i].stats[j].ptoCounts) {
+        uint64_t& element = *ptoCounts.AppendElement(fallible);
+        element = pto;
+      }
+    }
+  }
+
+  JS::Rooted<JS::Value> val(cx);
+  if (!ToJSValue(cx, dict, &val)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  data->mCallback->OnDashboardDataAvailable(val);
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 Dashboard::GetEnableLogging(bool* value) {
   *value = mEnableLogging;
@@ -765,7 +906,7 @@ Dashboard::RequestWebsocketConnections(nsINetDashboardCallback* aCallback) {
   RefPtr<WebSocketRequest> wsRequest = new WebSocketRequest();
   wsRequest->mCallback = new nsMainThreadPtrHolder<nsINetDashboardCallback>(
       "nsINetDashboardCallback", aCallback, true);
-  wsRequest->mEventTarget = GetCurrentEventTarget();
+  wsRequest->mEventTarget = GetCurrentSerialEventTarget();
 
   wsRequest->mEventTarget->Dispatch(
       NewRunnableMethod<RefPtr<WebSocketRequest>>(
@@ -801,7 +942,7 @@ nsresult Dashboard::GetWebSocketConnections(WebSocketRequest* aWsRequest) {
     websocket.mEncrypted = mWs.data[i].mEncrypted;
   }
 
-  JS::RootedValue val(cx);
+  JS::Rooted<JS::Value> val(cx);
   if (!ToJSValue(cx, dict, &val)) {
     return NS_ERROR_FAILURE;
   }
@@ -818,10 +959,10 @@ Dashboard::RequestDNSInfo(nsINetDashboardCallback* aCallback) {
 
   nsresult rv;
   dnsData->mData.Clear();
-  dnsData->mEventTarget = GetCurrentEventTarget();
+  dnsData->mEventTarget = GetCurrentSerialEventTarget();
 
   if (!mDnsService) {
-    mDnsService = do_GetService("@mozilla.org/network/dns-service;1", &rv);
+    mDnsService = mozilla::components::DNS::Service(&rv);
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -833,7 +974,9 @@ Dashboard::RequestDNSInfo(nsINetDashboardCallback* aCallback) {
     }
 
     RefPtr<Dashboard> self(this);
-    SocketProcessParent::GetSingleton()->SendGetDNSCacheEntries()->Then(
+    RefPtr<SocketProcessParent> socketParent =
+        SocketProcessParent::GetSingleton();
+    socketParent->SendGetDNSCacheEntries()->Then(
         GetMainThreadSerialEventTarget(), __func__,
         [self{std::move(self)},
          dnsData{std::move(dnsData)}](nsTArray<DNSCacheEntries>&& entries) {
@@ -905,17 +1048,21 @@ nsresult Dashboard::GetDNSCacheEntries(DnsData* dnsData) {
       CopyASCIItoUTF16(dnsData->mData[i].hostaddr[j], *addr);
     }
 
-    if (dnsData->mData[i].family == PR_AF_INET6) {
-      entry.mFamily.AssignLiteral(u"ipv6");
-    } else {
-      entry.mFamily.AssignLiteral(u"ipv4");
+    entry.mType = dnsData->mData[i].resolveType;
+    if (entry.mType == nsIDNSService::RESOLVE_TYPE_DEFAULT) {
+      if (dnsData->mData[i].family == PR_AF_INET6) {
+        entry.mFamily.AssignLiteral(u"ipv6");
+      } else {
+        entry.mFamily.AssignLiteral(u"ipv4");
+      }
     }
 
     entry.mOriginAttributesSuffix =
         NS_ConvertUTF8toUTF16(dnsData->mData[i].originAttributesSuffix);
+    entry.mFlags = NS_ConvertUTF8toUTF16(dnsData->mData[i].flags);
   }
 
-  JS::RootedValue val(cx);
+  JS::Rooted<JS::Value> val(cx);
   if (!ToJSValue(cx, dict, &val)) {
     return NS_ERROR_FAILURE;
   }
@@ -930,7 +1077,7 @@ Dashboard::RequestDNSLookup(const nsACString& aHost,
   nsresult rv;
 
   if (!mDnsService) {
-    mDnsService = do_GetService("@mozilla.org/network/dns-service;1", &rv);
+    mDnsService = mozilla::components::DNS::Service(&rv);
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -939,10 +1086,11 @@ Dashboard::RequestDNSLookup(const nsACString& aHost,
   RefPtr<LookupHelper> helper = new LookupHelper();
   helper->mCallback = new nsMainThreadPtrHolder<nsINetDashboardCallback>(
       "nsINetDashboardCallback", aCallback, true);
-  helper->mEventTarget = GetCurrentEventTarget();
+  helper->mEventTarget = GetCurrentSerialEventTarget();
   OriginAttributes attrs;
   rv = mDnsService->AsyncResolveNative(
-      aHost, nsIDNSService::RESOLVE_TYPE_DEFAULT, 0, nullptr, helper.get(),
+      aHost, nsIDNSService::RESOLVE_TYPE_DEFAULT,
+      nsIDNSService::RESOLVE_DEFAULT_FLAGS, nullptr, helper.get(),
       NS_GetCurrentThread(), attrs, getter_AddRefs(helper->mCancel));
   return rv;
 }
@@ -953,7 +1101,7 @@ Dashboard::RequestDNSHTTPSRRLookup(const nsACString& aHost,
   nsresult rv;
 
   if (!mDnsService) {
-    mDnsService = do_GetService("@mozilla.org/network/dns-service;1", &rv);
+    mDnsService = mozilla::components::DNS::Service(&rv);
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -962,10 +1110,11 @@ Dashboard::RequestDNSHTTPSRRLookup(const nsACString& aHost,
   RefPtr<LookupHelper> helper = new LookupHelper();
   helper->mCallback = new nsMainThreadPtrHolder<nsINetDashboardCallback>(
       "nsINetDashboardCallback", aCallback, true);
-  helper->mEventTarget = GetCurrentEventTarget();
+  helper->mEventTarget = GetCurrentSerialEventTarget();
   OriginAttributes attrs;
   rv = mDnsService->AsyncResolveNative(
-      aHost, nsIDNSService::RESOLVE_TYPE_HTTPSSVC, 0, nullptr, helper.get(),
+      aHost, nsIDNSService::RESOLVE_TYPE_HTTPSSVC,
+      nsIDNSService::RESOLVE_DEFAULT_FLAGS, nullptr, helper.get(),
       NS_GetCurrentThread(), attrs, getter_AddRefs(helper->mCancel));
   return rv;
 }
@@ -973,7 +1122,7 @@ Dashboard::RequestDNSHTTPSRRLookup(const nsACString& aHost,
 NS_IMETHODIMP
 Dashboard::RequestRcwnStats(nsINetDashboardCallback* aCallback) {
   RefPtr<RcwnData> rcwnData = new RcwnData();
-  rcwnData->mEventTarget = GetCurrentEventTarget();
+  rcwnData->mEventTarget = GetCurrentSerialEventTarget();
   rcwnData->mCallback = new nsMainThreadPtrHolder<nsINetDashboardCallback>(
       "nsINetDashboardCallback", aCallback, true);
 
@@ -1015,7 +1164,7 @@ nsresult Dashboard::GetRcwnData(RcwnData* aData) {
         CacheFileUtils::CachePerfStats::GetStdDev(perfType, true);
   }
 
-  JS::RootedValue val(cx);
+  JS::Rooted<JS::Value> val(cx);
   if (!ToJSValue(cx, dict, &val)) {
     return NS_ERROR_FAILURE;
   }
@@ -1069,7 +1218,7 @@ Dashboard::RequestConnection(const nsACString& aHost, uint32_t aPort,
   connectionData->mCallback =
       new nsMainThreadPtrHolder<nsINetDashboardCallback>(
           "nsINetDashboardCallback", aCallback, true);
-  connectionData->mEventTarget = GetCurrentEventTarget();
+  connectionData->mEventTarget = GetCurrentSerialEventTarget();
 
   rv = TestNewConnection(connectionData);
   if (NS_FAILED(rv)) {
@@ -1092,7 +1241,7 @@ nsresult Dashboard::GetConnectionStatus(ConnectionData* aConnectionData) {
   mozilla::dom::ConnStatusDict dict;
   dict.mStatus = connectionData->mStatus;
 
-  JS::RootedValue val(cx);
+  JS::Rooted<JS::Value> val(cx);
   if (!ToJSValue(cx, dict, &val)) return NS_ERROR_FAILURE;
 
   connectionData->mCallback->OnDashboardDataAvailable(val);
@@ -1105,7 +1254,7 @@ nsresult Dashboard::TestNewConnection(ConnectionData* aConnectionData) {
 
   nsresult rv;
   if (!connectionData->mHost.Length() ||
-      !net_IsValidHostName(connectionData->mHost)) {
+      !net_IsValidDNSHost(connectionData->mHost)) {
     return NS_ERROR_UNKNOWN_HOST;
   }
 
@@ -1124,7 +1273,7 @@ nsresult Dashboard::TestNewConnection(ConnectionData* aConnectionData) {
   }
 
   rv = connectionData->mSocket->SetEventSink(connectionData,
-                                             GetCurrentEventTarget());
+                                             GetCurrentSerialEventTarget());
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -1147,8 +1296,7 @@ using ErrorEntry = struct {
 };
 
 #undef ERROR
-#define ERROR(key, val) \
-  { key, #key }
+#define ERROR(key, val) {key, #key}
 
 ErrorEntry socketTransportStatuses[] = {
     ERROR(NS_NET_STATUS_RESOLVING_HOST, FAILURE(3)),
