@@ -14,14 +14,14 @@
 #include "mozilla/NonBlockingAsyncInputStream.h"
 #include "mozilla/ProfilerLabels.h"
 #include "mozilla/SlicedInputStream.h"
+#include "mozilla/StaticPrefs_network.h"
 #include "nsIStreamListener.h"
 #include "nsILoadGroup.h"
 #include "nsNetCID.h"
+#include "nsNetUtil.h"
 #include "nsStreamUtils.h"
 #include <algorithm>
 
-static NS_DEFINE_CID(kStreamTransportServiceCID, NS_STREAMTRANSPORTSERVICE_CID);
-
 //
 // MOZ_LOG=nsStreamPump:5
 //
@@ -38,7 +38,7 @@ nsInputStreamPump::nsInputStreamPump() : mOffMainThread(!NS_IsMainThread()) {}
 nsresult nsInputStreamPump::Create(nsInputStreamPump** result,
                                    nsIInputStream* stream, uint32_t segsize,
                                    uint32_t segcount, bool closeWhenDone,
-                                   nsIEventTarget* mainThreadTarget) {
+                                   nsISerialEventTarget* mainThreadTarget) {
   nsresult rv = NS_ERROR_OUT_OF_MEMORY;
   RefPtr<nsInputStreamPump> pump = new nsInputStreamPump();
   if (pump) {
@@ -86,7 +86,8 @@ nsresult nsInputStreamPump::PeekStream(PeekSegmentFun callback, void* closure) {
 
   PeekData data(callback, closure);
   return mAsyncStream->ReadSegments(
-      CallPeekFunc, &data, net::nsIOService::gDefaultSegmentSize, &dummy);
+      CallPeekFunc, &data, mozilla::net::nsIOService::gDefaultSegmentSize,
+      &dummy);
 }
 
 nsresult nsInputStreamPump::EnsureWaiting() {
@@ -99,9 +100,10 @@ nsresult nsInputStreamPump::EnsureWaiting() {
     // Ensure OnStateStop is called on the main thread only when this pump is
     // created on main thread.
     if (mState == STATE_STOP && !mOffMainThread) {
-      nsCOMPtr<nsIEventTarget> mainThread =
-          mLabeledMainThreadTarget ? mLabeledMainThreadTarget
-                                   : do_AddRef(GetMainThreadEventTarget());
+      nsCOMPtr<nsISerialEventTarget> mainThread =
+          mLabeledMainThreadTarget
+              ? mLabeledMainThreadTarget
+              : do_AddRef(mozilla::GetMainThreadSerialEventTarget());
       if (mTargetThread != mainThread) {
         mTargetThread = mainThread;
       }
@@ -113,7 +115,7 @@ nsresult nsInputStreamPump::EnsureWaiting() {
       return rv;
     }
     // Any retargeting during STATE_START or START_TRANSFER is complete
-    // after the call to AsyncWait; next callback wil be on mTargetThread.
+    // after the call to AsyncWait; next callback will be on mTargetThread.
     mRetargeting = false;
     mWaitingForInputStreamReady = true;
   }
@@ -154,7 +156,7 @@ NS_IMETHODIMP
 nsInputStreamPump::IsPending(bool* result) {
   RecursiveMutexAutoLock lock(mMutex);
 
-  *result = (mState != STATE_IDLE);
+  *result = (mState != STATE_IDLE && mState != STATE_DEAD);
   return NS_OK;
 }
 
@@ -166,18 +168,25 @@ nsInputStreamPump::GetStatus(nsresult* status) {
   return NS_OK;
 }
 
+NS_IMETHODIMP nsInputStreamPump::SetCanceledReason(const nsACString& aReason) {
+  return SetCanceledReasonImpl(aReason);
+}
+
+NS_IMETHODIMP nsInputStreamPump::GetCanceledReason(nsACString& aReason) {
+  return GetCanceledReasonImpl(aReason);
+}
+
+NS_IMETHODIMP nsInputStreamPump::CancelWithReason(nsresult aStatus,
+                                                  const nsACString& aReason) {
+  return CancelWithReasonImpl(aStatus, aReason);
+}
+
 NS_IMETHODIMP
 nsInputStreamPump::Cancel(nsresult status) {
-#if DEBUG
-  if (mOffMainThread) {
-    MOZ_ASSERT_IF(mTargetThread, mTargetThread->IsOnCurrentThread());
-  } else {
-    MOZ_ASSERT(NS_IsMainThread());
-  }
-#endif
-
   RecursiveMutexAutoLock lock(mMutex);
 
+  AssertOnThread();
+
   LOG(("nsInputStreamPump::Cancel [this=%p status=%" PRIx32 "]\n", this,
        static_cast<uint32_t>(status)));
 
@@ -191,12 +200,31 @@ nsInputStreamPump::Cancel(nsresult status) {
 
   // close input stream
   if (mAsyncStream) {
-    mAsyncStream->CloseWithStatus(status);
-    if (mSuspendCount == 0) EnsureWaiting();
-    // Otherwise, EnsureWaiting will be called by Resume().
+    // If mSuspendCount != 0, EnsureWaiting will be called by Resume().
     // Note that while suspended, OnInputStreamReady will
     // not do anything, and also note that calling asyncWait
     // on a closed stream works and will dispatch an event immediately.
+
+    nsCOMPtr<nsIEventTarget> currentTarget = NS_GetCurrentThread();
+    if (mTargetThread && currentTarget != mTargetThread) {
+      nsresult rv = mTargetThread->Dispatch(NS_NewRunnableFunction(
+          "nsInputStreamPump::Cancel", [self = RefPtr{this}, status] {
+            RecursiveMutexAutoLock lock(self->mMutex);
+            if (!self->mAsyncStream) {
+              return;
+            }
+            self->mAsyncStream->CloseWithStatus(status);
+            if (self->mSuspendCount == 0) {
+              self->EnsureWaiting();
+            }
+          }));
+      NS_ENSURE_SUCCESS(rv, rv);
+    } else {
+      mAsyncStream->CloseWithStatus(status);
+      if (mSuspendCount == 0) {
+        EnsureWaiting();
+      }
+    }
   }
   return NS_OK;
 }
@@ -206,7 +234,8 @@ nsInputStreamPump::Suspend() {
   RecursiveMutexAutoLock lock(mMutex);
 
   LOG(("nsInputStreamPump::Suspend [this=%p]\n", this));
-  NS_ENSURE_TRUE(mState != STATE_IDLE, NS_ERROR_UNEXPECTED);
+  NS_ENSURE_TRUE(mState != STATE_IDLE && mState != STATE_DEAD,
+                 NS_ERROR_UNEXPECTED);
   ++mSuspendCount;
   return NS_OK;
 }
@@ -217,12 +246,15 @@ nsInputStreamPump::Resume() {
 
   LOG(("nsInputStreamPump::Resume [this=%p]\n", this));
   NS_ENSURE_TRUE(mSuspendCount > 0, NS_ERROR_UNEXPECTED);
-  NS_ENSURE_TRUE(mState != STATE_IDLE, NS_ERROR_UNEXPECTED);
+  NS_ENSURE_TRUE(mState != STATE_IDLE && mState != STATE_DEAD,
+                 NS_ERROR_UNEXPECTED);
 
   // There is a brief in-between state when we null out mAsyncStream in
   // OnStateStop() before calling OnStopRequest, and only afterwards set
-  // STATE_IDLE, which we need to handle gracefully.
-  if (--mSuspendCount == 0 && mAsyncStream) EnsureWaiting();
+  // STATE_DEAD, which we need to handle gracefully.
+  if (--mSuspendCount == 0 && mAsyncStream) {
+    EnsureWaiting();
+  }
   return NS_OK;
 }
 
@@ -256,7 +288,7 @@ NS_IMETHODIMP
 nsInputStreamPump::GetLoadGroup(nsILoadGroup** aLoadGroup) {
   RecursiveMutexAutoLock lock(mMutex);
 
-  NS_IF_ADDREF(*aLoadGroup = mLoadGroup);
+  *aLoadGroup = do_AddRef(mLoadGroup).take();
   return NS_OK;
 }
 
@@ -275,7 +307,9 @@ nsInputStreamPump::SetLoadGroup(nsILoadGroup* aLoadGroup) {
 NS_IMETHODIMP
 nsInputStreamPump::Init(nsIInputStream* stream, uint32_t segsize,
                         uint32_t segcount, bool closeWhenDone,
-                        nsIEventTarget* mainThreadTarget) {
+                        nsISerialEventTarget* mainThreadTarget) {
+  // probably we can't be multithread-accessed yet
+  RecursiveMutexAutoLock lock(mMutex);
   NS_ENSURE_TRUE(mState == STATE_IDLE, NS_ERROR_IN_PROGRESS);
 
   mStream = stream;
@@ -297,6 +331,7 @@ NS_IMETHODIMP
 nsInputStreamPump::AsyncRead(nsIStreamListener* listener) {
   RecursiveMutexAutoLock lock(mMutex);
 
+  // This ensures only one thread can interact with a pump at a time
   NS_ENSURE_TRUE(mState == STATE_IDLE, NS_ERROR_IN_PROGRESS);
   NS_ENSURE_ARG_POINTER(listener);
   MOZ_ASSERT(NS_IsMainThread() || mOffMainThread,
@@ -320,7 +355,7 @@ nsInputStreamPump::AsyncRead(nsIStreamListener* listener) {
   if (NS_IsMainThread() && mLabeledMainThreadTarget) {
     mTargetThread = mLabeledMainThreadTarget;
   } else {
-    mTargetThread = GetCurrentEventTarget();
+    mTargetThread = mozilla::GetCurrentSerialEventTarget();
   }
   NS_ENSURE_STATE(mTargetThread);
 
@@ -362,7 +397,7 @@ nsInputStreamPump::OnInputStreamReady(nsIAsyncInputStream* stream) {
       break;
     }
     mProcessingCallbacks = true;
-    if (mSuspendCount || mState == STATE_IDLE) {
+    if (mSuspendCount || mState == STATE_IDLE || mState == STATE_DEAD) {
       mWaitingForInputStreamReady = false;
       mProcessingCallbacks = false;
       break;
@@ -439,7 +474,7 @@ nsInputStreamPump::OnInputStreamReady(nsIAsyncInputStream* stream) {
   return NS_OK;
 }
 
-uint32_t nsInputStreamPump::OnStateStart() {
+uint32_t nsInputStreamPump::OnStateStart() MOZ_REQUIRES(mMutex) {
   mMutex.AssertCurrentThreadIn();
 
   AUTO_PROFILER_LABEL("nsInputStreamPump::OnStateStart", NETWORK);
@@ -458,11 +493,15 @@ uint32_t nsInputStreamPump::OnStateStart() {
   }
 
   {
+    nsCOMPtr<nsIStreamListener> listener = mListener;
+    // We're on the writing thread
+    AssertOnThread();
+
     // Note: Must exit mutex for call to OnStartRequest to avoid
     // deadlocks when calls to RetargetDeliveryTo for multiple
     // nsInputStreamPumps are needed (e.g. nsHttpChannel).
     RecursiveMutexAutoUnlock unlock(mMutex);
-    rv = mListener->OnStartRequest(this);
+    rv = listener->OnStartRequest(this);
   }
 
   // an error returned from OnStartRequest should cause us to abort; however,
@@ -472,7 +511,7 @@ uint32_t nsInputStreamPump::OnStateStart() {
   return NS_SUCCEEDED(mStatus) ? STATE_TRANSFER : STATE_STOP;
 }
 
-uint32_t nsInputStreamPump::OnStateTransfer() {
+uint32_t nsInputStreamPump::OnStateTransfer() MOZ_REQUIRES(mMutex) {
   mMutex.AssertCurrentThreadIn();
 
   AUTO_PROFILER_LABEL("nsInputStreamPump::OnStateTransfer", NETWORK);
@@ -525,12 +564,27 @@ uint32_t nsInputStreamPump::OnStateTransfer() {
          mStreamOffset, avail, odaAvail));
 
     {
+      // We may be called on non-MainThread even if mOffMainThread is
+      // false, due to RetargetDeliveryTo(), so don't use AssertOnThread()
+      if (mTargetThread) {
+        MOZ_ASSERT(mTargetThread->IsOnCurrentThread());
+      } else {
+        MOZ_ASSERT(NS_IsMainThread());
+      }
+
+      nsCOMPtr<nsIStreamListener> listener = mListener;
       // Note: Must exit mutex for call to OnStartRequest to avoid
       // deadlocks when calls to RetargetDeliveryTo for multiple
       // nsInputStreamPumps are needed (e.g. nsHttpChannel).
       RecursiveMutexAutoUnlock unlock(mMutex);
-      rv = mListener->OnDataAvailable(this, mAsyncStream, mStreamOffset,
-                                      odaAvail);
+      // We're on the writing thread for mListener and mAsyncStream.
+      // mStreamOffset is only touched in OnStateTransfer, and AsyncRead
+      // shouldn't be called during OnDataAvailable()
+
+      MOZ_PUSH_IGNORE_THREAD_SAFETY
+      rv = listener->OnDataAvailable(this, mAsyncStream, mStreamOffset,
+                                     odaAvail);
+      MOZ_POP_THREAD_SAFETY
     }
 
     // don't enter this code if ODA failed or called Cancel
@@ -591,17 +645,25 @@ nsresult nsInputStreamPump::CallOnStateStop() {
   return NS_OK;
 }
 
-uint32_t nsInputStreamPump::OnStateStop() {
+uint32_t nsInputStreamPump::OnStateStop() MOZ_REQUIRES(mMutex) {
   mMutex.AssertCurrentThreadIn();
 
   if (!NS_IsMainThread() && !mOffMainThread) {
     // This method can be called on a different thread if nsInputStreamPump
     // is used off the main-thread.
+    if (NS_SUCCEEDED(mStatus) && mListener &&
+        mozilla::StaticPrefs::network_send_OnDataFinished_nsInputStreamPump()) {
+      nsCOMPtr<nsIThreadRetargetableStreamListener> retargetableListener =
+          do_QueryInterface(mListener);
+      if (retargetableListener) {
+        retargetableListener->OnDataFinished(mStatus);
+      }
+    }
     nsresult rv = mLabeledMainThreadTarget->Dispatch(
-        NewRunnableMethod("nsInputStreamPump::CallOnStateStop", this,
-                          &nsInputStreamPump::CallOnStateStop));
-    NS_ENSURE_SUCCESS(rv, STATE_IDLE);
-    return STATE_IDLE;
+        mozilla::NewRunnableMethod("nsInputStreamPump::CallOnStateStop", this,
+                                   &nsInputStreamPump::CallOnStateStop));
+    NS_ENSURE_SUCCESS(rv, STATE_DEAD);
+    return STATE_DEAD;
   }
 
   AUTO_PROFILER_LABEL("nsInputStreamPump::OnStateStop", NETWORK);
@@ -616,7 +678,7 @@ uint32_t nsInputStreamPump::OnStateStop() {
   if (!mAsyncStream || !mListener) {
     MOZ_ASSERT(mAsyncStream, "null mAsyncStream: OnStateStop called twice?");
     MOZ_ASSERT(mListener, "null mListener: OnStateStop called twice?");
-    return STATE_IDLE;
+    return STATE_DEAD;
   }
 
   if (NS_FAILED(mStatus)) {
@@ -626,20 +688,27 @@ uint32_t nsInputStreamPump::OnStateStop() {
   }
 
   mAsyncStream = nullptr;
-  mTargetThread = nullptr;
   mIsPending = false;
   {
+    // We're on the writing thread.
+    // We believe that mStatus can't be changed on us here.
+    AssertOnThread();
+
+    nsCOMPtr<nsIStreamListener> listener = mListener;
+    nsresult status = mStatus;
     // Note: Must exit mutex for call to OnStartRequest to avoid
     // deadlocks when calls to RetargetDeliveryTo for multiple
     // nsInputStreamPumps are needed (e.g. nsHttpChannel).
     RecursiveMutexAutoUnlock unlock(mMutex);
-    mListener->OnStopRequest(this, mStatus);
+
+    listener->OnStopRequest(this, status);
   }
+  mTargetThread = nullptr;
   mListener = nullptr;
 
   if (mLoadGroup) mLoadGroup->RemoveRequest(this, nullptr, mStatus);
 
-  return STATE_IDLE;
+  return STATE_DEAD;
 }
 
 nsresult nsInputStreamPump::CreateBufferedStreamIfNeeded() {
@@ -673,7 +742,7 @@ nsresult nsInputStreamPump::CreateBufferedStreamIfNeeded() {
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
-nsInputStreamPump::RetargetDeliveryTo(nsIEventTarget* aNewTarget) {
+nsInputStreamPump::RetargetDeliveryTo(nsISerialEventTarget* aNewTarget) {
   RecursiveMutexAutoLock lock(mMutex);
 
   NS_ENSURE_ARG(aNewTarget);
@@ -717,10 +786,10 @@ nsInputStreamPump::RetargetDeliveryTo(nsIEventTarget* aNewTarget) {
 }
 
 NS_IMETHODIMP
-nsInputStreamPump::GetDeliveryTarget(nsIEventTarget** aNewTarget) {
+nsInputStreamPump::GetDeliveryTarget(nsISerialEventTarget** aNewTarget) {
   RecursiveMutexAutoLock lock(mMutex);
 
-  nsCOMPtr<nsIEventTarget> target = mTargetThread;
+  nsCOMPtr<nsISerialEventTarget> target = mTargetThread;
   target.forget(aNewTarget);
   return NS_OK;
 }
