@@ -9,7 +9,7 @@
 #include "CacheEntry.h"
 #include "nsStreamUtils.h"
 #include "nsThreadUtils.h"
-#include "mozilla/DebugOnly.h"
+#include "mozilla/IntegerPrintfMacros.h"
 #include <algorithm>
 
 namespace mozilla::net {
@@ -78,10 +78,23 @@ CacheFileOutputStream::Flush() {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+CacheFileOutputStream::StreamStatus() {
+  CacheFileAutoLock lock(mFile);
+  mFile->AssertOwnsLock();  // For thread-safety analysis
+
+  LOG(("CacheFileOutputStream::Close() [this=%p]", this));
+  if (mClosed) {
+    return NS_FAILED(mStatus) ? mStatus : NS_BASE_STREAM_CLOSED;
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 CacheFileOutputStream::Write(const char* aBuf, uint32_t aCount,
                              uint32_t* _retval) {
   CacheFileAutoLock lock(mFile);
+  mFile->AssertOwnsLock();  // For thread-safety analysis
 
   LOG(("CacheFileOutputStream::Write() [this=%p, count=%d]", this, aCount));
 
@@ -246,6 +259,7 @@ CacheFileOutputStream::AsyncWait(nsIOutputStreamCallback* aCallback,
 NS_IMETHODIMP
 CacheFileOutputStream::Seek(int32_t whence, int64_t offset) {
   CacheFileAutoLock lock(mFile);
+  mFile->AssertOwnsLock();  // For thread-safety analysis
 
   LOG(("CacheFileOutputStream::Seek() [this=%p, whence=%d, offset=%" PRId64 "]",
        this, whence, offset));
@@ -296,6 +310,7 @@ CacheFileOutputStream::SetEOF() {
 NS_IMETHODIMP
 CacheFileOutputStream::Tell(int64_t* _retval) {
   CacheFileAutoLock lock(mFile);
+  mFile->AssertOwnsLock();  // For thread-safety analysis
 
   if (mClosed) {
     LOG(("CacheFileOutputStream::Tell() - Stream is closed. [this=%p]", this));
@@ -347,6 +362,8 @@ void CacheFileOutputStream::NotifyCloseListener() {
 }
 
 void CacheFileOutputStream::ReleaseChunk() {
+  mFile->AssertOwnsLock();
+
   LOG(("CacheFileOutputStream::ReleaseChunk() [this=%p, idx=%d]", this,
        mChunk->Index()));
 
@@ -435,7 +452,7 @@ void CacheFileOutputStream::NotifyListener() {
       LOG(
           ("CacheFileOutputStream::NotifyListener() - Cannot get Cache I/O "
            "thread! Using main thread for callback."));
-      mCallbackTarget = GetMainThreadEventTarget();
+      mCallbackTarget = GetMainThreadSerialEventTarget();
     }
   }
 
