@@ -6,6 +6,7 @@
 
 #include "CacheLog.h"
 #include "CacheStorageService.h"
+#include <iterator>
 #include "CacheFileIOManager.h"
 #include "CacheObserver.h"
 #include "CacheIndex.h"
@@ -14,12 +15,12 @@
 #include "CacheEntry.h"
 #include "CacheFileUtils.h"
 
-#include "nsDeleteDir.h"
-
+#include "ErrorList.h"
 #include "nsICacheStorageVisitor.h"
 #include "nsIObserverService.h"
 #include "nsIFile.h"
 #include "nsIURI.h"
+#include "nsINetworkPredictor.h"
 #include "nsCOMPtr.h"
 #include "nsContentUtils.h"
 #include "nsNetCID.h"
@@ -29,10 +30,12 @@
 #include "mozilla/AtomicBitfields.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/DebugOnly.h"
+#include "mozilla/glean/NetwerkMetrics.h"
 #include "mozilla/Services.h"
 #include "mozilla/StoragePrincipalHelper.h"
 #include "mozilla/IntegerPrintfMacros.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/NetwerkCache2Metrics.h"
+#include "mozilla/StaticPrefs_network.h"
 
 namespace mozilla::net {
 
@@ -72,8 +75,7 @@ void CacheMemoryConsumer::DoMemoryReport(uint32_t aCurrentSize) {
   }
 }
 
-CacheStorageService::MemoryPool::MemoryPool(EType aType)
-    : mType(aType), mMemorySize(0) {}
+CacheStorageService::MemoryPool::MemoryPool(EType aType) : mType(aType) {}
 
 CacheStorageService::MemoryPool::~MemoryPool() {
   if (mMemorySize != 0) {
@@ -113,17 +115,7 @@ NS_IMPL_ISUPPORTS(CacheStorageService, nsICacheStorageService,
 
 CacheStorageService* CacheStorageService::sSelf = nullptr;
 
-CacheStorageService::CacheStorageService()
-    : mLock("CacheStorageService.mLock"),
-      mForcedValidEntriesLock("CacheStorageService.mForcedValidEntriesLock"),
-      mShutdown(false),
-      mDiskPool(MemoryPool::DISK),
-      mMemoryPool(MemoryPool::MEMORY)
-#ifdef MOZ_TSAN
-      ,
-      mPurgeTimerActive(false)
-#endif
-{
+CacheStorageService::CacheStorageService() {
   CacheFileIOManager::Init();
 
   MOZ_ASSERT(XRE_IsParentProcess());
@@ -179,10 +171,8 @@ void CacheStorageService::ShutdownBackground() {
   }
 
 #ifdef NS_FREE_PERMANENT_DATA
-  Pool(false).mFrecencyArray.Clear();
-  Pool(false).mExpirationArray.Clear();
-  Pool(true).mFrecencyArray.Clear();
-  Pool(true).mExpirationArray.Clear();
+  Pool(MemoryPool::EType::DISK).mManagedEntries.clear();
+  Pool(MemoryPool::EType::MEMORY).mManagedEntries.clear();
 #endif
 
   LOG(("CacheStorageService::ShutdownBackground - done"));
@@ -200,9 +190,7 @@ class WalkCacheRunnable : public Runnable,
   WalkCacheRunnable(nsICacheStorageVisitor* aVisitor, bool aVisitEntries)
       : Runnable("net::WalkCacheRunnable"),
         mService(CacheStorageService::Self()),
-        mCallback(aVisitor),
-        mSize(0),
-        mCancel(false) {
+        mCallback(aVisitor) {
     MOZ_ASSERT(NS_IsMainThread());
     StoreNotifyStorage(true);
     StoreVisitEntries(aVisitEntries);
@@ -217,7 +205,7 @@ class WalkCacheRunnable : public Runnable,
   RefPtr<CacheStorageService> mService;
   nsCOMPtr<nsICacheStorageVisitor> mCallback;
 
-  uint64_t mSize;
+  uint64_t mSize{0};
 
   // clang-format off
   MOZ_ATOMIC_BITFIELDS(mAtomicBitfields, 8, (
@@ -226,7 +214,7 @@ class WalkCacheRunnable : public Runnable,
   ))
   // clang-format on
 
-  Atomic<bool> mCancel;
+  Atomic<bool> mCancel{false};
 };
 
 // WalkMemoryCacheRunnable
@@ -253,6 +241,17 @@ class WalkMemoryCacheRunnable : public WalkCacheRunnable {
 
       if (!CacheStorageService::IsRunning()) return NS_ERROR_NOT_INITIALIZED;
 
+      // Count the entries to allocate the array memory all at once.
+      size_t numEntries = 0;
+      for (const auto& entries : sGlobalEntryTables->Values()) {
+        if (entries->Type() != CacheEntryTable::MEMORY_ONLY) {
+          continue;
+        }
+        numEntries += entries->Values().Count();
+      }
+      mEntryArray.SetCapacity(numEntries);
+
+      // Collect the entries.
       for (const auto& entries : sGlobalEntryTables->Values()) {
         if (entries->Type() != CacheEntryTable::MEMORY_ONLY) {
           continue;
@@ -292,15 +291,14 @@ class WalkMemoryCacheRunnable : public WalkCacheRunnable {
         LOG(("  entry [left=%zu, canceled=%d]", mEntryArray.Length(),
              (bool)mCancel));
 
-        // Third, notify each entry until depleted or canceled
-        if (!mEntryArray.Length() || mCancel) {
+        // Third, notify each entry until depleted or canceled.
+        if (mNextEntryIdx >= mEntryArray.Length() || mCancel) {
           mCallback->OnCacheEntryVisitCompleted();
           return NS_OK;  // done
         }
 
-        // Grab the next entry
-        RefPtr<CacheEntry> entry = mEntryArray[0];
-        mEntryArray.RemoveElementAt(0);
+        // Grab the next entry.
+        RefPtr<CacheEntry> entry = std::move(mEntryArray[mNextEntryIdx++]);
 
         // Invokes this->OnEntryInfo, that calls the callback with all
         // information of the entry.
@@ -323,9 +321,9 @@ class WalkMemoryCacheRunnable : public WalkCacheRunnable {
 
   virtual void OnEntryInfo(const nsACString& aURISpec,
                            const nsACString& aIdEnhance, int64_t aDataSize,
-                           int32_t aFetchCount, uint32_t aLastModifiedTime,
-                           uint32_t aExpirationTime, bool aPinned,
-                           nsILoadContextInfo* aInfo) override {
+                           int64_t aAltDataSize, uint32_t aFetchCount,
+                           uint32_t aLastModifiedTime, uint32_t aExpirationTime,
+                           bool aPinned, nsILoadContextInfo* aInfo) override {
     nsresult rv;
 
     nsCOMPtr<nsIURI> uri;
@@ -334,9 +332,9 @@ class WalkMemoryCacheRunnable : public WalkCacheRunnable {
       return;
     }
 
-    rv = mCallback->OnCacheEntryInfo(uri, aIdEnhance, aDataSize, aFetchCount,
-                                     aLastModifiedTime, aExpirationTime,
-                                     aPinned, aInfo);
+    rv = mCallback->OnCacheEntryInfo(uri, aIdEnhance, aDataSize, aAltDataSize,
+                                     aFetchCount, aLastModifiedTime,
+                                     aExpirationTime, aPinned, aInfo);
     if (NS_FAILED(rv)) {
       LOG(("  callback failed, canceling the walk"));
       mCancel = true;
@@ -346,6 +344,7 @@ class WalkMemoryCacheRunnable : public WalkCacheRunnable {
  private:
   nsCString mContextKey;
   nsTArray<RefPtr<CacheEntry>> mEntryArray;
+  size_t mNextEntryIdx{0};
 };
 
 // WalkDiskCacheRunnable
@@ -381,12 +380,7 @@ class WalkDiskCacheRunnable : public WalkCacheRunnable {
    public:
     explicit OnCacheEntryInfoRunnable(WalkDiskCacheRunnable* aWalker)
         : Runnable("net::WalkDiskCacheRunnable::OnCacheEntryInfoRunnable"),
-          mWalker(aWalker),
-          mDataSize(0),
-          mFetchCount(0),
-          mLastModifiedTime(0),
-          mExpirationTime(0),
-          mPinned(false) {}
+          mWalker(aWalker) {}
 
     NS_IMETHOD Run() override {
       MOZ_ASSERT(NS_IsMainThread());
@@ -400,8 +394,8 @@ class WalkDiskCacheRunnable : public WalkCacheRunnable {
       }
 
       rv = mWalker->mCallback->OnCacheEntryInfo(
-          uri, mIdEnhance, mDataSize, mFetchCount, mLastModifiedTime,
-          mExpirationTime, mPinned, mInfo);
+          uri, mIdEnhance, mDataSize, mAltDataSize, mFetchCount,
+          mLastModifiedTime, mExpirationTime, mPinned, mInfo);
       if (NS_FAILED(rv)) {
         mWalker->mCancel = true;
       }
@@ -413,11 +407,12 @@ class WalkDiskCacheRunnable : public WalkCacheRunnable {
 
     nsCString mURISpec;
     nsCString mIdEnhance;
-    int64_t mDataSize;
-    int32_t mFetchCount;
-    uint32_t mLastModifiedTime;
-    uint32_t mExpirationTime;
-    bool mPinned;
+    int64_t mDataSize{0};
+    int64_t mAltDataSize{0};
+    uint32_t mFetchCount{0};
+    uint32_t mLastModifiedTime{0};
+    uint32_t mExpirationTime{0};
+    bool mPinned{false};
     nsCOMPtr<nsILoadContextInfo> mInfo;
   };
 
@@ -498,9 +493,9 @@ class WalkDiskCacheRunnable : public WalkCacheRunnable {
 
   virtual void OnEntryInfo(const nsACString& aURISpec,
                            const nsACString& aIdEnhance, int64_t aDataSize,
-                           int32_t aFetchCount, uint32_t aLastModifiedTime,
-                           uint32_t aExpirationTime, bool aPinned,
-                           nsILoadContextInfo* aInfo) override {
+                           int64_t aAltDataSize, uint32_t aFetchCount,
+                           uint32_t aLastModifiedTime, uint32_t aExpirationTime,
+                           bool aPinned, nsILoadContextInfo* aInfo) override {
     // Called directly from CacheFileIOManager::GetEntryInfo.
 
     // Invoke onCacheEntryInfo on the main thread for this entry.
@@ -508,6 +503,7 @@ class WalkDiskCacheRunnable : public WalkCacheRunnable {
     info->mURISpec = aURISpec;
     info->mIdEnhance = aIdEnhance;
     info->mDataSize = aDataSize;
+    info->mAltDataSize = aAltDataSize;
     info->mFetchCount = aFetchCount;
     info->mLastModifiedTime = aLastModifiedTime;
     info->mExpirationTime = aExpirationTime;
@@ -552,72 +548,6 @@ void CacheStorageService::DropPrivateBrowsingEntries() {
   }
 }
 
-namespace {
-
-class CleaupCacheDirectoriesRunnable : public Runnable {
- public:
-  NS_DECL_NSIRUNNABLE
-  static bool Post();
-
- private:
-  CleaupCacheDirectoriesRunnable()
-      : Runnable("net::CleaupCacheDirectoriesRunnable") {
-    CacheFileIOManager::GetCacheDirectory(getter_AddRefs(mCache2Dir));
-#if defined(MOZ_WIDGET_ANDROID)
-    CacheFileIOManager::GetProfilelessCacheDirectory(
-        getter_AddRefs(mCache2Profileless));
-#endif
-  }
-
-  virtual ~CleaupCacheDirectoriesRunnable() = default;
-  nsCOMPtr<nsIFile> mCache1Dir, mCache2Dir;
-#if defined(MOZ_WIDGET_ANDROID)
-  nsCOMPtr<nsIFile> mCache2Profileless;
-#endif
-};
-
-// static
-bool CleaupCacheDirectoriesRunnable::Post() {
-  // TODO: initialize nsDeleteDir
-  return true;
-}
-
-NS_IMETHODIMP CleaupCacheDirectoriesRunnable::Run() {
-  MOZ_ASSERT(!NS_IsMainThread());
-
-  if (mCache1Dir) {
-    nsDeleteDir::RemoveOldTrashes(mCache1Dir);
-  }
-  if (mCache2Dir) {
-    nsDeleteDir::RemoveOldTrashes(mCache2Dir);
-  }
-#if defined(MOZ_WIDGET_ANDROID)
-  if (mCache2Profileless) {
-    nsDeleteDir::RemoveOldTrashes(mCache2Profileless);
-    // Always delete the profileless cache on Android
-    nsDeleteDir::DeleteDir(mCache2Profileless, true, 30000);
-  }
-#endif
-
-  if (mCache1Dir) {
-    nsDeleteDir::DeleteDir(mCache1Dir, true, 30000);
-  }
-
-  return NS_OK;
-}
-
-}  // namespace
-
-// static
-void CacheStorageService::CleaupCacheDirectories() {
-  // Make sure we schedule just once in case CleaupCacheDirectories gets called
-  // multiple times from some reason.
-  static bool runOnce = CleaupCacheDirectoriesRunnable::Post();
-  if (!runOnce) {
-    NS_WARNING("Could not start cache trashes cleanup");
-  }
-}
-
 // Helper methods
 
 // static
@@ -735,19 +665,6 @@ NS_IMETHODIMP CacheStorageService::PinningCacheStorage(
   return NS_OK;
 }
 
-NS_IMETHODIMP CacheStorageService::SynthesizedCacheStorage(
-    nsILoadContextInfo* aLoadContextInfo, nsICacheStorage** _retval) {
-  NS_ENSURE_ARG(aLoadContextInfo);
-  NS_ENSURE_ARG(_retval);
-
-  nsCOMPtr<nsICacheStorage> storage =
-      new CacheStorage(aLoadContextInfo, false,
-                       true /* skip size checks for synthesized cache */,
-                       false /* no pinning */);
-  storage.forget(_retval);
-  return NS_OK;
-}
-
 NS_IMETHODIMP CacheStorageService::Clear() {
   nsresult rv;
 
@@ -778,7 +695,8 @@ NS_IMETHODIMP CacheStorageService::Clear() {
   return NS_OK;
 }
 
-NS_IMETHODIMP CacheStorageService::ClearOrigin(nsIPrincipal* aPrincipal) {
+NS_IMETHODIMP CacheStorageService::ClearOriginsByPrincipal(
+    nsIPrincipal* aPrincipal) {
   nsresult rv;
 
   if (NS_WARN_IF(!aPrincipal)) {
@@ -786,7 +704,7 @@ NS_IMETHODIMP CacheStorageService::ClearOrigin(nsIPrincipal* aPrincipal) {
   }
 
   nsAutoString origin;
-  rv = nsContentUtils::GetUTFOrigin(aPrincipal, origin);
+  rv = nsContentUtils::GetWebExposedOriginSerialization(aPrincipal, origin);
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = ClearOriginInternal(origin, aPrincipal->OriginAttributesRef(), true);
@@ -798,7 +716,7 @@ NS_IMETHODIMP CacheStorageService::ClearOrigin(nsIPrincipal* aPrincipal) {
   return NS_OK;
 }
 
-NS_IMETHODIMP CacheStorageService::ClearOriginAttributes(
+NS_IMETHODIMP CacheStorageService::ClearOriginsByOriginAttributes(
     const nsAString& aOriginAttributes) {
   nsresult rv;
 
@@ -884,7 +802,7 @@ NS_IMETHODIMP CacheStorageService::ClearBaseDomain(
         }
 
         bool hasRootDomain = false;
-        rv = NS_HasRootDomain(host, cBaseDomain, &hasRootDomain);
+        rv = HasRootDomain(host, cBaseDomain, &hasRootDomain);
         if (NS_WARN_IF(NS_FAILED(rv))) {
           continue;
         }
@@ -950,7 +868,7 @@ nsresult CacheStorageService::ClearOriginInternal(
         NS_ENSURE_SUCCESS(rv, rv);
 
         nsAutoString origin;
-        rv = nsContentUtils::GetUTFOrigin(uri, origin);
+        rv = nsContentUtils::GetWebExposedOriginSerialization(uri, origin);
         NS_ENSURE_SUCCESS(rv, rv);
 
         if (origin != aOrigin) {
@@ -967,13 +885,15 @@ nsresult CacheStorageService::ClearOriginInternal(
           NS_ERROR("aEntry->HashingKey() failed?");
           return rv;
         }
-
+        MOZ_ASSERT_IF(info->IsPrivate(), !entry->IsUsingDisk());
         RemoveExactEntry(table, entryKey, entry, false /* don't overwrite */);
       }
     }
   }
 
-  rv = CacheFileIOManager::EvictByContext(info, false /* pinned */, aOrigin);
+  if (!info->IsPrivate()) {
+    rv = CacheFileIOManager::EvictByContext(info, false /* pinned */, aOrigin);
+  }
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1017,9 +937,13 @@ NS_IMETHODIMP CacheStorageService::PurgeFromMemoryRunnable::Run() {
   }
 
   if (mService) {
-    // TODO not all flags apply to both pools
-    mService->Pool(true).PurgeAll(mWhat);
-    mService->Pool(false).PurgeAll(mWhat);
+    // Note that we seem to come here only in the case of "memory-pressure"
+    // being notified (or in case of tests), so we start from purging in-memory
+    // entries first and ignore minprogress for disk entries.
+    // TODO not all flags apply to both pools.
+    mService->Pool(MemoryPool::EType::MEMORY)
+        .PurgeAll(mWhat, StaticPrefs::network_cache_purge_minprogress_memory());
+    mService->Pool(MemoryPool::EType::DISK).PurgeAll(mWhat, 0);
     mService = nullptr;
   }
 
@@ -1061,40 +985,6 @@ NS_IMETHODIMP CacheStorageService::AsyncVisitAllStorages(
 
 // Methods used by CacheEntry for management of in-memory structures.
 
-namespace {
-
-class FrecencyComparator {
- public:
-  bool Equals(CacheEntry* a, CacheEntry* b) const {
-    return a->GetFrecency() == b->GetFrecency();
-  }
-  bool LessThan(CacheEntry* a, CacheEntry* b) const {
-    // We deliberately want to keep the '0' frecency entries at the tail of the
-    // aray, because these are new entries and would just slow down purging of
-    // the pools based on frecency.
-    if (a->GetFrecency() == 0.0 && b->GetFrecency() > 0.0) {
-      return false;
-    }
-    if (a->GetFrecency() > 0.0 && b->GetFrecency() == 0.0) {
-      return true;
-    }
-
-    return a->GetFrecency() < b->GetFrecency();
-  }
-};
-
-class ExpirationComparator {
- public:
-  bool Equals(CacheEntry* a, CacheEntry* b) const {
-    return a->GetExpirationTime() == b->GetExpirationTime();
-  }
-  bool LessThan(CacheEntry* a, CacheEntry* b) const {
-    return a->GetExpirationTime() < b->GetExpirationTime();
-  }
-};
-
-}  // namespace
-
 void CacheStorageService::RegisterEntry(CacheEntry* aEntry) {
   MOZ_ASSERT(IsOnManagementThread());
 
@@ -1105,8 +995,7 @@ void CacheStorageService::RegisterEntry(CacheEntry* aEntry) {
   LOG(("CacheStorageService::RegisterEntry [entry=%p]", aEntry));
 
   MemoryPool& pool = Pool(aEntry->IsUsingDisk());
-  pool.mFrecencyArray.AppendElement(aEntry);
-  pool.mExpirationArray.AppendElement(aEntry);
+  pool.mManagedEntries.insertBack(aEntry);
 
   aEntry->SetRegistered(true);
 }
@@ -1121,12 +1010,7 @@ void CacheStorageService::UnregisterEntry(CacheEntry* aEntry) {
   LOG(("CacheStorageService::UnregisterEntry [entry=%p]", aEntry));
 
   MemoryPool& pool = Pool(aEntry->IsUsingDisk());
-  mozilla::DebugOnly<bool> removedFrecency =
-      pool.mFrecencyArray.RemoveElement(aEntry);
-  mozilla::DebugOnly<bool> removedExpiration =
-      pool.mExpirationArray.RemoveElement(aEntry);
-
-  MOZ_ASSERT(mShutdown || (removedFrecency && removedExpiration));
+  aEntry->removeFrom(pool.mManagedEntries);
 
   // Note: aEntry->CanRegister() since now returns false
   aEntry->SetRegistered(false);
@@ -1257,26 +1141,47 @@ bool CacheStorageService::IsForcedValidEntry(
     nsACString const& aContextEntryKey) {
   mozilla::MutexAutoLock lock(mForcedValidEntriesLock);
 
-  TimeStamp validUntil;
+  ForcedValidData data;
 
-  if (!mForcedValidEntries.Get(aContextEntryKey, &validUntil)) {
+  if (!mForcedValidEntries.Get(aContextEntryKey, &data)) {
     return false;
   }
 
-  if (validUntil.IsNull()) {
+  if (data.validUntil.IsNull()) {
+    MOZ_ASSERT_UNREACHABLE("the timeStamp should never be null");
     return false;
   }
 
   // Entry timeout not reached yet
-  if (TimeStamp::NowLoRes() <= validUntil) {
+  if (TimeStamp::NowLoRes() <= data.validUntil) {
     return true;
   }
 
   // Entry timeout has been reached
   mForcedValidEntries.Remove(aContextEntryKey);
+
+  if (!data.viewed) {
+    glean::predictor::prefetch_use_status
+        .EnumGet(glean::predictor::PrefetchUseStatusLabel::eWaitedtoolong)
+        .Add();
+  }
   return false;
 }
 
+void CacheStorageService::MarkForcedValidEntryUse(nsACString const& aContextKey,
+                                                  nsACString const& aEntryKey) {
+  mozilla::MutexAutoLock lock(mForcedValidEntriesLock);
+
+  ForcedValidData data;
+
+  if (!mForcedValidEntries.Get(aContextKey + aEntryKey, &data)) {
+    return;
+  }
+
+  data.viewed = true;
+  mForcedValidEntries.InsertOrUpdate(aContextKey + aEntryKey, data);
+}
+
 // Allows a cache entry to be loaded directly from cache without further
 // validation - see nsICacheEntry.idl for further details
 void CacheStorageService::ForceEntryValidFor(nsACString const& aContextKey,
@@ -1287,10 +1192,11 @@ void CacheStorageService::ForceEntryValidFor(nsACString const& aContextKey,
   TimeStamp now = TimeStamp::NowLoRes();
   ForcedValidEntriesPrune(now);
 
-  // This will be the timeout
-  TimeStamp validUntil = now + TimeDuration::FromSeconds(aSecondsToTheFuture);
+  ForcedValidData data;
+  data.validUntil = now + TimeDuration::FromSeconds(aSecondsToTheFuture);
+  data.viewed = false;
 
-  mForcedValidEntries.InsertOrUpdate(aContextKey + aEntryKey, validUntil);
+  mForcedValidEntries.InsertOrUpdate(aContextKey + aEntryKey, data);
 }
 
 void CacheStorageService::RemoveEntryForceValid(nsACString const& aContextKey,
@@ -1299,6 +1205,13 @@ void CacheStorageService::RemoveEntryForceValid(nsACString const& aContextKey,
 
   LOG(("CacheStorageService::RemoveEntryForceValid context='%s' entryKey=%s",
        aContextKey.BeginReading(), aEntryKey.BeginReading()));
+  ForcedValidData data;
+  bool ok = mForcedValidEntries.Get(aContextKey + aEntryKey, &data);
+  if (ok && !data.viewed) {
+    glean::predictor::prefetch_use_status
+        .EnumGet(glean::predictor::PrefetchUseStatusLabel::eWaitedtoolong)
+        .Add();
+  }
   mForcedValidEntries.Remove(aContextKey + aEntryKey);
 }
 
@@ -1309,7 +1222,12 @@ void CacheStorageService::ForcedValidEntriesPrune(TimeStamp& now) {
   if (now < dontPruneUntil) return;
 
   for (auto iter = mForcedValidEntries.Iter(); !iter.Done(); iter.Next()) {
-    if (iter.Data() < now) {
+    if (iter.Data().validUntil < now) {
+      if (!iter.Data().viewed) {
+        glean::predictor::prefetch_use_status
+            .EnumGet(glean::predictor::PrefetchUseStatusLabel::eWaitedtoolong)
+            .Add();
+      }
       iter.Remove();
     }
   }
@@ -1333,8 +1251,8 @@ void CacheStorageService::OnMemoryConsumptionChange(
 
   if (!overLimit) return;
 
-    // It's likely the timer has already been set when we get here,
-    // check outside the lock to save resources.
+  // It's likely the timer has already been set when we get here,
+  // check outside the lock to save resources.
 #ifdef MOZ_TSAN
   if (mPurgeTimerActive) {
 #else
@@ -1408,10 +1326,12 @@ CacheStorageService::Notify(nsITimer* aTimer) {
 #endif
     mPurgeTimer = nullptr;
 
-    nsCOMPtr<nsIRunnable> event =
-        NewRunnableMethod("net::CacheStorageService::PurgeOverMemoryLimit",
-                          this, &CacheStorageService::PurgeOverMemoryLimit);
-    Dispatch(event);
+    if (!mShutdown) {
+      nsCOMPtr<nsIRunnable> event = NewRunnableMethod(
+          "net::CacheStorageService::PurgeExpiredOrOverMemoryLimit", this,
+          &CacheStorageService::PurgeExpiredOrOverMemoryLimit);
+      Dispatch(event);
+    }
   }
 
   return NS_OK;
@@ -1423,10 +1343,12 @@ CacheStorageService::GetName(nsACString& aName) {
   return NS_OK;
 }
 
-void CacheStorageService::PurgeOverMemoryLimit() {
+void CacheStorageService::PurgeExpiredOrOverMemoryLimit() {
   MOZ_ASSERT(IsOnManagementThread());
 
-  LOG(("CacheStorageService::PurgeOverMemoryLimit"));
+  LOG(("CacheStorageService::PurgeExpiredOrOverMemoryLimit"));
+
+  if (mShutdown) return;
 
   static TimeDuration const kFourSeconds = TimeDuration::FromSeconds(4);
   TimeStamp now = TimeStamp::NowLoRes();
@@ -1438,127 +1360,200 @@ void CacheStorageService::PurgeOverMemoryLimit() {
 
   mLastPurgeTime = now;
 
-  Pool(true).PurgeOverMemoryLimit();
-  Pool(false).PurgeOverMemoryLimit();
+  // We start purging memory entries first as we care more about RAM over
+  // disk space beeing freed in case we are interrupted.
+  Pool(MemoryPool::EType::MEMORY).PurgeExpiredOrOverMemoryLimit();
+  Pool(MemoryPool::EType::DISK).PurgeExpiredOrOverMemoryLimit();
 }
 
-void CacheStorageService::MemoryPool::PurgeOverMemoryLimit() {
+void CacheStorageService::MemoryPool::PurgeExpiredOrOverMemoryLimit() {
   TimeStamp start(TimeStamp::Now());
 
   uint32_t const memoryLimit = Limit();
-  if (mMemorySize > memoryLimit) {
-    LOG(("  memory data consumption over the limit, abandon expired entries"));
-    PurgeExpired();
-  }
+  size_t minprogress =
+      (mType == EType::DISK)
+          ? StaticPrefs::network_cache_purge_minprogress_disk()
+          : StaticPrefs::network_cache_purge_minprogress_memory();
 
-  // No longer makes sense since:
-  // Memory entries are never purged partially, only as a whole when the memory
-  // cache limit is overreached.
-  // Disk entries throw the data away ASAP so that only metadata are kept.
-  // TODO when this concept of two separate pools is found working, the code
-  // should clean up.
-#if 0
-  if (mMemorySize > memoryLimit) {
-    LOG(("  memory data consumption over the limit, abandon disk backed data"));
-    PurgeByFrecency(CacheEntry::PURGE_DATA_ONLY_DISK_BACKED);
+  // We always purge expired entries, even if under our limit.
+  size_t numExpired = PurgeExpired(minprogress);
+  if (numExpired > 0) {
+    LOG(("  found and purged %zu expired entries", numExpired));
   }
+  minprogress = (minprogress > numExpired) ? minprogress - numExpired : 0;
 
+  // If we are still under pressure, purge LFU entries until we aren't.
   if (mMemorySize > memoryLimit) {
-    LOG(("  metadata consumtion over the limit, abandon disk backed entries"));
-    PurgeByFrecency(CacheEntry::PURGE_WHOLE_ONLY_DISK_BACKED);
-  }
-#endif
+    // Do not enter PurgeByFrecency if we reached the minimum and are asked to
+    // deliver entries.
+    if (minprogress == 0 && CacheIOThread::YieldAndRerun()) {
+      return;
+    }
 
-  if (mMemorySize > memoryLimit) {
-    LOG(("  memory data consumption over the limit, abandon any entry"));
-    PurgeByFrecency(CacheEntry::PURGE_WHOLE);
+    if (mType == EType::DISK) {
+      mozilla::glean::networking::cache_purge_due_to_memory_limit
+          .Get("meta_data_file_size_limit"_ns)
+          .Add(1);
+    } else if (mType == EType::MEMORY) {
+      mozilla::glean::networking::cache_purge_due_to_memory_limit
+          .Get("cache_memory_limit"_ns)
+          .Add(1);
+    }
+
+    auto r = PurgeByFrecency(minprogress);
+    if (MOZ_LIKELY(r.isOk())) {
+      size_t numPurged = r.unwrap();
+      LOG((
+          "  memory data consumption over the limit, abandoned %zu LFU entries",
+          numPurged));
+    } else {
+      // If we hit an error (OOM), do an emergency PurgeAll.
+      size_t numPurged = PurgeAll(CacheEntry::PURGE_WHOLE, minprogress);
+      LOG(
+          ("  memory data consumption over the limit, emergency purged all %zu "
+           "entries",
+           numPurged));
+    }
   }
 
   LOG(("  purging took %1.2fms", (TimeStamp::Now() - start).ToMilliseconds()));
 }
 
-void CacheStorageService::MemoryPool::PurgeExpired() {
+// This function purges ALL expired entries.
+size_t CacheStorageService::MemoryPool::PurgeExpired(size_t minprogress) {
   MOZ_ASSERT(IsOnManagementThread());
 
-  mExpirationArray.Sort(ExpirationComparator());
   uint32_t now = NowInSeconds();
 
-  uint32_t const memoryLimit = Limit();
+  size_t numPurged = 0;
+  // Scan for items to purge. mManagedEntries is not sorted but comparing just
+  // one integer should be faster than anything else, so go scan.
+  RefPtr<CacheEntry> entry = mManagedEntries.getFirst();
+  while (entry) {
+    // Get the next entry before we may be removed from our list.
+    RefPtr<CacheEntry> nextEntry = entry->getNext();
 
-  for (uint32_t i = 0;
-       mMemorySize > memoryLimit && i < mExpirationArray.Length();) {
-    if (CacheIOThread::YieldAndRerun()) return;
-
-    RefPtr<CacheEntry> entry = mExpirationArray[i];
-
-    uint32_t expirationTime = entry->GetExpirationTime();
-    if (expirationTime > 0 && expirationTime <= now &&
-        entry->Purge(CacheEntry::PURGE_WHOLE)) {
-      LOG(("  purged expired, entry=%p, exptime=%u (now=%u)", entry.get(),
-           entry->GetExpirationTime(), now));
-      continue;
+    if (entry->GetExpirationTime() <= now) {
+      // Purge will modify our mManagedEntries list but we are prepared for it.
+      if (entry->Purge(CacheEntry::PURGE_WHOLE)) {
+        numPurged++;
+        LOG(("  purged expired, entry=%p, exptime=%u (now=%u)", entry.get(),
+             entry->GetExpirationTime(), now));
+      }
     }
 
-    // not purged, move to the next one
-    ++i;
+    entry = std::move(nextEntry);
+
+    // To have some progress even under load, we do the check only after
+    // purging at least minprogress items if under pressure.
+    if ((numPurged >= minprogress || mMemorySize <= Limit()) &&
+        CacheIOThread::YieldAndRerun()) {
+      break;
+    }
   }
+
+  return numPurged;
 }
 
-void CacheStorageService::MemoryPool::PurgeByFrecency(uint32_t aWhat) {
+Result<size_t, nsresult> CacheStorageService::MemoryPool::PurgeByFrecency(
+    size_t minprogress) {
   MOZ_ASSERT(IsOnManagementThread());
 
   // Pretend the limit is 10% lower so that we get rid of more entries at one
   // shot and save the sorting below.
-  uint32_t const memoryLimit = Limit() * 0.9;
+  uint32_t const memoryLimit = (uint32_t)(Limit() * 0.9);
+  if (mMemorySize <= memoryLimit) {
+    return 0;
+  }
 
-  // Let's do our best and try to shorten the array to at least this size so
-  // that it doesn't overgrow.  We will ignore higher priority events and keep
-  // looping to try to purge while the array is larget than this size.
-  static size_t const kFrecencyArrayLengthLimit = 2000;
+  LOG(("MemoryPool::PurgeByFrecency, len=%zu", mManagedEntries.length()));
 
-  LOG(("MemoryPool::PurgeByFrecency, len=%zu", mFrecencyArray.Length()));
+  // We want to have an array snapshot for sorting and iterating.
+  struct mayPurgeEntry {
+    RefPtr<CacheEntry> mEntry;
+    double mFrecency;
 
-  mFrecencyArray.Sort(FrecencyComparator());
+    explicit mayPurgeEntry(CacheEntry* aEntry) {
+      mEntry = aEntry;
+      mFrecency = aEntry->GetFrecency();
+    }
 
-  for (uint32_t i = 0;
-       mMemorySize > memoryLimit && i < mFrecencyArray.Length();) {
-    if (mFrecencyArray.Length() <= kFrecencyArrayLengthLimit &&
-        CacheIOThread::YieldAndRerun()) {
+    bool operator<(const mayPurgeEntry& aOther) const {
+      return mFrecency < aOther.mFrecency;
+    }
+  };
+
+  nsTArray<mayPurgeEntry> mayPurgeSorted;
+  if (!mayPurgeSorted.SetCapacity(mManagedEntries.length(),
+                                  mozilla::fallible)) {
+    return Err(NS_ERROR_OUT_OF_MEMORY);
+  }
+  {
+    mozilla::MutexAutoLock lock(CacheStorageService::Self()->Lock());
+
+    for (const auto& entry : mManagedEntries) {
+      // Referenced items cannot be purged and we deliberately want to not look
+      // at '0' frecency entries, these are new entries and can be ignored.
+      if (!entry->IsReferenced() && entry->GetFrecency() > 0.0) {
+        mayPurgeEntry copy(entry);
+        mayPurgeSorted.AppendElement(std::move(copy));
+      }
+    }
+  }
+  if (mayPurgeSorted.Length() == 0) {
+    return 0;
+  }
+  mayPurgeSorted.Sort();
+
+  size_t numPurged = 0;
+
+  for (auto& checkPurge : mayPurgeSorted) {
+    if (mMemorySize <= memoryLimit) {
+      break;
+    }
+
+    RefPtr<CacheEntry> entry = checkPurge.mEntry;
+
+    if (entry->Purge(CacheEntry::PURGE_WHOLE)) {
+      numPurged++;
+      LOG(("  abandoned (%d), entry=%p, frecency=%1.10f",
+           CacheEntry::PURGE_WHOLE, entry.get(), entry->GetFrecency()));
+    }
+
+    if (numPurged >= minprogress && CacheIOThread::YieldAndRerun()) {
       LOG(("MemoryPool::PurgeByFrecency interrupted"));
-      return;
+      return numPurged;
     }
-
-    RefPtr<CacheEntry> entry = mFrecencyArray[i];
-    if (entry->Purge(aWhat)) {
-      LOG(("  abandoned (%d), entry=%p, frecency=%1.10f", aWhat, entry.get(),
-           entry->GetFrecency()));
-      continue;
-    }
-
-    // not purged, move to the next one
-    ++i;
   }
 
   LOG(("MemoryPool::PurgeByFrecency done"));
+
+  return numPurged;
 }
 
-void CacheStorageService::MemoryPool::PurgeAll(uint32_t aWhat) {
+size_t CacheStorageService::MemoryPool::PurgeAll(uint32_t aWhat,
+                                                 size_t minprogress) {
   LOG(("CacheStorageService::MemoryPool::PurgeAll aWhat=%d", aWhat));
   MOZ_ASSERT(IsOnManagementThread());
 
-  for (uint32_t i = 0; i < mFrecencyArray.Length();) {
-    if (CacheIOThread::YieldAndRerun()) return;
+  size_t numPurged = 0;
 
-    RefPtr<CacheEntry> entry = mFrecencyArray[i];
+  RefPtr<CacheEntry> entry = mManagedEntries.getFirst();
+  while (entry) {
+    if (numPurged >= minprogress && CacheIOThread::YieldAndRerun()) break;
+
+    // Get the next entry before we may be removed from our list.
+    RefPtr<CacheEntry> nextEntry = entry->getNext();
 
     if (entry->Purge(aWhat)) {
+      numPurged++;
       LOG(("  abandoned entry=%p", entry.get()));
-      continue;
     }
 
-    // not purged, move to the next one
-    ++i;
+    entry = std::move(nextEntry);
   }
+
+  return numPurged;
 }
 
 // Methods exposed to and used by CacheStorage.
@@ -1566,7 +1561,7 @@ void CacheStorageService::MemoryPool::PurgeAll(uint32_t aWhat) {
 nsresult CacheStorageService::AddStorageEntry(CacheStorage const* aStorage,
                                               const nsACString& aURI,
                                               const nsACString& aIdExtension,
-                                              bool aReplace,
+                                              uint32_t aFlags,
                                               CacheEntryHandle** aResult) {
   NS_ENSURE_FALSE(mShutdown, NS_ERROR_NOT_INITIALIZED);
 
@@ -1577,13 +1572,13 @@ nsresult CacheStorageService::AddStorageEntry(CacheStorage const* aStorage,
 
   return AddStorageEntry(contextKey, aURI, aIdExtension,
                          aStorage->WriteToDisk(), aStorage->SkipSizeCheck(),
-                         aStorage->Pinning(), aReplace, aResult);
+                         aStorage->Pinning(), aFlags, aResult);
 }
 
 nsresult CacheStorageService::AddStorageEntry(
     const nsACString& aContextKey, const nsACString& aURI,
     const nsACString& aIdExtension, bool aWriteToDisk, bool aSkipSizeCheck,
-    bool aPin, bool aReplace, CacheEntryHandle** aResult) {
+    bool aPin, uint32_t aFlags, CacheEntryHandle** aResult) {
   nsresult rv;
 
   nsAutoCString entryKey;
@@ -1615,17 +1610,24 @@ nsresult CacheStorageService::AddStorageEntry(
             .get();
 
     bool entryExists = entries->Get(entryKey, getter_AddRefs(entry));
+    if (!entryExists && (aFlags & nsICacheStorage::OPEN_READONLY) &&
+        (aFlags & nsICacheStorage::OPEN_SECRETLY) &&
+        StaticPrefs::network_cache_bug1708673()) {
+      return NS_ERROR_CACHE_KEY_NOT_FOUND;
+    }
 
-    if (entryExists && !aReplace) {
+    bool replace = aFlags & nsICacheStorage::OPEN_TRUNCATE;
+
+    if (entryExists && !replace) {
       // check whether we want to turn this entry to a memory-only.
       if (MOZ_UNLIKELY(!aWriteToDisk) && MOZ_LIKELY(entry->IsUsingDisk())) {
         LOG(("  entry is persistent but we want mem-only, replacing it"));
-        aReplace = true;
+        replace = true;
       }
     }
 
     // If truncate is demanded, delete and doom the current entry
-    if (entryExists && aReplace) {
+    if (entryExists && replace) {
       entries->Remove(entryKey);
 
       LOG(("  dooming entry %p for %s because of OPEN_TRUNCATE", entry.get(),
@@ -1640,14 +1642,14 @@ nsresult CacheStorageService::AddStorageEntry(
 
       // Would only lead to deleting force-valid timestamp again.  We don't need
       // the replace information anymore after this point anyway.
-      aReplace = false;
+      replace = false;
     }
 
     // Ensure entry for the particular URL
     if (!entryExists) {
       // When replacing with a new entry, always remove the current force-valid
       // timestamp, this is the only place to do it.
-      if (aReplace) {
+      if (replace) {
         RemoveEntryForceValid(aContextKey, entryKey);
       }
 
@@ -2143,7 +2145,11 @@ void CacheStorageService::GetCacheEntryInfo(CacheEntry* aEntry,
   if (NS_FAILED(aEntry->GetStorageDataSize(&dataSize))) {
     dataSize = 0;
   }
-  int32_t fetchCount;
+  int64_t altDataSize;
+  if (NS_FAILED(aEntry->GetAltDataSize(&altDataSize))) {
+    altDataSize = 0;
+  }
+  uint32_t fetchCount;
   if (NS_FAILED(aEntry->GetFetchCount(&fetchCount))) {
     fetchCount = 0;
   }
@@ -2156,8 +2162,9 @@ void CacheStorageService::GetCacheEntryInfo(CacheEntry* aEntry,
     expirationTime = 0;
   }
 
-  aCallback->OnEntryInfo(uriSpec, enhanceId, dataSize, fetchCount, lastModified,
-                         expirationTime, aEntry->IsPinned(), info);
+  aCallback->OnEntryInfo(uriSpec, enhanceId, dataSize, altDataSize, fetchCount,
+                         lastModified, expirationTime, aEntry->IsPinned(),
+                         info);
 }
 
 // static
@@ -2228,11 +2235,11 @@ void CacheStorageService::TelemetryRecordEntryCreation(
 
   mPurgeTimeStamps.Remove(key);
 
-  Telemetry::AccumulateTimeDelta(Telemetry::HTTP_CACHE_ENTRY_RELOAD_TIME,
-                                 timeStamp, TimeStamp::NowLoRes());
+  glean::network::http_cache_entry_reload_time.AccumulateRawDuration(
+      TimeStamp::NowLoRes() - timeStamp);
 }
 
-void CacheStorageService::TelemetryRecordEntryRemoval(CacheEntry const* entry) {
+void CacheStorageService::TelemetryRecordEntryRemoval(CacheEntry* entry) {
   MOZ_ASSERT(CacheStorageService::IsOnManagementThread());
 
   // Doomed entries must not be considered, we are only interested in purged
@@ -2252,10 +2259,10 @@ void CacheStorageService::TelemetryRecordEntryRemoval(CacheEntry const* entry) {
   TelemetryPrune(now);
   mPurgeTimeStamps.InsertOrUpdate(key, now);
 
-  Telemetry::Accumulate(Telemetry::HTTP_CACHE_ENTRY_REUSE_COUNT,
-                        entry->UseCount());
-  Telemetry::AccumulateTimeDelta(Telemetry::HTTP_CACHE_ENTRY_ALIVE_TIME,
-                                 entry->LoadStart(), TimeStamp::NowLoRes());
+  glean::network::http_cache_entry_reuse_count.AccumulateSingleSample(
+      entry->UseCount());
+  glean::network::http_cache_entry_alive_time.AccumulateRawDuration(
+      TimeStamp::NowLoRes() - entry->LoadStart());
 }
 
 // nsIMemoryReporter
@@ -2266,11 +2273,8 @@ size_t CacheStorageService::SizeOfExcludingThis(
 
   size_t n = 0;
   // The elemets are referenced by sGlobalEntryTables and are reported from
-  // there
-  n += Pool(true).mFrecencyArray.ShallowSizeOfExcludingThis(mallocSizeOf);
-  n += Pool(true).mExpirationArray.ShallowSizeOfExcludingThis(mallocSizeOf);
-  n += Pool(false).mFrecencyArray.ShallowSizeOfExcludingThis(mallocSizeOf);
-  n += Pool(false).mExpirationArray.ShallowSizeOfExcludingThis(mallocSizeOf);
+  // there.
+
   // Entries reported manually in CacheStorageService::CollectReports callback
   if (sGlobalEntryTables) {
     n += sGlobalEntryTables->ShallowSizeOfIncludingThis(mallocSizeOf);
@@ -2288,6 +2292,7 @@ size_t CacheStorageService::SizeOfIncludingThis(
 NS_IMETHODIMP
 CacheStorageService::CollectReports(nsIHandleReportCallback* aHandleReport,
                                     nsISupports* aData, bool aAnonymize) {
+  MutexAutoLock lock(mLock);
   MOZ_COLLECT_REPORT("explicit/network/cache2/io", KIND_HEAP, UNITS_BYTES,
                      CacheFileIOManager::SizeOfIncludingThis(MallocSizeOf),
                      "Memory used by the cache IO manager.");
@@ -2296,8 +2301,6 @@ CacheStorageService::CollectReports(nsIHandleReportCallback* aHandleReport,
                      CacheIndex::SizeOfIncludingThis(MallocSizeOf),
                      "Memory used by the cache index.");
 
-  MutexAutoLock lock(mLock);
-
   // Report the service instance, this doesn't report entries, done lower
   MOZ_COLLECT_REPORT("explicit/network/cache2/service", KIND_HEAP, UNITS_BYTES,
                      SizeOfIncludingThis(MallocSizeOf),
