@@ -9,17 +9,20 @@
 #include "CookiePersistentStorage.h"
 
 #include "mozilla/FileUtils.h"
+#include "mozilla/StaticPrefs_network.h"
+#include "mozilla/glean/NetwerkMetrics.h"
 #include "mozilla/ScopeExit.h"
-#include "mozilla/Telemetry.h"
 #include "mozIStorageAsyncStatement.h"
 #include "mozIStorageError.h"
 #include "mozIStorageFunction.h"
 #include "mozIStorageService.h"
 #include "mozStorageHelper.h"
 #include "nsAppDirectoryServiceDefs.h"
+#include "nsICookieNotification.h"
 #include "nsICookieService.h"
 #include "nsIEffectiveTLDService.h"
 #include "nsILineInputStream.h"
+#include "nsIURIMutator.h"
 #include "nsNetUtil.h"
 #include "nsVariant.h"
 #include "prprf.h"
@@ -28,7 +31,7 @@
 // This is a hack to hide HttpOnly cookies from older browsers
 #define HTTP_ONLY_PREFIX "#HttpOnly_"
 
-constexpr auto COOKIES_SCHEMA_VERSION = 12;
+constexpr auto COOKIES_SCHEMA_VERSION = 15;
 
 // parameter indexes; see |Read|
 constexpr auto IDX_NAME = 0;
@@ -42,8 +45,8 @@ constexpr auto IDX_SECURE = 7;
 constexpr auto IDX_HTTPONLY = 8;
 constexpr auto IDX_ORIGIN_ATTRIBUTES = 9;
 constexpr auto IDX_SAME_SITE = 10;
-constexpr auto IDX_RAW_SAME_SITE = 11;
-constexpr auto IDX_SCHEME_MAP = 12;
+constexpr auto IDX_SCHEME_MAP = 11;
+constexpr auto IDX_PARTITIONED_ATTRIBUTE_SET = 12;
 
 #define COOKIES_FILE "cookies.sqlite"
 
@@ -100,10 +103,11 @@ void BindCookieParameters(mozIStorageBindingParamsArray* aParamsArray,
   rv = params->BindInt32ByName("sameSite"_ns, aCookie->SameSite());
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 
-  rv = params->BindInt32ByName("rawSameSite"_ns, aCookie->RawSameSite());
+  rv = params->BindInt32ByName("schemeMap"_ns, aCookie->SchemeMap());
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 
-  rv = params->BindInt32ByName("schemeMap"_ns, aCookie->SchemeMap());
+  rv = params->BindInt32ByName("isPartitionedAttributeSet"_ns,
+                               aCookie->RawIsPartitioned());
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 
   // Bind the params to the array.
@@ -124,14 +128,7 @@ NS_IMETHODIMP
 ConvertAppIdToOriginAttrsSQLFunction::OnFunctionCall(
     mozIStorageValueArray* aFunctionArguments, nsIVariant** aResult) {
   nsresult rv;
-  int32_t inIsolatedMozBrowser;
-
-  rv = aFunctionArguments->GetInt32(1, &inIsolatedMozBrowser);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Create an originAttributes object by inIsolatedMozBrowser.
-  // Then create the originSuffix string from this object.
-  OriginAttributes attrs(inIsolatedMozBrowser != 0);
+  OriginAttributes attrs;
   nsAutoCString suffix;
   attrs.CreateSuffix(suffix);
 
@@ -197,13 +194,109 @@ SetInBrowserFromOriginAttributesSQLFunction::OnFunctionCall(
   NS_ENSURE_TRUE(success, NS_ERROR_FAILURE);
 
   RefPtr<nsVariant> outVar(new nsVariant());
-  rv = outVar->SetAsInt32(attrs.mInIsolatedMozBrowser);
+  rv = outVar->SetAsInt32(false);
   NS_ENSURE_SUCCESS(rv, rv);
 
   outVar.forget(aResult);
   return NS_OK;
 }
 
+class FetchPartitionKeyFromOAsSQLFunction final : public mozIStorageFunction {
+  ~FetchPartitionKeyFromOAsSQLFunction() = default;
+
+  NS_DECL_ISUPPORTS
+  NS_DECL_MOZISTORAGEFUNCTION
+};
+
+NS_IMPL_ISUPPORTS(FetchPartitionKeyFromOAsSQLFunction, mozIStorageFunction);
+
+NS_IMETHODIMP
+FetchPartitionKeyFromOAsSQLFunction::OnFunctionCall(
+    mozIStorageValueArray* aFunctionArguments, nsIVariant** aResult) {
+  nsresult rv;
+
+  nsAutoCString suffix;
+  rv = aFunctionArguments->GetUTF8String(0, suffix);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  OriginAttributes attrsFromSuffix;
+  bool success = attrsFromSuffix.PopulateFromSuffix(suffix);
+  NS_ENSURE_TRUE(success, NS_ERROR_FAILURE);
+
+  RefPtr<nsVariant> outVar(new nsVariant());
+  rv = outVar->SetAsAString(attrsFromSuffix.mPartitionKey);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  outVar.forget(aResult);
+
+  return NS_OK;
+}
+
+class UpdateOAsWithPartitionHostSQLFunction final : public mozIStorageFunction {
+  ~UpdateOAsWithPartitionHostSQLFunction() = default;
+
+  NS_DECL_ISUPPORTS
+  NS_DECL_MOZISTORAGEFUNCTION
+};
+
+NS_IMPL_ISUPPORTS(UpdateOAsWithPartitionHostSQLFunction, mozIStorageFunction);
+
+NS_IMETHODIMP
+UpdateOAsWithPartitionHostSQLFunction::OnFunctionCall(
+    mozIStorageValueArray* aFunctionArguments, nsIVariant** aResult) {
+  nsresult rv;
+
+  nsAutoCString formattedOriginAttributes;
+  rv = aFunctionArguments->GetUTF8String(0, formattedOriginAttributes);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsAutoCString partitionKeyHost;
+  rv = aFunctionArguments->GetUTF8String(1, partitionKeyHost);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  OriginAttributes attrsFromSuffix;
+  bool success = attrsFromSuffix.PopulateFromSuffix(formattedOriginAttributes);
+  // On failure, do not alter the OA.
+  if (!success) {
+    RefPtr<nsVariant> outVar(new nsVariant());
+    rv = outVar->SetAsACString(formattedOriginAttributes);
+    NS_ENSURE_SUCCESS(rv, rv);
+    outVar.forget(aResult);
+    return NS_OK;
+  }
+
+  // This is a bit hacky. However, CHIPS cookies can only be set in secure
+  // contexts. So, the scheme has to be https.
+  nsAutoCString schemeHost;
+  schemeHost.AssignLiteral("https://");
+
+  if (*partitionKeyHost.get() == '.') {
+    schemeHost.Append(nsDependentCSubstring(partitionKeyHost, 1));
+  } else {
+    schemeHost.Append(partitionKeyHost);
+  }
+
+  nsCOMPtr<nsIURI> uri;
+  rv = NS_NewURI(getter_AddRefs(uri), schemeHost);
+  // On failure, do not alter the OA.
+  if (NS_FAILED(rv)) {
+    RefPtr<nsVariant> outVar(new nsVariant());
+    rv = outVar->SetAsACString(formattedOriginAttributes);
+    NS_ENSURE_SUCCESS(rv, rv);
+    outVar.forget(aResult);
+    return NS_OK;
+  }
+
+  attrsFromSuffix.SetPartitionKey(uri, false);
+  attrsFromSuffix.CreateSuffix(formattedOriginAttributes);
+
+  RefPtr<nsVariant> outVar(new nsVariant());
+  rv = outVar->SetAsACString(formattedOriginAttributes);
+  NS_ENSURE_SUCCESS(rv, rv);
+  outVar.forget(aResult);
+  return NS_OK;
+}
+
 /******************************************************************************
  * DBListenerErrorHandler impl:
  * Parent class for our async storage listeners that handles the logging of
@@ -362,6 +455,7 @@ NS_IMPL_ISUPPORTS(CloseCookieDBListener, mozIStorageCompletionCallback)
 already_AddRefed<CookiePersistentStorage> CookiePersistentStorage::Create() {
   RefPtr<CookiePersistentStorage> storage = new CookiePersistentStorage();
   storage->Init();
+  storage->Activate();
 
   return storage.forget();
 }
@@ -371,26 +465,31 @@ CookiePersistentStorage::CookiePersistentStorage()
       mInitialized(false),
       mCorruptFlag(OK) {}
 
-void CookiePersistentStorage::NotifyChangedInternal(nsISupports* aSubject,
-                                                    const char16_t* aData,
-                                                    bool aOldCookieIsSession) {
+void CookiePersistentStorage::NotifyChangedInternal(
+    nsICookieNotification* aNotification, bool aOldCookieIsSession) {
+  MOZ_ASSERT(aNotification);
   // Notify for topic "session-cookie-changed" to update the copy of session
   // cookies in session restore component.
 
+  nsICookieNotification::Action action = aNotification->GetAction();
+
   // Filter out notifications for individual non-session cookies.
-  if (u"changed"_ns.Equals(aData) || u"deleted"_ns.Equals(aData) ||
-      u"added"_ns.Equals(aData)) {
-    nsCOMPtr<nsICookie> xpcCookie = do_QueryInterface(aSubject);
-    MOZ_ASSERT(xpcCookie);
-    auto* cookie = static_cast<Cookie*>(xpcCookie.get());
-    if (!cookie->IsSession() && !aOldCookieIsSession) {
+  if (action == nsICookieNotification::COOKIE_CHANGED ||
+      action == nsICookieNotification::COOKIE_DELETED ||
+      action == nsICookieNotification::COOKIE_ADDED) {
+    nsCOMPtr<nsICookie> xpcCookie;
+    DebugOnly<nsresult> rv =
+        aNotification->GetCookie(getter_AddRefs(xpcCookie));
+    MOZ_ASSERT(NS_SUCCEEDED(rv) && xpcCookie);
+    const Cookie& cookie = xpcCookie->AsCookie();
+    if (!cookie.IsSession() && !aOldCookieIsSession) {
       return;
     }
   }
 
   nsCOMPtr<nsIObserverService> os = services::GetObserverService();
   if (os) {
-    os->NotifyObservers(aSubject, "session-cookie-changed", aData);
+    os->NotifyObservers(aNotification, "session-cookie-changed", u"");
   }
 }
 
@@ -477,16 +576,16 @@ void CookiePersistentStorage::RemoveCookiesFromExactHost(
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 }
 
-void CookiePersistentStorage::RemoveCookieFromDB(const CookieListIter& aIter) {
+void CookiePersistentStorage::RemoveCookieFromDB(const Cookie& aCookie) {
   // if it's a non-session cookie, remove it from the db
-  if (aIter.Cookie()->IsSession() || !mDBConn) {
+  if (aCookie.IsSession() || !mDBConn) {
     return;
   }
 
   nsCOMPtr<mozIStorageBindingParamsArray> paramsArray;
   mStmtDelete->NewBindingParamsArray(getter_AddRefs(paramsArray));
 
-  PrepareCookieRemoval(aIter, paramsArray);
+  PrepareCookieRemoval(aCookie, paramsArray);
 
   DebugOnly<nsresult> rv = mStmtDelete->BindParameters(paramsArray);
   MOZ_ASSERT(NS_SUCCEEDED(rv));
@@ -497,9 +596,9 @@ void CookiePersistentStorage::RemoveCookieFromDB(const CookieListIter& aIter) {
 }
 
 void CookiePersistentStorage::PrepareCookieRemoval(
-    const CookieListIter& aIter, mozIStorageBindingParamsArray* aParamsArray) {
+    const Cookie& aCookie, mozIStorageBindingParamsArray* aParamsArray) {
   // if it's a non-session cookie, remove it from the db
-  if (aIter.Cookie()->IsSession() || !mDBConn) {
+  if (aCookie.IsSession() || !mDBConn) {
     return;
   }
 
@@ -507,17 +606,17 @@ void CookiePersistentStorage::PrepareCookieRemoval(
   aParamsArray->NewBindingParams(getter_AddRefs(params));
 
   DebugOnly<nsresult> rv =
-      params->BindUTF8StringByName("name"_ns, aIter.Cookie()->Name());
+      params->BindUTF8StringByName("name"_ns, aCookie.Name());
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 
-  rv = params->BindUTF8StringByName("host"_ns, aIter.Cookie()->Host());
+  rv = params->BindUTF8StringByName("host"_ns, aCookie.Host());
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 
-  rv = params->BindUTF8StringByName("path"_ns, aIter.Cookie()->Path());
+  rv = params->BindUTF8StringByName("path"_ns, aCookie.Path());
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 
   nsAutoCString suffix;
-  aIter.Cookie()->OriginAttributesRef().CreateSuffix(suffix);
+  aCookie.OriginAttributesRef().CreateSuffix(suffix);
   rv = params->BindUTF8StringByName("originAttributes"_ns, suffix);
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 
@@ -614,8 +713,8 @@ void CookiePersistentStorage::MaybeStoreCookiesToDB(
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 }
 
-void CookiePersistentStorage::StaleCookies(const nsTArray<Cookie*>& aCookieList,
-                                           int64_t aCurrentTimeInUsec) {
+void CookiePersistentStorage::StaleCookies(
+    const nsTArray<RefPtr<Cookie>>& aCookieList, int64_t aCurrentTimeInUsec) {
   // Create an array of parameters to bind to our update statement. Batching
   // is OK here since we're updating cookies with no interleaved operations.
   nsCOMPtr<mozIStorageBindingParamsArray> paramsArray;
@@ -809,15 +908,15 @@ CookiePersistentStorage::OpenDBResult CookiePersistentStorage::TryInitDB(
 
   // This block provides scope for the Telemetry AutoTimer
   {
-    Telemetry::AutoTimer<Telemetry::MOZ_SQLITE_COOKIES_OPEN_READAHEAD_MS>
-        telemetry;
+    auto timer = glean::network_cookies::sqlite_open_readahead.Measure();
     ReadAheadFile(mCookieFile);
 
     // open a connection to the cookie database, and only cache our connection
     // and statements upon success. The connection is opened unshared to
     // eliminate cache contention between the main and background threads.
-    rv = mStorageService->OpenUnsharedDatabase(mCookieFile,
-                                               getter_AddRefs(mSyncConn));
+    rv = mStorageService->OpenUnsharedDatabase(
+        mCookieFile, mozIStorageService::CONNECTION_DEFAULT,
+        getter_AddRefs(mSyncConn));
     NS_ENSURE_SUCCESS(rv, RESULT_RETRY);
   }
 
@@ -1380,12 +1479,66 @@ CookiePersistentStorage::OpenDBResult CookiePersistentStorage::TryInitDB(
 
         COOKIE_LOGSTRING(LogLevel::Debug,
                          ("Upgraded database to schema version 12"));
+      }
+        [[fallthrough]];
+
+      case 12: {
+        // Add the isPartitionedAttributeSet column to the table.
+        rv = mSyncConn->ExecuteSimpleSQL(
+            nsLiteralCString("ALTER TABLE moz_cookies ADD "
+                             "isPartitionedAttributeSet INTEGER DEFAULT 0;"));
+        NS_ENSURE_SUCCESS(rv, RESULT_RETRY);
+
+        COOKIE_LOGSTRING(LogLevel::Debug,
+                         ("Upgraded database to schema version 13"));
+
+        [[fallthrough]];
+      }
+
+      case 13: {
+        rv = mSyncConn->ExecuteSimpleSQL(
+            nsLiteralCString("UPDATE moz_cookies SET expiry = unixepoch() + "
+                             "34560000 WHERE expiry > unixepoch() + 34560000"));
+        NS_ENSURE_SUCCESS(rv, RESULT_RETRY);
+
+        [[fallthrough]];
+      }
+
+      case 14: {
+        nsCOMPtr<mozIStorageStatement> update;
+        rv = mSyncConn->CreateStatement(
+            nsLiteralCString("UPDATE moz_cookies SET sameSite = "
+                             ":unsetValue WHERE sameSite = :laxValue AND "
+                             "rawSameSite = :noneValue"),
+            getter_AddRefs(update));
+        NS_ENSURE_SUCCESS(rv, RESULT_RETRY);
+
+        mozStorageStatementScoper scoper(update);
+
+        rv =
+            update->BindInt32ByName("unsetValue"_ns, nsICookie::SAMESITE_UNSET);
+        MOZ_ASSERT(NS_SUCCEEDED(rv));
+
+        rv = update->BindInt32ByName("laxValue"_ns, nsICookie::SAMESITE_LAX);
+        MOZ_ASSERT(NS_SUCCEEDED(rv));
+
+        rv = update->BindInt32ByName("noneValue"_ns, nsICookie::SAMESITE_NONE);
+        MOZ_ASSERT(NS_SUCCEEDED(rv));
+
+        bool hasResult;
+        rv = update->ExecuteStep(&hasResult);
+        NS_ENSURE_SUCCESS(rv, RESULT_RETRY);
+
+        rv = mSyncConn->ExecuteSimpleSQL(nsLiteralCString(
+            "ALTER TABLE moz_cookies DROP COLUMN rawSameSite;"));
+        NS_ENSURE_SUCCESS(rv, RESULT_RETRY);
 
         // No more upgrades. Update the schema version.
         rv = mSyncConn->SetSchemaVersion(COOKIES_SCHEMA_VERSION);
         NS_ENSURE_SUCCESS(rv, RESULT_RETRY);
-      }
+
         [[fallthrough]];
+      }
 
       case COOKIES_SCHEMA_VERSION:
         break;
@@ -1413,23 +1566,24 @@ CookiePersistentStorage::OpenDBResult CookiePersistentStorage::TryInitDB(
       default: {
         // check if all the expected columns exist
         nsCOMPtr<mozIStorageStatement> stmt;
-        rv = mSyncConn->CreateStatement(nsLiteralCString("SELECT "
-                                                         "id, "
-                                                         "originAttributes, "
-                                                         "name, "
-                                                         "value, "
-                                                         "host, "
-                                                         "path, "
-                                                         "expiry, "
-                                                         "lastAccessed, "
-                                                         "creationTime, "
-                                                         "isSecure, "
-                                                         "isHttpOnly, "
-                                                         "sameSite, "
-                                                         "rawSameSite, "
-                                                         "schemeMap "
-                                                         "FROM moz_cookies"),
-                                        getter_AddRefs(stmt));
+        rv = mSyncConn->CreateStatement(
+            nsLiteralCString("SELECT "
+                             "id, "
+                             "originAttributes, "
+                             "name, "
+                             "value, "
+                             "host, "
+                             "path, "
+                             "expiry, "
+                             "lastAccessed, "
+                             "creationTime, "
+                             "isSecure, "
+                             "isHttpOnly, "
+                             "sameSite, "
+                             "schemeMap, "
+                             "isPartitionedAttributeSet "
+                             "FROM moz_cookies"),
+            getter_AddRefs(stmt));
         if (NS_SUCCEEDED(rv)) {
           break;
         }
@@ -1449,6 +1603,12 @@ CookiePersistentStorage::OpenDBResult CookiePersistentStorage::TryInitDB(
     return RESULT_OK;
   }
 
+  if (StaticPrefs::network_cookie_CHIPS_enabled() &&
+      StaticPrefs::network_cookie_CHIPS_lastMigrateDatabase() <
+          StaticPrefs::network_cookie_CHIPS_migrateDatabaseTarget()) {
+    CookiePersistentStorage::MoveUnpartitionedChipsCookies();
+  }
+
   // check whether to import or just read in the db
   if (tableExists) {
     return Read();
@@ -1457,6 +1617,47 @@ CookiePersistentStorage::OpenDBResult CookiePersistentStorage::TryInitDB(
   return RESULT_OK;
 }
 
+void CookiePersistentStorage::MoveUnpartitionedChipsCookies() {
+  nsCOMPtr<mozIStorageFunction> fetchPartitionKeyFromOAs(
+      new FetchPartitionKeyFromOAsSQLFunction());
+  NS_ENSURE_TRUE_VOID(fetchPartitionKeyFromOAs);
+
+  constexpr auto fetchPartitionKeyFromOAsName =
+      "FETCH_PARTITIONKEY_FROM_OAS"_ns;
+
+  nsresult rv = mSyncConn->CreateFunction(fetchPartitionKeyFromOAsName, 1,
+                                          fetchPartitionKeyFromOAs);
+  NS_ENSURE_SUCCESS_VOID(rv);
+
+  nsCOMPtr<mozIStorageFunction> updateOAsWithPartitionHost(
+      new UpdateOAsWithPartitionHostSQLFunction());
+  NS_ENSURE_TRUE_VOID(updateOAsWithPartitionHost);
+
+  constexpr auto updateOAsWithPartitionHostName =
+      "UPDATE_OAS_WITH_PARTITION_HOST"_ns;
+
+  rv = mSyncConn->CreateFunction(updateOAsWithPartitionHostName, 2,
+                                 updateOAsWithPartitionHost);
+  NS_ENSURE_SUCCESS_VOID(rv);
+
+  // Move all cookies with the Partitioned attribute set into their first-party
+  // partitioned storage by updating the origin attributes. Overwrite any
+  // existing cookies that may already be there.
+  rv = mSyncConn->ExecuteSimpleSQL(nsLiteralCString(
+      "UPDATE OR REPLACE moz_cookies  "
+      "SET originAttributes = UPDATE_OAS_WITH_PARTITION_HOST(originAttributes, "
+      "host) "
+      "WHERE FETCH_PARTITIONKEY_FROM_OAS(originAttributes) = '' "
+      "AND isPartitionedAttributeSet = 1;"));
+  NS_ENSURE_SUCCESS_VOID(rv);
+
+  rv = mSyncConn->RemoveFunction(fetchPartitionKeyFromOAsName);
+  NS_ENSURE_SUCCESS_VOID(rv);
+
+  rv = mSyncConn->RemoveFunction(updateOAsWithPartitionHostName);
+  NS_ENSURE_SUCCESS_VOID(rv);
+}
+
 void CookiePersistentStorage::RebuildCorruptDB() {
   NS_ASSERTION(!mDBConn, "shouldn't have an open db connection");
   NS_ASSERTION(mCorruptFlag == CookiePersistentStorage::CLOSING_FOR_REBUILD,
@@ -1587,22 +1788,23 @@ CookiePersistentStorage::OpenDBResult CookiePersistentStorage::Read() {
   // Read in the data synchronously.
   // see IDX_NAME, etc. for parameter indexes
   nsCOMPtr<mozIStorageStatement> stmt;
-  nsresult rv = mSyncConn->CreateStatement(nsLiteralCString("SELECT "
-                                                            "name, "
-                                                            "value, "
-                                                            "host, "
-                                                            "path, "
-                                                            "expiry, "
-                                                            "lastAccessed, "
-                                                            "creationTime, "
-                                                            "isSecure, "
-                                                            "isHttpOnly, "
-                                                            "originAttributes, "
-                                                            "sameSite, "
-                                                            "rawSameSite, "
-                                                            "schemeMap "
-                                                            "FROM moz_cookies"),
-                                           getter_AddRefs(stmt));
+  nsresult rv =
+      mSyncConn->CreateStatement(nsLiteralCString("SELECT "
+                                                  "name, "
+                                                  "value, "
+                                                  "host, "
+                                                  "path, "
+                                                  "expiry, "
+                                                  "lastAccessed, "
+                                                  "creationTime, "
+                                                  "isSecure, "
+                                                  "isHttpOnly, "
+                                                  "originAttributes, "
+                                                  "sameSite, "
+                                                  "schemeMap, "
+                                                  "isPartitionedAttributeSet "
+                                                  "FROM moz_cookies"),
+                                 getter_AddRefs(stmt));
 
   NS_ENSURE_SUCCESS(rv, RESULT_RETRY);
 
@@ -1679,40 +1881,47 @@ UniquePtr<CookieStruct> CookiePersistentStorage::GetCookieFromRow(
   bool isSecure = 0 != aRow->AsInt32(IDX_SECURE);
   bool isHttpOnly = 0 != aRow->AsInt32(IDX_HTTPONLY);
   int32_t sameSite = aRow->AsInt32(IDX_SAME_SITE);
-  int32_t rawSameSite = aRow->AsInt32(IDX_RAW_SAME_SITE);
   int32_t schemeMap = aRow->AsInt32(IDX_SCHEME_MAP);
+  bool isPartitionedAttributeSet =
+      0 != aRow->AsInt32(IDX_PARTITIONED_ATTRIBUTE_SET);
 
   // Create a new constCookie and assign the data.
   return MakeUnique<CookieStruct>(
       name, value, host, path, expiry, lastAccessed, creationTime, isHttpOnly,
-      false, isSecure, sameSite, rawSameSite,
+      false, isSecure, isPartitionedAttributeSet, sameSite,
       static_cast<nsICookie::schemeType>(schemeMap));
 }
 
-void CookiePersistentStorage::EnsureReadComplete() {
+void CookiePersistentStorage::EnsureInitialized() {
   MOZ_ASSERT(NS_IsMainThread());
 
   bool isAccumulated = false;
 
   if (!mInitialized) {
+#ifndef ANDROID
     TimeStamp startBlockTime = TimeStamp::Now();
+#endif
     MonitorAutoLock lock(mMonitor);
 
     while (!mInitialized) {
       mMonitor.Wait();
     }
-
-    Telemetry::AccumulateTimeDelta(
-        Telemetry::MOZ_SQLITE_COOKIES_BLOCK_MAIN_THREAD_MS_V2, startBlockTime);
-    Telemetry::Accumulate(
-        Telemetry::MOZ_SQLITE_COOKIES_TIME_TO_BLOCK_MAIN_THREAD_MS, 0);
+#ifndef ANDROID
+    TimeStamp endBlockTime = TimeStamp::Now();
+    mozilla::glean::networking::sqlite_cookies_block_main_thread
+        .AccumulateRawDuration(endBlockTime - startBlockTime);
+    mozilla::glean::networking::sqlite_cookies_time_to_block_main_thread
+        .AccumulateRawDuration(TimeDuration::Zero());
+#endif
     isAccumulated = true;
   } else if (!mEndInitDBConn.IsNull()) {
     // We didn't block main thread, and here comes the first cookie request.
     // Collect how close we're going to block main thread.
-    Telemetry::Accumulate(
-        Telemetry::MOZ_SQLITE_COOKIES_TIME_TO_BLOCK_MAIN_THREAD_MS,
-        (TimeStamp::Now() - mEndInitDBConn).ToMilliseconds());
+#ifndef ANDROID
+    TimeStamp now = TimeStamp::Now();
+    mozilla::glean::networking::sqlite_cookies_time_to_block_main_thread
+        .AccumulateRawDuration(now - mEndInitDBConn);
+#endif
     // Nullify the timestamp so wo don't accumulate this telemetry probe again.
     mEndInitDBConn = TimeStamp();
     isAccumulated = true;
@@ -1720,8 +1929,10 @@ void CookiePersistentStorage::EnsureReadComplete() {
     // A request comes while we finished cookie thread task and InitDBConn is
     // on the way from cookie thread to main thread. We're very close to block
     // main thread.
-    Telemetry::Accumulate(
-        Telemetry::MOZ_SQLITE_COOKIES_TIME_TO_BLOCK_MAIN_THREAD_MS, 0);
+#ifndef ANDROID
+    mozilla::glean::networking::sqlite_cookies_time_to_block_main_thread
+        .AccumulateRawDuration(TimeDuration::Zero());
+#endif
     isAccumulated = true;
   }
 
@@ -1745,12 +1956,60 @@ void CookiePersistentStorage::InitDBConn() {
     return;
   }
 
+  nsCOMPtr<nsIURI> dummyUri;
+  nsresult rv = NS_NewURI(getter_AddRefs(dummyUri), "https://example.com");
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
+  nsTArray<RefPtr<Cookie>> cleanupCookies;
+
   for (uint32_t i = 0; i < mReadArray.Length(); ++i) {
     CookieDomainTuple& tuple = mReadArray[i];
     MOZ_ASSERT(!tuple.cookie->isSession());
 
+    // filter invalid non-ipv4 host ending in number from old db values
+    nsCOMPtr<nsIURIMutator> outMut;
+    nsCOMPtr<nsIURIMutator> dummyMut;
+    rv = dummyUri->Mutate(getter_AddRefs(dummyMut));
+    MOZ_ASSERT(NS_SUCCEEDED(rv));
+    rv = dummyMut->SetHost(tuple.cookie->host(), getter_AddRefs(outMut));
+
+    if (NS_FAILED(rv)) {
+      COOKIE_LOGSTRING(LogLevel::Debug, ("Removing cookie from db with "
+                                         "newly invalid hostname: '%s'",
+                                         tuple.cookie->host().get()));
+      RefPtr<Cookie> cookie =
+          Cookie::Create(*tuple.cookie, tuple.originAttributes);
+      cleanupCookies.AppendElement(cookie);
+      continue;
+    }
+
+    // CreateValidated fixes up the creation and lastAccessed times.
+    // If the DB is corrupted and the timestaps are far away in the future
+    // we don't want the creation timestamp to update gLastCreationTime
+    // as that would contaminate all the next creation times.
+    // We fix up these dates to not be later than the current time.
+    // The downside is that if the user sets the date far away in the past
+    // then back to the current date, those cookies will be stale,
+    // but if we don't fix their dates, those cookies might never be
+    // evicted.
     RefPtr<Cookie> cookie =
-        Cookie::Create(*tuple.cookie, tuple.originAttributes);
+        Cookie::CreateValidated(*tuple.cookie, tuple.originAttributes);
+
+    // Clean up the invalid first-party partitioned cookies that don't have
+    // the 'partitioned' cookie attribution. This will also ensure that we don't
+    // read the cookie into memory.
+    if (CookieCommons::IsFirstPartyPartitionedCookieWithoutCHIPS(
+            cookie, tuple.key.mBaseDomain, tuple.key.mOriginAttributes)) {
+      // We cannot directly use the cookie after validation because the
+      // timestamps could be different from the cookies in DB. So, we need to
+      // create one from the cookie struct.
+      RefPtr<Cookie> invalidCookie =
+          Cookie::Create(*tuple.cookie, tuple.originAttributes);
+      cleanupCookies.AppendElement(invalidCookie);
+      mozilla::glean::networking::
+          cookie_count_invalid_first_party_partitioned_in_db.Add(1);
+      continue;
+    }
+
     AddCookieToList(tuple.key.mBaseDomain, tuple.key.mOriginAttributes, cookie);
   }
 
@@ -1775,6 +2034,19 @@ void CookiePersistentStorage::InitDBConn() {
                    ("InitDBConn(): mInitializedDBConn = true"));
   mEndInitDBConn = TimeStamp::Now();
 
+  for (const auto& cookie : cleanupCookies) {
+    RemoveCookieFromDB(*cookie);
+  }
+
+  // We will have migrated CHIPS cookies if the pref is set, and .unset it
+  // to prevent dupliacted work. This has to happen in the main thread though,
+  // so we waited to this point.
+  if (StaticPrefs::network_cookie_CHIPS_enabled()) {
+    Preferences::SetUint(
+        "network.cookie.CHIPS.lastMigrateDatabase",
+        StaticPrefs::network_cookie_CHIPS_migrateDatabaseTarget());
+  }
+
   nsCOMPtr<nsIObserverService> os = services::GetObserverService();
   if (os) {
     os->NotifyObservers(nullptr, "cookie-db-read", nullptr);
@@ -1785,8 +2057,9 @@ void CookiePersistentStorage::InitDBConn() {
 nsresult CookiePersistentStorage::InitDBConnInternal() {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsresult rv = mStorageService->OpenUnsharedDatabase(mCookieFile,
-                                                      getter_AddRefs(mDBConn));
+  nsresult rv = mStorageService->OpenUnsharedDatabase(
+      mCookieFile, mozIStorageService::CONNECTION_DEFAULT,
+      getter_AddRefs(mDBConn));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Set up our listeners.
@@ -1808,37 +2081,37 @@ nsresult CookiePersistentStorage::InitDBConnInternal() {
   mDBConn->ExecuteSimpleSQL("PRAGMA wal_autocheckpoint = 16"_ns);
 
   // cache frequently used statements (for insertion, deletion, and updating)
-  rv =
-      mDBConn->CreateAsyncStatement(nsLiteralCString("INSERT INTO moz_cookies ("
-                                                     "originAttributes, "
-                                                     "name, "
-                                                     "value, "
-                                                     "host, "
-                                                     "path, "
-                                                     "expiry, "
-                                                     "lastAccessed, "
-                                                     "creationTime, "
-                                                     "isSecure, "
-                                                     "isHttpOnly, "
-                                                     "sameSite, "
-                                                     "rawSameSite, "
-                                                     "schemeMap "
-                                                     ") VALUES ("
-                                                     ":originAttributes, "
-                                                     ":name, "
-                                                     ":value, "
-                                                     ":host, "
-                                                     ":path, "
-                                                     ":expiry, "
-                                                     ":lastAccessed, "
-                                                     ":creationTime, "
-                                                     ":isSecure, "
-                                                     ":isHttpOnly, "
-                                                     ":sameSite, "
-                                                     ":rawSameSite, "
-                                                     ":schemeMap "
-                                                     ")"),
-                                    getter_AddRefs(mStmtInsert));
+  rv = mDBConn->CreateAsyncStatement(
+      nsLiteralCString("INSERT INTO moz_cookies ("
+                       "originAttributes, "
+                       "name, "
+                       "value, "
+                       "host, "
+                       "path, "
+                       "expiry, "
+                       "lastAccessed, "
+                       "creationTime, "
+                       "isSecure, "
+                       "isHttpOnly, "
+                       "sameSite, "
+                       "schemeMap, "
+                       "isPartitionedAttributeSet "
+                       ") VALUES ("
+                       ":originAttributes, "
+                       ":name, "
+                       ":value, "
+                       ":host, "
+                       ":path, "
+                       ":expiry, "
+                       ":lastAccessed, "
+                       ":creationTime, "
+                       ":isSecure, "
+                       ":isHttpOnly, "
+                       ":sameSite, "
+                       ":schemeMap, "
+                       ":isPartitionedAttributeSet "
+                       ")"),
+      getter_AddRefs(mStmtInsert));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = mDBConn->CreateAsyncStatement(
@@ -1879,8 +2152,8 @@ nsresult CookiePersistentStorage::CreateTableWorker(const char* aName) {
       "isHttpOnly INTEGER, "
       "inBrowserElement INTEGER DEFAULT 0, "
       "sameSite INTEGER DEFAULT 0, "
-      "rawSameSite INTEGER DEFAULT 0, "
       "schemeMap INTEGER DEFAULT 0, "
+      "isPartitionedAttributeSet INTEGER DEFAULT 0, "
       "CONSTRAINT moz_uniqueid UNIQUE (name, host, path, originAttributes)"
       ")");
   return mSyncConn->ExecuteSimpleSQL(command);
@@ -2015,7 +2288,7 @@ already_AddRefed<nsIArray> CookiePersistentStorage::PurgeCookies(
   return PurgeCookiesWithCallbacks(
       aCurrentTimeInUsec, aMaxNumberOfCookies, aCookiePurgeAge,
       [paramsArray, self](const CookieListIter& aIter) {
-        self->PrepareCookieRemoval(aIter, paramsArray);
+        self->PrepareCookieRemoval(*aIter.Cookie(), paramsArray);
         self->RemoveCookieFromListInternal(aIter);
       },
       [paramsArray, self]() {
@@ -2025,5 +2298,33 @@ already_AddRefed<nsIArray> CookiePersistentStorage::PurgeCookies(
       });
 }
 
+void CookiePersistentStorage::CollectCookieJarSizeData() {
+  COOKIE_LOGSTRING(LogLevel::Debug,
+                   ("CookiePersistentStorage::CollectCookieJarSizeData"));
+
+  uint32_t sumPartitioned = 0;
+  uint32_t sumUnpartitioned = 0;
+  for (const auto& cookieEntry : mHostTable) {
+    if (cookieEntry.IsPartitioned()) {
+      uint16_t cePartitioned = cookieEntry.GetCookies().Length();
+      sumPartitioned += cePartitioned;
+      mozilla::glean::networking::cookie_count_part_by_key
+          .AccumulateSingleSample(cePartitioned);
+    } else {
+      uint16_t ceUnpartitioned = cookieEntry.GetCookies().Length();
+      sumUnpartitioned += ceUnpartitioned;
+      mozilla::glean::networking::cookie_count_unpart_by_key
+          .AccumulateSingleSample(ceUnpartitioned);
+    }
+  }
+
+  mozilla::glean::networking::cookie_count_total.AccumulateSingleSample(
+      mCookieCount);
+  mozilla::glean::networking::cookie_count_partitioned.AccumulateSingleSample(
+      sumPartitioned);
+  mozilla::glean::networking::cookie_count_unpartitioned.AccumulateSingleSample(
+      sumUnpartitioned);
+}
+
 }  // namespace net
 }  // namespace mozilla
