@@ -6,6 +6,8 @@
 #include "Cookie.h"
 #include "CookieCommons.h"
 #include "CookieLogging.h"
+#include "CookieNotification.h"
+#include "CookieParser.h"
 #include "CookieService.h"
 #include "mozilla/net/CookieServiceChild.h"
 #include "ErrorList.h"
@@ -14,8 +16,10 @@
 #include "mozilla/LoadInfo.h"
 #include "mozilla/BasePrincipal.h"
 #include "mozilla/ClearOnShutdown.h"
+#include "mozilla/ConsoleReportCollector.h"
 #include "mozilla/dom/ContentChild.h"
 #include "mozilla/dom/Document.h"
+#include "mozilla/glean/NetwerkMetrics.h"
 #include "mozilla/ipc/URIUtils.h"
 #include "mozilla/net/NeckoChild.h"
 #include "mozilla/StaticPrefs_network.h"
@@ -33,7 +37,6 @@
 #include "nsIWebProgressListener.h"
 #include "nsQueryObject.h"
 #include "nsServiceManagerUtils.h"
-#include "mozilla/Telemetry.h"
 #include "mozilla/TimeStamp.h"
 #include "ThirdPartyUtil.h"
 #include "nsIConsoleReportCollector.h"
@@ -118,7 +121,7 @@ RefPtr<GenericPromise> CookieServiceChild::TrackCookieLoad(
   nsCOMPtr<nsICookieJarSettings> cookieJarSettings =
       CookieCommons::GetCookieJarSettings(aChannel);
   bool isCHIPS = StaticPrefs::network_cookie_CHIPS_enabled() &&
-                 cookieJarSettings->GetPartitionForeign();
+                 !cookieJarSettings->GetBlockingAllContexts();
   bool isUnpartitioned =
       !result.contains(ThirdPartyAnalysis::IsForeign) ||
       result.contains(ThirdPartyAnalysis::IsStorageAccessPermissionGranted);
@@ -180,9 +183,10 @@ IPCResult CookieServiceChild::RecvRemoveAll() {
   return IPC_OK();
 }
 
-IPCResult CookieServiceChild::RecvRemoveCookie(const CookieStruct& aCookie,
-                                               const OriginAttributes& aAttrs) {
-  RemoveSingleCookie(aCookie, aAttrs);
+IPCResult CookieServiceChild::RecvRemoveCookie(
+    const CookieStruct& aCookie, const OriginAttributes& aAttrs,
+    const Maybe<nsID>& aOperationID) {
+  RemoveSingleCookie(aCookie, aAttrs, aOperationID);
 
   nsCOMPtr<nsIObserverService> obsService = services::GetObserverService();
   if (obsService) {
@@ -192,7 +196,8 @@ IPCResult CookieServiceChild::RecvRemoveCookie(const CookieStruct& aCookie,
 }
 
 void CookieServiceChild::RemoveSingleCookie(const CookieStruct& aCookie,
-                                            const OriginAttributes& aAttrs) {
+                                            const OriginAttributes& aAttrs,
+                                            const Maybe<nsID>& aOperationID) {
   nsCString baseDomain;
   CookieCommons::GetBaseDomainFromHost(mTLDService, aCookie.host(), baseDomain);
   CookieKey key(baseDomain, aAttrs);
@@ -204,7 +209,7 @@ void CookieServiceChild::RemoveSingleCookie(const CookieStruct& aCookie,
   }
 
   for (uint32_t i = 0; i < cookiesList->Length(); i++) {
-    Cookie* cookie = cookiesList->ElementAt(i);
+    RefPtr<Cookie> cookie = cookiesList->ElementAt(i);
     // bug 1858366: In the case that we are updating a stale cookie
     // from the content process: the parent process will signal
     // a batch deletion for the old cookie.
@@ -216,15 +221,20 @@ void CookieServiceChild::RemoveSingleCookie(const CookieStruct& aCookie,
         cookie->Path().Equals(aCookie.path()) &&
         cookie->Expiry() <= aCookie.expiry()) {
       cookiesList->RemoveElementAt(i);
+      NotifyObservers(cookie, aAttrs, CookieNotificationAction::CookieDeleted,
+                      aOperationID);
       break;
     }
   }
 }
 
 IPCResult CookieServiceChild::RecvAddCookie(const CookieStruct& aCookie,
-                                            const OriginAttributes& aAttrs) {
+                                            const OriginAttributes& aAttrs,
+                                            const Maybe<nsID>& aOperationID) {
   RefPtr<Cookie> cookie = Cookie::Create(aCookie, aAttrs);
-  RecordDocumentCookie(cookie, aAttrs);
+
+  CookieNotificationAction action = RecordDocumentCookie(cookie, aAttrs);
+  NotifyObservers(cookie, aAttrs, action, aOperationID);
 
   // signal test code to check their cookie list
   nsCOMPtr<nsIObserverService> obsService = services::GetObserverService();
@@ -241,7 +251,7 @@ IPCResult CookieServiceChild::RecvRemoveBatchDeletedCookies(
   MOZ_ASSERT(aCookiesList.Length() == aAttrsList.Length());
   for (uint32_t i = 0; i < aCookiesList.Length(); i++) {
     CookieStruct cookieStruct = aCookiesList.ElementAt(i);
-    RemoveSingleCookie(cookieStruct, aAttrsList.ElementAt(i));
+    RemoveSingleCookie(cookieStruct, aAttrsList.ElementAt(i), Nothing());
   }
 
   nsCOMPtr<nsIObserverService> obsService = services::GetObserverService();
@@ -271,17 +281,6 @@ IPCResult CookieServiceChild::RecvTrackCookiesLoad(
   return IPC_OK();
 }
 
-uint32_t CookieServiceChild::CountCookiesFromHashTable(
-    const nsACString& aBaseDomain, const OriginAttributes& aOriginAttrs) {
-  CookiesList* cookiesList = nullptr;
-
-  nsCString baseDomain;
-  CookieKey key(aBaseDomain, aOriginAttrs);
-  mCookiesMap.Get(key, &cookiesList);
-
-  return cookiesList ? cookiesList->Length() : 0;
-}
-
 /* static */ bool CookieServiceChild::RequireThirdPartyCheck(
     nsILoadInfo* aLoadInfo) {
   if (!aLoadInfo) {
@@ -300,17 +299,27 @@ uint32_t CookieServiceChild::CountCookiesFromHashTable(
          cookieBehavior == nsICookieService::BEHAVIOR_LIMIT_FOREIGN ||
          cookieBehavior == nsICookieService::BEHAVIOR_REJECT_TRACKER ||
          cookieBehavior ==
-             nsICookieService::BEHAVIOR_REJECT_TRACKER_AND_PARTITION_FOREIGN ||
-         StaticPrefs::network_cookie_thirdparty_sessionOnly() ||
-         StaticPrefs::network_cookie_thirdparty_nonsecureSessionOnly();
+             nsICookieService::BEHAVIOR_REJECT_TRACKER_AND_PARTITION_FOREIGN;
 }
 
-void CookieServiceChild::RecordDocumentCookie(Cookie* aCookie,
-                                              const OriginAttributes& aAttrs) {
+CookieServiceChild::CookieNotificationAction
+CookieServiceChild::RecordDocumentCookie(Cookie* aCookie,
+                                         const OriginAttributes& aAttrs) {
   nsAutoCString baseDomain;
   CookieCommons::GetBaseDomainFromHost(mTLDService, aCookie->Host(),
                                        baseDomain);
 
+  if (CookieCommons::IsFirstPartyPartitionedCookieWithoutCHIPS(
+          aCookie, baseDomain, aAttrs)) {
+    COOKIE_LOGSTRING(LogLevel::Error,
+                     ("Invalid first-party partitioned cookie without "
+                      "partitioned cookie attribution from the document."));
+    mozilla::glean::networking::set_invalid_first_party_partitioned_cookie.Add(
+        1);
+    MOZ_ASSERT(false);
+    return CookieNotificationAction::NoActionNeeded;
+  }
+
   CookieKey key(baseDomain, aAttrs);
   CookiesList* cookiesList = nullptr;
   mCookiesMap.Get(key, &cookiesList);
@@ -318,6 +327,9 @@ void CookieServiceChild::RecordDocumentCookie(Cookie* aCookie,
   if (!cookiesList) {
     cookiesList = mCookiesMap.GetOrInsertNew(key);
   }
+
+  bool cookieFound = false;
+
   for (uint32_t i = 0; i < cookiesList->Length(); i++) {
     Cookie* cookie = cookiesList->ElementAt(i);
     if (cookie->Name().Equals(aCookie->Name()) &&
@@ -327,156 +339,26 @@ void CookieServiceChild::RecordDocumentCookie(Cookie* aCookie,
           cookie->Expiry() == aCookie->Expiry() &&
           cookie->IsSecure() == aCookie->IsSecure() &&
           cookie->SameSite() == aCookie->SameSite() &&
-          cookie->RawSameSite() == aCookie->RawSameSite() &&
           cookie->IsSession() == aCookie->IsSession() &&
           cookie->IsHttpOnly() == aCookie->IsHttpOnly()) {
         cookie->SetLastAccessed(aCookie->LastAccessed());
-        return;
+        return CookieNotificationAction::NoActionNeeded;
       }
       cookiesList->RemoveElementAt(i);
+      cookieFound = true;
       break;
     }
   }
 
   int64_t currentTime = PR_Now() / PR_USEC_PER_SEC;
   if (aCookie->Expiry() <= currentTime) {
-    return;
+    return cookieFound ? CookieNotificationAction::CookieDeleted
+                       : CookieNotificationAction::NoActionNeeded;
   }
 
   cookiesList->AppendElement(aCookie);
-}
-
-NS_IMETHODIMP
-CookieServiceChild::GetCookieStringFromDocument(dom::Document* aDocument,
-                                                nsACString& aCookieString) {
-  NS_ENSURE_ARG(aDocument);
-
-  aCookieString.Truncate();
-
-  bool thirdParty = true;
-  nsPIDOMWindowInner* innerWindow = aDocument->GetInnerWindow();
-  // in gtests we don't have a window, let's consider those requests as 3rd
-  // party.
-  if (innerWindow) {
-    ThirdPartyUtil* thirdPartyUtil = ThirdPartyUtil::GetInstance();
-
-    if (thirdPartyUtil) {
-      Unused << thirdPartyUtil->IsThirdPartyWindow(
-          innerWindow->GetOuterWindow(), nullptr, &thirdParty);
-    }
-  }
-
-  nsCOMPtr<nsIPrincipal> cookiePrincipal =
-      aDocument->EffectiveCookiePrincipal();
-
-  nsTArray<nsCOMPtr<nsIPrincipal>> principals;
-  principals.AppendElement(cookiePrincipal);
-
-  // CHIPS - If CHIPS is enabled the partitioned cookie jar is always available
-  // (and therefore the partitioned principal), the unpartitioned cookie jar is
-  // only available in first-party or third-party with storageAccess contexts.
-  // In both cases, the document will have storage access.
-  bool isCHIPS = aDocument->CookieJarSettings()->GetPartitionForeign() &&
-                 StaticPrefs::network_cookie_CHIPS_enabled();
-  bool documentHasStorageAccess = false;
-  nsresult rv = aDocument->HasStorageAccessSync(documentHasStorageAccess);
-  NS_ENSURE_SUCCESS(rv, rv);
-  if (isCHIPS && documentHasStorageAccess) {
-    /// Assert that the cookie principal is unpartitioned.
-    MOZ_ASSERT(cookiePrincipal->OriginAttributesRef().mPartitionKey.IsEmpty());
-    // Only append the partitioned originAttributes if the partitionKey is set.
-    // The partitionKey could be empty for partitionKey in partitioned
-    // originAttributes if the document is for privilege context, such as the
-    // extension's background page.
-    if (!aDocument->PartitionedPrincipal()
-             ->OriginAttributesRef()
-             .mPartitionKey.IsEmpty()) {
-      principals.AppendElement(aDocument->PartitionedPrincipal());
-    }
-  }
-
-  for (auto& principal : principals) {
-    if (!CookieCommons::IsSchemeSupported(principal)) {
-      return NS_OK;
-    }
-
-    nsAutoCString baseDomain;
-    rv = CookieCommons::GetBaseDomain(principal, baseDomain);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return NS_OK;
-    }
-
-    CookieKey key(baseDomain, principal->OriginAttributesRef());
-    CookiesList* cookiesList = nullptr;
-    mCookiesMap.Get(key, &cookiesList);
-
-    if (!cookiesList) {
-      continue;
-    }
-
-    nsAutoCString hostFromURI;
-    rv = nsContentUtils::GetHostOrIPv6WithBrackets(principal, hostFromURI);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return NS_OK;
-    }
-
-    nsAutoCString pathFromURI;
-    principal->GetFilePath(pathFromURI);
-
-    bool isPotentiallyTrustworthy =
-        principal->GetIsOriginPotentiallyTrustworthy();
-    int64_t currentTimeInUsec = PR_Now();
-    int64_t currentTime = currentTimeInUsec / PR_USEC_PER_SEC;
-
-    cookiesList->Sort(CompareCookiesForSending());
-    for (uint32_t i = 0; i < cookiesList->Length(); i++) {
-      Cookie* cookie = cookiesList->ElementAt(i);
-      // check the host, since the base domain lookup is conservative.
-      if (!CookieCommons::DomainMatches(cookie, hostFromURI)) {
-        continue;
-      }
-
-      // We don't show HttpOnly cookies in content processes.
-      if (cookie->IsHttpOnly()) {
-        continue;
-      }
-
-      if (thirdParty && !CookieCommons::ShouldIncludeCrossSiteCookieForDocument(
-                            cookie, aDocument)) {
-        continue;
-      }
-
-      // do not display the cookie if it is secure and the host scheme isn't
-      if (cookie->IsSecure() && !isPotentiallyTrustworthy) {
-        continue;
-      }
-
-      // if the nsIURI path doesn't match the cookie path, don't send it back
-      if (!CookieCommons::PathMatches(cookie, pathFromURI)) {
-        continue;
-      }
-
-      // check if the cookie has expired
-      if (cookie->Expiry() <= currentTime) {
-        continue;
-      }
-
-      if (!cookie->Name().IsEmpty() || !cookie->Value().IsEmpty()) {
-        if (!aCookieString.IsEmpty()) {
-          aCookieString.AppendLiteral("; ");
-        }
-        if (!cookie->Name().IsEmpty()) {
-          aCookieString.Append(cookie->Name().get());
-          aCookieString.AppendLiteral("=");
-          aCookieString.Append(cookie->Value().get());
-        } else {
-          aCookieString.Append(cookie->Value().get());
-        }
-      }
-    }
-  }
-
-  return NS_OK;
+  return cookieFound ? CookieNotificationAction::CookieChanged
+                     : CookieNotificationAction::CookieAdded;
 }
 
 NS_IMETHODIMP
@@ -486,107 +368,6 @@ CookieServiceChild::GetCookieStringFromHttp(nsIURI* /*aHostURI*/,
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP
-CookieServiceChild::SetCookieStringFromDocument(
-    dom::Document* aDocument, const nsACString& aCookieString) {
-  NS_ENSURE_ARG(aDocument);
-
-  nsCOMPtr<nsIURI> documentURI;
-  nsAutoCString baseDomain;
-  OriginAttributes attrs;
-
-  // This function is executed in this context, I don't need to keep objects
-  // alive.
-  auto hasExistingCookiesLambda = [&](const nsACString& aBaseDomain,
-                                      const OriginAttributes& aAttrs) {
-    return !!CountCookiesFromHashTable(aBaseDomain, aAttrs);
-  };
-
-  RefPtr<Cookie> cookie = CookieCommons::CreateCookieFromDocument(
-      aDocument, aCookieString, PR_Now(), mTLDService, mThirdPartyUtil,
-      hasExistingCookiesLambda, getter_AddRefs(documentURI), baseDomain, attrs);
-  if (!cookie) {
-    return NS_OK;
-  }
-
-  bool thirdParty = true;
-  nsPIDOMWindowInner* innerWindow = aDocument->GetInnerWindow();
-  // in gtests we don't have a window, let's consider those requests as 3rd
-  // party.
-  if (innerWindow) {
-    ThirdPartyUtil* thirdPartyUtil = ThirdPartyUtil::GetInstance();
-
-    if (thirdPartyUtil) {
-      Unused << thirdPartyUtil->IsThirdPartyWindow(
-          innerWindow->GetOuterWindow(), nullptr, &thirdParty);
-    }
-  }
-
-  if (thirdParty && !CookieCommons::ShouldIncludeCrossSiteCookieForDocument(
-                        cookie, aDocument)) {
-    return NS_OK;
-  }
-
-  CookieKey key(baseDomain, attrs);
-  CookiesList* cookies = mCookiesMap.Get(key);
-
-  if (cookies) {
-    // We need to see if the cookie we're setting would overwrite an httponly
-    // or a secure one. This would not affect anything we send over the net
-    // (those come from the parent, which already checks this),
-    // but script could see an inconsistent view of things.
-
-    // CHIPS - If the cookie has the "Partitioned" attribute set it will be
-    // stored in the partitioned cookie jar.
-    bool needPartitioned = StaticPrefs::network_cookie_CHIPS_enabled() &&
-                           cookie->RawIsPartitioned();
-    nsCOMPtr<nsIPrincipal> principal =
-        needPartitioned ? aDocument->PartitionedPrincipal()
-                        : aDocument->EffectiveCookiePrincipal();
-    bool isPotentiallyTrustworthy =
-        principal->GetIsOriginPotentiallyTrustworthy();
-
-    for (uint32_t i = 0; i < cookies->Length(); ++i) {
-      RefPtr<Cookie> existingCookie = cookies->ElementAt(i);
-      if (existingCookie->Name().Equals(cookie->Name()) &&
-          existingCookie->Host().Equals(cookie->Host()) &&
-          existingCookie->Path().Equals(cookie->Path())) {
-        // Can't overwrite an httponly cookie from a script context.
-        if (existingCookie->IsHttpOnly()) {
-          return NS_OK;
-        }
-
-        // prevent insecure cookie from overwriting a secure one in insecure
-        // context.
-        if (existingCookie->IsSecure() && !isPotentiallyTrustworthy) {
-          return NS_OK;
-        }
-      }
-    }
-  }
-
-  RecordDocumentCookie(cookie, attrs);
-
-  if (CanSend()) {
-    nsTArray<CookieStruct> cookiesToSend;
-    cookiesToSend.AppendElement(cookie->ToIPC());
-
-    // Asynchronously call the parent.
-    dom::WindowGlobalChild* windowGlobalChild =
-        aDocument->GetWindowGlobalChild();
-
-    // If there is no WindowGlobalChild fall back to PCookieService SetCookies.
-    if (NS_WARN_IF(!windowGlobalChild)) {
-      SendSetCookies(baseDomain, attrs, documentURI, false, cookiesToSend);
-      return NS_OK;
-    }
-    windowGlobalChild->SendSetCookies(baseDomain, attrs, documentURI, false,
-                                      cookiesToSend);
-  }
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 CookieServiceChild::SetCookieStringFromHttp(nsIURI* aHostURI,
                                             const nsACString& aCookieString,
@@ -636,7 +417,7 @@ CookieServiceChild::SetCookieStringFromHttp(nsIURI* aHostURI,
       result.contains(ThirdPartyAnalysis::IsThirdPartySocialTrackingResource),
       result.contains(ThirdPartyAnalysis::IsStorageAccessPermissionGranted),
       aCookieString,
-      CountCookiesFromHashTable(baseDomain, storagePrincipalOriginAttributes),
+      HasExistingCookies(baseDomain, storagePrincipalOriginAttributes),
       storagePrincipalOriginAttributes, &rejectedReason);
 
   if (cookieStatus != STATUS_ACCEPTED &&
@@ -685,7 +466,7 @@ CookieServiceChild::SetCookieStringFromHttp(nsIURI* aHostURI,
   bool isPartitionedPrincipal =
       !storagePrincipalOriginAttributes.mPartitionKey.IsEmpty();
   bool isCHIPS = StaticPrefs::network_cookie_CHIPS_enabled() &&
-                 cookieJarSettings->GetPartitionForeign();
+                 !cookieJarSettings->GetBlockingAllContexts();
   // Only need to get OAs if we don't already use the partitioned principal.
   if (isCHIPS && !isPartitionedPrincipal) {
     StoragePrincipalHelper::GetOriginAttributes(
@@ -693,80 +474,65 @@ CookieServiceChild::SetCookieStringFromHttp(nsIURI* aHostURI,
         StoragePrincipalHelper::ePartitionedPrincipal);
   }
 
-  nsTArray<CookieStruct> cookiesToSend, partitionedCookiesToSend;
-  bool moreCookies;
-  do {
-    CookieStruct cookieData;
-    bool canSetCookie = false;
-    moreCookies = CookieService::CanSetCookie(
-        aHostURI, baseDomain, cookieData, requireHostMatch, cookieStatus,
-        cookieString, true, isForeignAndNotAddon, mustBePartitioned,
-        storagePrincipalOriginAttributes.mPrivateBrowsingId !=
-            nsIScriptSecurityManager::DEFAULT_PRIVATE_BROWSING_ID,
-        crc, canSetCookie);
-    if (!canSetCookie) {
-      continue;
-    }
+  nsAutoCString dateHeader;
+  CookieCommons::GetServerDateHeader(aChannel, dateHeader);
 
-    // check permissions from site permission list.
-    if (!CookieCommons::CheckCookiePermission(aChannel, cookieData)) {
-      COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
-                        "cookie rejected by permission manager");
-      constexpr auto CONSOLE_REJECTION_CATEGORY = "cookiesRejection"_ns;
-      CookieLogging::LogMessageToConsole(
-          crc, aHostURI, nsIScriptError::warningFlag,
-          CONSOLE_REJECTION_CATEGORY, "CookieRejectedByPermissionManager"_ns,
-          AutoTArray<nsString, 1>{
-              NS_ConvertUTF8toUTF16(cookieData.name()),
-          });
-      CookieCommons::NotifyRejected(
-          aHostURI, aChannel,
-          nsIWebProgressListener::STATE_COOKIES_BLOCKED_BY_PERMISSION,
-          OPERATION_WRITE);
-      continue;
-    }
+  CookieParser parser(crc, aHostURI);
+  parser.Parse(baseDomain, requireHostMatch, cookieStatus, cookieString,
+               dateHeader, true, isForeignAndNotAddon, mustBePartitioned,
+               storagePrincipalOriginAttributes.IsPrivateBrowsing(),
+               loadInfo->GetIsOn3PCBExceptionList());
+  if (!parser.ContainsCookie()) {
+    return NS_OK;
+  }
 
-    // CHIPS - If the partitioned attribute is set, store cookie in partitioned
-    // cookie jar independent of context. If the cookies are stored in the
-    // partitioned cookie jar anyway no special treatment of CHIPS cookies
-    // necessary.
-    bool needPartitioned =
-        isCHIPS && cookieData.isPartitioned() && !isPartitionedPrincipal;
-    nsTArray<CookieStruct>& cookiesToSendRef =
-        needPartitioned ? partitionedCookiesToSend : cookiesToSend;
-    OriginAttributes& cookieOriginAttributes =
-        needPartitioned ? partitionedPrincipalOriginAttributes
-                        : storagePrincipalOriginAttributes;
-    // Assert that partitionedPrincipalOriginAttributes are initialized if used.
-    MOZ_ASSERT_IF(
-        needPartitioned,
-        !partitionedPrincipalOriginAttributes.mPartitionKey.IsEmpty());
+  // check permissions from site permission list.
+  if (!CookieCommons::CheckCookiePermission(aChannel, parser.CookieData())) {
+    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieString,
+                      "cookie rejected by permission manager");
+    parser.RejectCookie(CookieParser::RejectedByPermissionManager);
+    CookieCommons::NotifyRejected(
+        aHostURI, aChannel,
+        nsIWebProgressListener::STATE_COOKIES_BLOCKED_BY_PERMISSION,
+        OPERATION_WRITE);
+    return NS_OK;
+  }
 
-    RefPtr<Cookie> cookie = Cookie::Create(cookieData, cookieOriginAttributes);
-    MOZ_ASSERT(cookie);
+  // CHIPS - If the partitioned attribute is set, store cookie in partitioned
+  // cookie jar independent of context. If the cookies are stored in the
+  // partitioned cookie jar anyway no special treatment of CHIPS cookies
+  // necessary.
+  bool needPartitioned =
+      isCHIPS && parser.CookieData().isPartitioned() && !isPartitionedPrincipal;
+  OriginAttributes& cookieOriginAttributes =
+      needPartitioned ? partitionedPrincipalOriginAttributes
+                      : storagePrincipalOriginAttributes;
+  // Assert that partitionedPrincipalOriginAttributes are initialized if used.
+  MOZ_ASSERT_IF(needPartitioned,
+                !partitionedPrincipalOriginAttributes.mPartitionKey.IsEmpty());
 
-    cookie->SetLastAccessed(currentTimeInUsec);
-    cookie->SetCreationTime(
-        Cookie::GenerateUniqueCreationTime(currentTimeInUsec));
+  RefPtr<Cookie> cookie =
+      Cookie::Create(parser.CookieData(), cookieOriginAttributes);
+  MOZ_ASSERT(cookie);
 
-    RecordDocumentCookie(cookie, cookieOriginAttributes);
-    cookiesToSendRef.AppendElement(cookieData);
-  } while (moreCookies);
+  cookie->SetLastAccessed(currentTimeInUsec);
+  cookie->SetCreationTime(
+      Cookie::GenerateUniqueCreationTime(currentTimeInUsec));
+
+  CookieNotificationAction action =
+      RecordDocumentCookie(cookie, cookieOriginAttributes);
+  NotifyObservers(cookie, cookieOriginAttributes, action);
 
   // Asynchronously call the parent.
   if (CanSend()) {
+    nsTArray<CookieStruct> cookies;
+    cookies.AppendElement(parser.CookieData());
+
     RefPtr<HttpChannelChild> httpChannelChild = do_QueryObject(aChannel);
     MOZ_ASSERT(httpChannelChild);
-    if (!cookiesToSend.IsEmpty()) {
-      httpChannelChild->SendSetCookies(baseDomain,
-                                       storagePrincipalOriginAttributes,
-                                       aHostURI, true, cookiesToSend);
-    }
-    if (!partitionedCookiesToSend.IsEmpty()) {
-      httpChannelChild->SendSetCookies(
-          baseDomain, partitionedPrincipalOriginAttributes, aHostURI, true,
-          partitionedCookiesToSend);
-    }
+    httpChannelChild->SendSetCookies(baseDomain, cookieOriginAttributes,
+                                     aHostURI, true, isForeignAndNotAddon,
+                                     cookies);
   }
 
   return NS_OK;
@@ -778,5 +544,145 @@ CookieServiceChild::RunInTransaction(
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+void CookieServiceChild::GetCookiesFromHost(
+    const nsACString& aBaseDomain, const OriginAttributes& aOriginAttributes,
+    nsTArray<RefPtr<Cookie>>& aCookies) {
+  CookieKey key(aBaseDomain, aOriginAttributes);
+
+  CookiesList* cookiesList = nullptr;
+  mCookiesMap.Get(key, &cookiesList);
+
+  if (cookiesList) {
+    aCookies.AppendElements(*cookiesList);
+  }
+}
+
+void CookieServiceChild::StaleCookies(const nsTArray<RefPtr<Cookie>>& aCookies,
+                                      int64_t aCurrentTimeInUsec) {
+  // Nothing to do here.
+}
+
+bool CookieServiceChild::HasExistingCookies(
+    const nsACString& aBaseDomain, const OriginAttributes& aOriginAttributes) {
+  CookiesList* cookiesList = nullptr;
+
+  CookieKey key(aBaseDomain, aOriginAttributes);
+  mCookiesMap.Get(key, &cookiesList);
+
+  return cookiesList ? cookiesList->Length() : 0;
+}
+
+void CookieServiceChild::AddCookieFromDocument(
+    CookieParser& aCookieParser, const nsACString& aBaseDomain,
+    const OriginAttributes& aOriginAttributes, Cookie& aCookie,
+    int64_t aCurrentTimeInUsec, nsIURI* aDocumentURI, bool aThirdParty,
+    dom::Document* aDocument) {
+  MOZ_ASSERT(aDocumentURI);
+  MOZ_ASSERT(aDocument);
+
+  CookieKey key(aBaseDomain, aOriginAttributes);
+  CookiesList* cookies = mCookiesMap.Get(key);
+
+  if (cookies) {
+    // We need to see if the cookie we're setting would overwrite an httponly
+    // or a secure one. This would not affect anything we send over the net
+    // (those come from the parent, which already checks this),
+    // but script could see an inconsistent view of things.
+
+    // CHIPS - If the cookie has the "Partitioned" attribute set it will be
+    // stored in the partitioned cookie jar.
+    bool needPartitioned = StaticPrefs::network_cookie_CHIPS_enabled() &&
+                           aCookie.RawIsPartitioned();
+    nsCOMPtr<nsIPrincipal> principal =
+        needPartitioned ? aDocument->PartitionedPrincipal()
+                        : aDocument->EffectiveCookiePrincipal();
+    bool isPotentiallyTrustworthy =
+        principal->GetIsOriginPotentiallyTrustworthy();
+
+    for (uint32_t i = 0; i < cookies->Length(); ++i) {
+      RefPtr<Cookie> existingCookie = cookies->ElementAt(i);
+      if (existingCookie->Name().Equals(aCookie.Name()) &&
+          existingCookie->Host().Equals(aCookie.Host()) &&
+          existingCookie->Path().Equals(aCookie.Path())) {
+        // Can't overwrite an httponly cookie from a script context.
+        if (existingCookie->IsHttpOnly()) {
+          return;
+        }
+
+        // prevent insecure cookie from overwriting a secure one in insecure
+        // context.
+        if (existingCookie->IsSecure() && !isPotentiallyTrustworthy) {
+          return;
+        }
+      }
+    }
+  }
+
+  CookieNotificationAction action =
+      RecordDocumentCookie(&aCookie, aOriginAttributes);
+  NotifyObservers(&aCookie, aOriginAttributes, action);
+
+  if (CanSend()) {
+    nsTArray<CookieStruct> cookiesToSend;
+    cookiesToSend.AppendElement(aCookie.ToIPC());
+
+    // Asynchronously call the parent.
+    dom::WindowGlobalChild* windowGlobalChild =
+        aDocument->GetWindowGlobalChild();
+
+    // If there is no WindowGlobalChild fall back to PCookieService SetCookies.
+    if (NS_WARN_IF(!windowGlobalChild)) {
+      SendSetCookies(aBaseDomain, aOriginAttributes, aDocumentURI, false,
+                     aThirdParty, cookiesToSend);
+      return;
+    }
+
+    windowGlobalChild->SendSetCookies(aBaseDomain, aOriginAttributes,
+                                      aDocumentURI, false, aThirdParty,
+                                      cookiesToSend);
+  }
+}
+
+void CookieServiceChild::NotifyObservers(Cookie* aCookie,
+                                         const OriginAttributes& aAttrs,
+                                         CookieNotificationAction aAction,
+                                         const Maybe<nsID>& aOperationID) {
+  nsICookieNotification::Action notificationAction;
+  switch (aAction) {
+    case CookieNotificationAction::NoActionNeeded:
+      return;
+
+    case CookieNotificationAction::CookieAdded:
+      notificationAction = nsICookieNotification::COOKIE_ADDED;
+      break;
+
+    case CookieNotificationAction::CookieChanged:
+      notificationAction = nsICookieNotification::COOKIE_CHANGED;
+      break;
+
+    case CookieNotificationAction::CookieDeleted:
+      notificationAction = nsICookieNotification::COOKIE_DELETED;
+      break;
+  }
+
+  nsCOMPtr<nsIObserverService> os = services::GetObserverService();
+  if (!os) {
+    return;
+  }
+
+  nsAutoCString baseDomain;
+  CookieCommons::GetBaseDomainFromHost(mTLDService, aCookie->Host(),
+                                       baseDomain);
+
+  nsCOMPtr<nsICookieNotification> notification =
+      new CookieNotification(notificationAction, aCookie, baseDomain, false,
+                             nullptr, 0, aOperationID.ptrOr(nullptr));
+
+  os->NotifyObservers(
+      notification,
+      aAttrs.IsPrivateBrowsing() ? "private-cookie-changed" : "cookie-changed",
+      u"");
+}
+
 }  // namespace net
 }  // namespace mozilla
