@@ -6,11 +6,24 @@
 #include "Cookie.h"
 #include "CookieCommons.h"
 #include "CookieLogging.h"
+#include "CookieParser.h"
+#include "CookieNotification.h"
+#include "mozilla/net/MozURL_ffi.h"
+#include "CookieService.h"
+#include "nsCOMPtr.h"
+#include "nsICookieNotification.h"
 #include "CookieStorage.h"
 #include "mozilla/dom/nsMixedContentBlocker.h"
+#include "mozilla/glean/NetwerkMetrics.h"
+#include "mozilla/StaticPrefs_network.h"
 #include "nsIMutableArray.h"
 #include "nsTPriorityQueue.h"
+#include "nsIScriptError.h"
+#include "nsIUserIdleService.h"
+#include "nsServiceManagerUtils.h"
+#include "nsComponentManagerUtils.h"
 #include "prprf.h"
+#include "nsIPrefService.h"
 
 #undef ADD_TEN_PERCENT
 #define ADD_TEN_PERCENT(i) static_cast<uint32_t>((i) + (i) / 10)
@@ -19,6 +32,10 @@
 #define LIMIT(x, low, high, default) \
   ((x) >= (low) && (x) <= (high) ? (x) : (default))
 
+// in order to keep our metrics consistent
+// we only send metrics when the pref hasn't been manipulated from the default
+static const uint32_t kChipsPartitionByteCapacityDefault = 10240;
+
 namespace mozilla {
 namespace net {
 
@@ -103,6 +120,10 @@ size_t CookieEntry::SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const {
   return amount;
 }
 
+bool CookieEntry::IsPartitioned() const {
+  return !mOriginAttributes.mPartitionKey.IsEmpty();
+}
+
 // ---------------------------------------------------------------------------
 // CookieStorage
 
@@ -117,6 +138,13 @@ void CookieStorage::Init() {
     prefBranch->AddObserver(kPrefCookiePurgeAge, this, true);
     PrefChanged(prefBranch);
   }
+
+  nsCOMPtr<nsIObserverService> observerService = services::GetObserverService();
+  NS_ENSURE_TRUE_VOID(observerService);
+
+  nsresult rv =
+      observerService->AddObserver(this, OBSERVER_TOPIC_IDLE_DAILY, true);
+  NS_ENSURE_SUCCESS_VOID(rv);
 }
 
 size_t CookieStorage::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const {
@@ -203,7 +231,7 @@ bool CookieStorage::FindSecureCookie(const nsACString& aBaseDomain,
       // aren't "/", then this situation needs to compare paths to
       // ensure only that a newly-created non-secure cookie does not
       // overlay an existing secure cookie.
-      if (CookieCommons::PathMatches(cookie, aCookie->GetFilePath())) {
+      if (CookieCommons::PathMatches(cookie, aCookie->Path())) {
         return true;
       }
     }
@@ -222,6 +250,25 @@ uint32_t CookieStorage::CountCookiesFromHost(const nsACString& aBaseDomain,
   return entry ? entry->GetCookies().Length() : 0;
 }
 
+uint32_t CookieStorage::CountCookieBytesNotMatchingCookie(
+    const Cookie& cookie, const nsACString& baseDomain) {
+  nsTArray<RefPtr<Cookie>> cookies;
+  GetCookiesFromHost(baseDomain, cookie.OriginAttributesRef(), cookies);
+
+  // count cookies with different name to the cookie being added
+  uint32_t cookieBytes = 0;
+  for (Cookie* c : cookies) {
+    nsAutoCString name;
+    nsAutoCString value;
+    c->GetName(name);
+    c->GetValue(value);
+    if (!cookie.Name().Equals(name)) {
+      cookieBytes += name.Length() + value.Length();
+    }
+  }
+  return cookieBytes;
+}
+
 void CookieStorage::GetAll(nsTArray<RefPtr<nsICookie>>& aResult) const {
   aResult.SetCapacity(mCookieCount);
 
@@ -233,16 +280,21 @@ void CookieStorage::GetAll(nsTArray<RefPtr<nsICookie>>& aResult) const {
   }
 }
 
-const nsTArray<RefPtr<Cookie>>* CookieStorage::GetCookiesFromHost(
-    const nsACString& aBaseDomain, const OriginAttributes& aOriginAttributes) {
+void CookieStorage::GetCookiesFromHost(
+    const nsACString& aBaseDomain, const OriginAttributes& aOriginAttributes,
+    nsTArray<RefPtr<Cookie>>& aCookies) {
   CookieEntry* entry =
       mHostTable.GetEntry(CookieKey(aBaseDomain, aOriginAttributes));
-  return entry ? &entry->GetCookies() : nullptr;
+  if (!entry) {
+    return;
+  }
+
+  aCookies = entry->GetCookies().Clone();
 }
 
 void CookieStorage::GetCookiesWithOriginAttributes(
     const OriginAttributesPattern& aPattern, const nsACString& aBaseDomain,
-    nsTArray<RefPtr<nsICookie>>& aResult) {
+    bool aSorted, nsTArray<RefPtr<nsICookie>>& aResult) {
   for (auto iter = mHostTable.Iter(); !iter.Done(); iter.Next()) {
     CookieEntry* entry = iter.Get();
 
@@ -260,24 +312,36 @@ void CookieStorage::GetCookiesWithOriginAttributes(
       aResult.AppendElement(entryCookies[i]);
     }
   }
+
+  if (aSorted) {
+    aResult.Sort(CompareCookiesForSending());
+  }
 }
 
 void CookieStorage::RemoveCookie(const nsACString& aBaseDomain,
                                  const OriginAttributes& aOriginAttributes,
                                  const nsACString& aHost,
                                  const nsACString& aName,
-                                 const nsACString& aPath) {
+                                 const nsACString& aPath, bool aFromHttp,
+                                 const nsID* aOperationID) {
   CookieListIter matchIter{};
   RefPtr<Cookie> cookie;
   if (FindCookie(aBaseDomain, aOriginAttributes, aHost, aName, aPath,
                  matchIter)) {
     cookie = matchIter.Cookie();
+
+    // If the old cookie is httponly, make sure we're not coming from script.
+    if (cookie && !aFromHttp && cookie->IsHttpOnly()) {
+      return;
+    }
+
     RemoveCookieFromList(matchIter);
   }
 
   if (cookie) {
     // Everything's done. Notify observers.
-    NotifyChanged(cookie, u"deleted");
+    NotifyChanged(cookie, nsICookieNotification::COOKIE_DELETED, aBaseDomain,
+                  aOperationID);
   }
 }
 
@@ -307,20 +371,85 @@ void CookieStorage::RemoveCookiesWithOriginAttributes(
       RemoveCookieFromList(iter);
 
       if (cookie) {
-        NotifyChanged(cookie, u"deleted");
+        NotifyChanged(cookie, nsICookieNotification::COOKIE_DELETED,
+                      aBaseDomain);
       }
     }
   }
 }
 
+/* static */ bool CookieStorage::SerializeIPv6BaseDomain(
+    nsACString& aBaseDomain) {
+  bool hasStartBracket = aBaseDomain.First() == '[';
+  bool hasEndBracket = aBaseDomain.Last() == ']';
+
+  // If only start or end bracket exists host is malformed.
+  if (hasStartBracket != hasEndBracket) {
+    return false;
+  }
+
+  // If the base domain is not in URL format (e.g. [::1]) add brackets so we
+  // can use rusturl_parse_ipv6addr().
+  if (!hasStartBracket) {
+    aBaseDomain.Insert('[', 0);
+    aBaseDomain.Append(']');
+  }
+
+  // Serialize base domain to "zero abbreviation" and lower-case hex
+  // representation.
+  nsAutoCString baseDomain;
+  nsresult rv = (nsresult)rusturl_parse_ipv6addr(&aBaseDomain, &baseDomain);
+  NS_ENSURE_SUCCESS(rv, false);
+
+  // Strip brackets to match principal representation.
+  aBaseDomain = Substring(baseDomain, 1, baseDomain.Length() - 2);
+
+  return true;
+}
+
 void CookieStorage::RemoveCookiesFromExactHost(
     const nsACString& aHost, const nsACString& aBaseDomain,
     const OriginAttributesPattern& aPattern) {
+  // Intermediate fix until Bug 1882259 is resolved.
+  // Bug 1860033 - Cookies do not serialize IPv6 host / base domain in contrast
+  // to principals. To allow deletion by principal serialize before comparison.
+  // We check the base domain since it is used as the CookieList key and equals
+  // the normalized (ASCII) host for IP addresses
+  // (it is equal to the CookieService::NormalizeHost() output).
+  nsAutoCString removeBaseDomain;
+  bool isIPv6 = CookieCommons::IsIPv6BaseDomain(aBaseDomain);
+  if (isIPv6) {
+    MOZ_ASSERT(!aBaseDomain.IsEmpty());
+    // Copy base domain since argument is immutable.
+    removeBaseDomain = aBaseDomain;
+    if (NS_WARN_IF(!SerializeIPv6BaseDomain(removeBaseDomain))) {
+      // Return on malformed base domains.
+      return;
+    }
+  }
+
   // Iterate the hash table of CookieEntry.
   for (auto iter = mHostTable.Iter(); !iter.Done(); iter.Next()) {
     CookieEntry* entry = iter.Get();
 
-    if (!aBaseDomain.Equals(entry->mBaseDomain)) {
+    // IPv6 host / base domain cookies
+    if (isIPv6) {
+      // If we look for a IPv6 cookie skip non-IPv6 cookie entries.
+      if (!CookieCommons::IsIPv6BaseDomain(entry->mBaseDomain)) {
+        continue;
+      }
+      // Serialize IPv6 base domains before comparison.
+      // Copy base domain since argument is immutable.
+      nsAutoCString entryBaseDomain;
+      entryBaseDomain = entry->mBaseDomain;
+      if (NS_WARN_IF(!SerializeIPv6BaseDomain(entryBaseDomain))) {
+        continue;
+      }
+      if (!removeBaseDomain.Equals(entryBaseDomain)) {
+        continue;
+      }
+      // Non-IPv6 cookies
+    } else if (!aBaseDomain.Equals(entry->mBaseDomain)) {
       continue;
     }
 
@@ -333,7 +462,9 @@ void CookieStorage::RemoveCookiesFromExactHost(
       CookieListIter iter(entry, i - 1);
       RefPtr<Cookie> cookie = iter.Cookie();
 
-      if (!aHost.Equals(cookie->RawHost())) {
+      // For IP addresses (ASCII normalized) host == baseDomain, we checked
+      // equality already.
+      if (!isIPv6 && !aHost.Equals(cookie->RawHost())) {
         continue;
       }
 
@@ -341,7 +472,8 @@ void CookieStorage::RemoveCookiesFromExactHost(
       RemoveCookieFromList(iter);
 
       if (cookie) {
-        NotifyChanged(cookie, u"deleted");
+        NotifyChanged(cookie, nsICookieNotification::COOKIE_DELETED,
+                      aBaseDomain);
       }
     }
   }
@@ -356,27 +488,111 @@ void CookieStorage::RemoveAll() {
 
   RemoveAllInternal();
 
-  NotifyChanged(nullptr, u"cleared");
+  NotifyChanged(nullptr, nsICookieNotification::ALL_COOKIES_CLEARED, ""_ns);
 }
 
-// notify observers that the cookie list changed. there are five possible
-// values for aData:
-// "deleted" means a cookie was deleted. aSubject is the deleted cookie.
-// "added"   means a cookie was added. aSubject is the added cookie.
-// "changed" means a cookie was altered. aSubject is the new cookie.
-// "cleared" means the entire cookie list was cleared. aSubject is null.
-// "batch-deleted" means a set of cookies was purged. aSubject is the list of
-// cookies.
-void CookieStorage::NotifyChanged(nsISupports* aSubject, const char16_t* aData,
-                                  bool aOldCookieIsSession) {
+// notify observers that the cookie list changed.
+void CookieStorage::NotifyChanged(nsISupports* aSubject,
+                                  nsICookieNotification::Action aAction,
+                                  const nsACString& aBaseDomain,
+                                  bool aIsThirdParty,
+                                  dom::BrowsingContext* aBrowsingContext,
+                                  bool aOldCookieIsSession,
+                                  const nsID* aOperationID) {
   nsCOMPtr<nsIObserverService> os = services::GetObserverService();
   if (!os) {
     return;
   }
-  // Notify for topic "private-cookie-changed" or "cookie-changed"
-  os->NotifyObservers(aSubject, NotificationTopic(), aData);
 
-  NotifyChangedInternal(aSubject, aData, aOldCookieIsSession);
+  nsCOMPtr<nsICookie> cookie;
+  nsCOMPtr<nsIArray> batchDeletedCookies;
+
+  if (aAction == nsICookieNotification::COOKIES_BATCH_DELETED) {
+    batchDeletedCookies = do_QueryInterface(aSubject);
+  } else {
+    cookie = do_QueryInterface(aSubject);
+  }
+
+  uint64_t browsingContextId = 0;
+  if (aBrowsingContext) {
+    browsingContextId = aBrowsingContext->Id();
+  }
+
+  nsCOMPtr<nsICookieNotification> notification = new CookieNotification(
+      aAction, cookie, aBaseDomain, aIsThirdParty, batchDeletedCookies,
+      browsingContextId, aOperationID);
+  // Notify for topic "private-cookie-changed" or "cookie-changed"
+  os->NotifyObservers(notification, NotificationTopic(), u"");
+
+  NotifyChangedInternal(notification, aOldCookieIsSession);
+}
+
+// return true if we finish within the partition byte limit
+bool CookieStorage::RemoveCookiesFromBackUntilUnderLimit(
+    nsTArray<CookieListIter>& aCookieListIter, Cookie* aCookie,
+    const nsACString& aBaseDomain, nsCOMPtr<nsIArray>& aPurgedList) {
+  MOZ_ASSERT(aCookie);
+  auto it = aCookieListIter.rbegin();
+  while (it != aCookieListIter.rend()) {
+    RefPtr<Cookie> evictedCookie = (*it).Cookie();
+    COOKIE_LOGEVICTED(evictedCookie,
+                      "Too many cookie bytes for this partition");
+    RemoveCookieFromList(*it);
+    CreateOrUpdatePurgeList(aPurgedList, evictedCookie);
+    MOZ_ASSERT((*it).entry);
+    if (PartitionLimitExceededBytes(aCookie, aBaseDomain, false) <= 0) {
+      return true;
+    }
+  }
+  return false;
+}
+
+void CookieStorage::RemoveOlderCookiesUntilUnderLimit(
+    CookieEntry* aEntry, Cookie* aCookie, const nsACString& aBaseDomain,
+    nsCOMPtr<nsIArray>& aPurgedList) {
+  MOZ_ASSERT(aEntry);
+  // remove insecure older cookies until we are within the byte limit
+  // so CHIPS-partitioned cookies will not be detected here since they must be
+  // secure
+  const CookieEntry::ArrayType& cookies = aEntry->GetCookies();
+  using MaybePurgeList = nsTArray<CookieListIter>;
+  MaybePurgeList maybePurgeListInsecure(aEntry->GetCookies().Length());
+  for (CookieEntry::IndexType i = 0; i < cookies.Length(); ++i) {
+    CookieListIter iter(aEntry, i);
+    if (!iter.Cookie()->IsSecure()) {
+      maybePurgeListInsecure.AppendElement(iter);
+    }
+  }
+  maybePurgeListInsecure.Sort(CompareCookiesByAge());
+  maybePurgeListInsecure.Reverse();
+  bool underLimit = RemoveCookiesFromBackUntilUnderLimit(
+      maybePurgeListInsecure, aCookie, aBaseDomain, aPurgedList);
+
+  // remove secure older cookies until we are within the byte limit
+  if (!underLimit) {
+    MOZ_LOG(gCookieLog, LogLevel::Debug,
+            ("Still too many cookies for partition, purging secure\n"));
+    MaybePurgeList maybePurgeList(aEntry->GetCookies().Length());
+    for (CookieEntry::IndexType i = 0; i < cookies.Length(); ++i) {
+      CookieListIter iter(aEntry, i);
+      maybePurgeList.AppendElement(iter);
+    }
+    maybePurgeList.Sort(CompareCookiesByAge());
+    maybePurgeList.Reverse();
+    Unused << RemoveCookiesFromBackUntilUnderLimit(maybePurgeList, aCookie,
+                                                   aBaseDomain, aPurgedList);
+  }
+}
+
+int32_t CookieStorage::PartitionLimitExceededBytes(
+    Cookie* aCookie, const nsACString& aBaseDomain, bool aHardMax) {
+  uint32_t newCount = CountCookieBytesNotMatchingCookie(*aCookie, aBaseDomain) +
+                      aCookie->NameAndValueBytes();
+  double factor = aHardMax ? 1.2 : 1;
+  // shouldn't expect more the 4000 cookies * 4000 bytes/cookie -> 16MB
+  return static_cast<int32_t>(
+      newCount -
+      StaticPrefs::network_cookie_chips_partitionLimitByteCapacity() * factor);
 }
 
 // this is a backend function for adding a cookie to the list, via SetCookie.
@@ -384,12 +600,25 @@ void CookieStorage::NotifyChanged(nsISupports* aSubject, const char16_t* aData,
 // replaces an existing cookie; or adds the cookie to the hashtable, and
 // deletes a cookie (if maximum number of cookies has been reached). also
 // performs list maintenance by removing expired cookies.
-void CookieStorage::AddCookie(nsIConsoleReportCollector* aCRC,
+void CookieStorage::AddCookie(CookieParser* aCookieParser,
                               const nsACString& aBaseDomain,
                               const OriginAttributes& aOriginAttributes,
                               Cookie* aCookie, int64_t aCurrentTimeInUsec,
                               nsIURI* aHostURI, const nsACString& aCookieHeader,
-                              bool aFromHttp) {
+                              bool aFromHttp, bool aIsThirdParty,
+                              dom::BrowsingContext* aBrowsingContext,
+                              const nsID* aOperationID) {
+  if (CookieCommons::IsFirstPartyPartitionedCookieWithoutCHIPS(
+          aCookie, aBaseDomain, aOriginAttributes)) {
+    COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieHeader,
+                      "Invalid first-party partitioned cookie without "
+                      "partitioned cookie attribution.");
+    mozilla::glean::networking::set_invalid_first_party_partitioned_cookie.Add(
+        1);
+    MOZ_ASSERT(false);
+    return;
+  }
+
   int64_t currentTime = aCurrentTimeInUsec / PR_USEC_PER_SEC;
 
   CookieListIter exactIter{};
@@ -420,12 +649,9 @@ void CookieStorage::AddCookie(nsIConsoleReportCollector* aCRC,
     COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieHeader,
                       "cookie can't save because older cookie is secure "
                       "cookie but newer cookie is non-secure cookie");
-    CookieLogging::LogMessageToConsole(
-        aCRC, aHostURI, nsIScriptError::warningFlag, CONSOLE_REJECTION_CATEGORY,
-        "CookieRejectedNonsecureOverSecure"_ns,
-        AutoTArray<nsString, 1>{
-            NS_ConvertUTF8toUTF16(aCookie->Name()),
-        });
+    if (aCookieParser) {
+      aCookieParser->RejectCookie(CookieParser::RejectedNonsecureOverSecure);
+    }
     return;
   }
 
@@ -444,12 +670,6 @@ void CookieStorage::AddCookie(nsIConsoleReportCollector* aCRC,
         // The new cookie has expired and the old one is stale. Nothing to do.
         COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieHeader,
                           "cookie has already expired");
-        CookieLogging::LogMessageToConsole(
-            aCRC, aHostURI, nsIScriptError::warningFlag,
-            CONSOLE_REJECTION_CATEGORY, "CookieRejectedExpired"_ns,
-            AutoTArray<nsString, 1>{
-                NS_ConvertUTF8toUTF16(aCookie->Name()),
-            });
         return;
       }
 
@@ -471,13 +691,10 @@ void CookieStorage::AddCookie(nsIConsoleReportCollector* aCRC,
         COOKIE_LOGFAILURE(
             SET_COOKIE, aHostURI, aCookieHeader,
             "previously stored cookie is httponly; coming from script");
-        CookieLogging::LogMessageToConsole(
-            aCRC, aHostURI, nsIScriptError::warningFlag,
-            CONSOLE_REJECTION_CATEGORY,
-            "CookieRejectedHttpOnlyButFromScript"_ns,
-            AutoTArray<nsString, 1>{
-                NS_ConvertUTF8toUTF16(aCookie->Name()),
-            });
+        if (aCookieParser) {
+          aCookieParser->RejectCookie(
+              CookieParser::RejectedHttpOnlyButFromScript);
+        }
         return;
       }
 
@@ -513,13 +730,9 @@ void CookieStorage::AddCookie(nsIConsoleReportCollector* aCRC,
       if (aCookie->Expiry() <= currentTime) {
         COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieHeader,
                           "previously stored cookie was deleted");
-        CookieLogging::LogMessageToConsole(
-            aCRC, aHostURI, nsIScriptError::warningFlag,
-            CONSOLE_REJECTION_CATEGORY, "CookieRejectedExpired"_ns,
-            AutoTArray<nsString, 1>{
-                NS_ConvertUTF8toUTF16(aCookie->Name()),
-            });
-        NotifyChanged(oldCookie, u"deleted", oldCookieIsSession);
+        NotifyChanged(oldCookie, nsICookieNotification::COOKIE_DELETED,
+                      aBaseDomain, false, aBrowsingContext, oldCookieIsSession,
+                      aOperationID);
         return;
       }
 
@@ -527,28 +740,52 @@ void CookieStorage::AddCookie(nsIConsoleReportCollector* aCRC,
       aCookie->SetCreationTime(oldCookie->CreationTime());
     }
 
+    // check for CHIPS-partitioned exceeding byte limit
+    // when we overwrite a cookie with a cookie)
+    if (CookieCommons::ChipsLimitEnabledAndChipsCookie(*aCookie,
+                                                       aBrowsingContext)) {
+      CookieEntry* entry =
+          mHostTable.GetEntry(CookieKey(aBaseDomain, aOriginAttributes));
+      if (entry) {
+        int32_t exceededBytes =
+            PartitionLimitExceededBytes(aCookie, aBaseDomain, true);
+        if (exceededBytes > 0) {
+          MOZ_LOG(gCookieLog, LogLevel::Debug,
+                  ("Partition byte limit exceeded on cookie overwrite\n"));
+          if (!StaticPrefs::network_cookie_chips_partitionLimitDryRun()) {
+            RemoveOlderCookiesUntilUnderLimit(entry, aCookie, aBaseDomain,
+                                              purgedList);
+          }
+          if (StaticPrefs::network_cookie_chips_partitionLimitByteCapacity() ==
+              kChipsPartitionByteCapacityDefault) {
+            mozilla::glean::networking::cookie_chips_partition_limit_overflow
+                .AccumulateSingleSample(exceededBytes);
+          }
+        }
+      }
+    }
   } else {
     // check if cookie has already expired
     if (aCookie->Expiry() <= currentTime) {
       COOKIE_LOGFAILURE(SET_COOKIE, aHostURI, aCookieHeader,
                         "cookie has already expired");
-      CookieLogging::LogMessageToConsole(
-          aCRC, aHostURI, nsIScriptError::warningFlag,
-          CONSOLE_REJECTION_CATEGORY, "CookieRejectedExpired"_ns,
-          AutoTArray<nsString, 1>{
-              NS_ConvertUTF8toUTF16(aCookie->Name()),
-          });
       return;
     }
 
     // check if we have to delete an old cookie.
     CookieEntry* entry =
         mHostTable.GetEntry(CookieKey(aBaseDomain, aOriginAttributes));
+    int32_t partitionLimitExceededBytes = 0;
+    // we haven't yet added the new cookie so we compare cookie list with >=
     if (entry && entry->GetCookies().Length() >= mMaxCookiesPerHost) {
       nsTArray<CookieListIter> removedIterList;
+      // +1 to account for the cookie that we are adding,
+      // to ensure that we end up with mCookieQuotaPerHost cookies.
+      // "excess" should only be > 1 when prefs have been manipulated
+      uint32_t excess = entry->GetCookies().Length() - mMaxCookiesPerHost + 1;
+      uint32_t limit = mMaxCookiesPerHost - mCookieQuotaPerHost + excess;
       // Prioritize evicting insecure cookies.
       // (draft-ietf-httpbis-cookie-alone section 3.3)
-      uint32_t limit = mMaxCookiesPerHost - mCookieQuotaPerHost;
       FindStaleCookies(entry, currentTime, false, removedIterList, limit);
       if (removedIterList.Length() == 0) {
         if (aCookie->IsSecure()) {
@@ -571,10 +808,30 @@ void CookieStorage::AddCookie(nsIConsoleReportCollector* aCRC,
         RefPtr<Cookie> evictedCookie = (*it).Cookie();
         COOKIE_LOGEVICTED(evictedCookie, "Too many cookies for this domain");
         RemoveCookieFromList(*it);
-        CreateOrUpdatePurgeList(getter_AddRefs(purgedList), evictedCookie);
+        CreateOrUpdatePurgeList(purgedList, evictedCookie);
         MOZ_ASSERT((*it).entry);
       }
+      uint32_t purgedLength = 0;
+      purgedList->GetLength(&purgedLength);
+      mozilla::glean::networking::cookie_purge_entry_max.AccumulateSingleSample(
+          purgedLength);
 
+    } else if (CookieCommons::ChipsLimitEnabledAndChipsCookie(
+                   *aCookie, aBrowsingContext) &&
+               entry &&
+               (partitionLimitExceededBytes = PartitionLimitExceededBytes(
+                    aCookie, aBaseDomain, true)) > 0) {
+      MOZ_LOG(gCookieLog, LogLevel::Debug,
+              ("Partition byte limit exceeded on cookie add\n"));
+      if (!StaticPrefs::network_cookie_chips_partitionLimitDryRun()) {
+        RemoveOlderCookiesUntilUnderLimit(entry, aCookie, aBaseDomain,
+                                          purgedList);
+      }
+      if (StaticPrefs::network_cookie_chips_partitionLimitByteCapacity() ==
+          kChipsPartitionByteCapacityDefault) {
+        mozilla::glean::networking::cookie_chips_partition_limit_overflow
+            .AccumulateSingleSample(partitionLimitExceededBytes);
+      }
     } else if (mCookieCount >= ADD_TEN_PERCENT(mMaxNumberOfCookies)) {
       int64_t maxAge = aCurrentTimeInUsec - mCookieOldestTime;
       int64_t purgeAge = ADD_TEN_PERCENT(mCookiePurgeAge);
@@ -588,6 +845,10 @@ void CookieStorage::AddCookie(nsIConsoleReportCollector* aCRC,
         // eagerly.
         purgedList = PurgeCookies(aCurrentTimeInUsec, mMaxNumberOfCookies,
                                   mCookiePurgeAge);
+        uint32_t purgedLength = 0;
+        purgedList->GetLength(&purgedLength);
+        mozilla::glean::networking::cookie_purge_max.AccumulateSingleSample(
+            purgedLength);
       }
     }
   }
@@ -602,11 +863,16 @@ void CookieStorage::AddCookie(nsIConsoleReportCollector* aCRC,
   // Now that list mutations are complete, notify observers. We do it here
   // because observers may themselves attempt to mutate the list.
   if (purgedList) {
-    NotifyChanged(purgedList, u"batch-deleted");
+    NotifyChanged(purgedList, nsICookieNotification::COOKIES_BATCH_DELETED,
+                  ""_ns, false, nullptr, false, aOperationID);
   }
 
-  NotifyChanged(aCookie, foundCookie ? u"changed" : u"added",
-                oldCookieIsSession);
+  // Notify for topic "private-cookie-changed" or "cookie-changed"
+  NotifyChanged(aCookie,
+                foundCookie ? nsICookieNotification::COOKIE_CHANGED
+                            : nsICookieNotification::COOKIE_ADDED,
+                aBaseDomain, aIsThirdParty, aBrowsingContext,
+                oldCookieIsSession, aOperationID);
 }
 
 void CookieStorage::UpdateCookieOldestTime(Cookie* aCookie) {
@@ -690,16 +956,15 @@ void CookieStorage::FindStaleCookies(CookieEntry* aEntry, int64_t aCurrentTime,
 }
 
 // static
-void CookieStorage::CreateOrUpdatePurgeList(nsIArray** aPurgedList,
+void CookieStorage::CreateOrUpdatePurgeList(nsCOMPtr<nsIArray>& aPurgedList,
                                             nsICookie* aCookie) {
-  if (!*aPurgedList) {
+  if (!aPurgedList) {
     COOKIE_LOGSTRING(LogLevel::Debug, ("Creating new purge list"));
-    nsCOMPtr<nsIArray> purgedList = CreatePurgeList(aCookie);
-    purgedList.forget(aPurgedList);
+    aPurgedList = CreatePurgeList(aCookie);
     return;
   }
 
-  nsCOMPtr<nsIMutableArray> purgedList = do_QueryInterface(*aPurgedList);
+  nsCOMPtr<nsIMutableArray> purgedList = do_QueryInterface(aPurgedList);
   if (purgedList) {
     COOKIE_LOGSTRING(LogLevel::Debug, ("Updating existing purge list"));
     purgedList->AppendElement(aCookie);
@@ -817,7 +1082,7 @@ already_AddRefed<nsIArray> CookieStorage::PurgeCookiesWithCallbacks(
 
 // remove a cookie from the hashtable, and update the iterator state.
 void CookieStorage::RemoveCookieFromList(const CookieListIter& aIter) {
-  RemoveCookieFromDB(aIter);
+  RemoveCookieFromDB(*aIter.Cookie());
   RemoveCookieFromListInternal(aIter);
 }
 
@@ -845,12 +1110,12 @@ void CookieStorage::PrefChanged(nsIPrefBranch* aPrefBranch) {
 
   if (NS_SUCCEEDED(aPrefBranch->GetIntPref(kPrefCookieQuotaPerHost, &val))) {
     mCookieQuotaPerHost = static_cast<uint16_t> LIMIT(
-        val, 1, mMaxCookiesPerHost - 1, kCookieQuotaPerHost);
+        val, 1, mMaxCookiesPerHost, kCookieQuotaPerHost);
   }
 
   if (NS_SUCCEEDED(aPrefBranch->GetIntPref(kPrefMaxCookiesPerHost, &val))) {
     mMaxCookiesPerHost = static_cast<uint16_t> LIMIT(
-        val, mCookieQuotaPerHost + 1, 0xFFFF, kMaxCookiesPerHost);
+        val, mCookieQuotaPerHost, 0xFFFF, kMaxCookiesPerHost);
   }
 
   if (NS_SUCCEEDED(aPrefBranch->GetIntPref(kPrefCookiePurgeAge, &val))) {
@@ -867,6 +1132,8 @@ CookieStorage::Observe(nsISupports* aSubject, const char* aTopic,
     if (prefBranch) {
       PrefChanged(prefBranch);
     }
+  } else if (!strcmp(aTopic, OBSERVER_TOPIC_IDLE_DAILY)) {
+    CollectCookieJarSizeData();
   }
 
   return NS_OK;
