@@ -13,8 +13,12 @@
 #define LOG_ENABLED() LOG5_ENABLED()
 
 #include "ConnectionEntry.h"
+#include "HttpConnectionUDP.h"
 #include "nsQueryObject.h"
 #include "mozilla/ChaosMode.h"
+#include "mozilla/StaticPrefs_network.h"
+#include "nsHttpHandler.h"
+#include "mozilla/net/neqo_glue_ffi_generated.h"
 
 namespace mozilla {
 namespace net {
@@ -122,6 +126,7 @@ void ConnectionEntry::DisallowHttp2() {
 
   // Can't coalesce if we're not using spdy
   mCoalescingKeys.Clear();
+  mAddresses.Clear();
 }
 
 void ConnectionEntry::DontReuseHttp3Conn() {
@@ -135,6 +140,7 @@ void ConnectionEntry::DontReuseHttp3Conn() {
 
   // Can't coalesce if we're not using http3
   mCoalescingKeys.Clear();
+  mAddresses.Clear();
 }
 
 void ConnectionEntry::RecordIPFamilyPreference(uint16_t family) {
@@ -215,7 +221,7 @@ void ConnectionEntry::InsertTransaction(
     bool aInsertAsFirstForTheSamePriority /* = false */) {
   mPendingQ.InsertTransaction(pendingTransInfo,
                               aInsertAsFirstForTheSamePriority);
-  pendingTransInfo->Transaction()->OnPendingQueueInserted();
+  pendingTransInfo->Transaction()->OnPendingQueueInserted(mConnInfo->HashKey());
 }
 
 nsTArray<RefPtr<PendingTransactionInfo>>*
@@ -241,8 +247,8 @@ bool ConnectionEntry::RestrictConnections() {
   // don't create any new ssl connections until the result of the
   // negotiation is known.
 
-  bool doRestrict = mConnInfo->FirstHopSSL() && gHttpHandler->IsSpdyEnabled() &&
-                    mUsingSpdy &&
+  bool doRestrict = mConnInfo->FirstHopSSL() &&
+                    StaticPrefs::network_http_http2_enabled() && mUsingSpdy &&
                     (mDnsAndConnectSockets.Length() || mActiveConns.Length());
 
   // If there are no restrictions, we are done
@@ -352,6 +358,17 @@ void ConnectionEntry::CloseIdleConnections(uint32_t maxToClose) {
   }
 }
 
+void ConnectionEntry::CloseExtendedCONNECTConnections() {
+  while (mExtendedCONNECTConns.Length()) {
+    RefPtr<HttpConnectionBase> conn(mExtendedCONNECTConns[0]);
+    mExtendedCONNECTConns.RemoveElementAt(0);
+
+    // safe to close connection since we are on the socket thread
+    // closing via transaction to break connection/transaction bond
+    conn->CloseTransaction(conn->Transaction(), NS_ERROR_ABORT, true);
+  }
+}
+
 nsresult ConnectionEntry::RemoveIdleConnection(nsHttpConnection* conn) {
   MOZ_ASSERT(OnSocketThread(), "not on socket thread");
 
@@ -413,6 +430,16 @@ nsresult ConnectionEntry::RemoveActiveConnection(HttpConnectionBase* conn) {
   return NS_OK;
 }
 
+nsresult ConnectionEntry::RemovePendingConnection(HttpConnectionBase* conn) {
+  MOZ_ASSERT(OnSocketThread(), "not on socket thread");
+
+  if (!mPendingConns.RemoveElement(conn)) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  return NS_OK;
+}
+
 void ConnectionEntry::ClosePersistentConnections() {
   LOG(("ConnectionEntry::ClosePersistentConnections [ci=%s]\n",
        mConnInfo->HashKey().get()));
@@ -422,6 +449,9 @@ void ConnectionEntry::ClosePersistentConnections() {
   for (int32_t i = 0; i < activeCount; i++) {
     mActiveConns[i]->DontReuse();
   }
+
+  mCoalescingKeys.Clear();
+  mAddresses.Clear();
 }
 
 uint32_t ConnectionEntry::PruneDeadConnections() {
@@ -459,20 +489,53 @@ uint32_t ConnectionEntry::PruneDeadConnections() {
   return timeToNextExpire;
 }
 
+void ConnectionEntry::MakeConnectionPendingAndDontReuse(
+    HttpConnectionBase* conn) {
+  gHttpHandler->ConnMgr()->DecrementActiveConnCount(conn);
+  mPendingConns.AppendElement(conn);
+  // After DontReuse(), the connection will be closed after the last
+  // transition is done.
+  conn->DontReuse();
+  LOG(("Move active connection to pending list [conn=%p]\n", conn));
+}
+
+template <typename ConnType>
+static void CheckForTrafficForConns(nsTArray<RefPtr<ConnType>>& aConns,
+                                    bool aCheck) {
+  for (uint32_t index = 0; index < aConns.Length(); ++index) {
+    RefPtr<nsHttpConnection> conn = do_QueryObject(aConns[index]);
+    if (conn) {
+      conn->CheckForTraffic(aCheck);
+    }
+  }
+}
+
 void ConnectionEntry::VerifyTraffic() {
   if (!mConnInfo->IsHttp3()) {
-    // Iterate over all active connections and check them.
-    for (uint32_t index = 0; index < mActiveConns.Length(); ++index) {
+    CheckForTrafficForConns(mPendingConns, true);
+    // Iterate the idle connections and unmark them for traffic checks.
+    CheckForTrafficForConns(mIdleConns, false);
+  }
+
+  uint32_t numConns = mActiveConns.Length();
+  if (numConns) {
+    // Walk the list backwards to allow us to remove entries easily.
+    for (int index = numConns - 1; index >= 0; index--) {
       RefPtr<nsHttpConnection> conn = do_QueryObject(mActiveConns[index]);
+      RefPtr<HttpConnectionUDP> connUDP = do_QueryObject(mActiveConns[index]);
       if (conn) {
         conn->CheckForTraffic(true);
-      }
-    }
-    // Iterate the idle connections and unmark them for traffic checks.
-    for (uint32_t index = 0; index < mIdleConns.Length(); ++index) {
-      RefPtr<nsHttpConnection> conn = do_QueryObject(mIdleConns[index]);
-      if (conn) {
-        conn->CheckForTraffic(false);
+        if (conn->EverUsedSpdy() &&
+            StaticPrefs::
+                network_http_move_to_pending_list_after_network_change()) {
+          mActiveConns.RemoveElementAt(index);
+          MakeConnectionPendingAndDontReuse(conn);
+        }
+      } else if (connUDP &&
+                 StaticPrefs::
+                     network_http_move_to_pending_list_after_network_change()) {
+        mActiveConns.RemoveElementAt(index);
+        MakeConnectionPendingAndDontReuse(connUDP);
       }
     }
   }
@@ -506,6 +569,19 @@ void ConnectionEntry::InsertIntoActiveConns(HttpConnectionBase* conn) {
   gHttpHandler->ConnMgr()->IncrementActiveConnCount();
 }
 
+bool ConnectionEntry::IsInExtendedCONNECTConns(HttpConnectionBase* conn) {
+  return mExtendedCONNECTConns.Contains(conn);
+}
+
+void ConnectionEntry::InsertIntoExtendedCONNECTConns(HttpConnectionBase* conn) {
+  // no incrementing of connection count since it is a tunneled connection
+  mExtendedCONNECTConns.AppendElement(conn);
+}
+
+void ConnectionEntry::RemoveExtendedCONNECTConns(HttpConnectionBase* conn) {
+  mExtendedCONNECTConns.RemoveElement(conn);
+}
+
 void ConnectionEntry::MakeAllDontReuseExcept(HttpConnectionBase* conn) {
   for (uint32_t index = 0; index < mActiveConns.Length(); ++index) {
     HttpConnectionBase* otherConn = mActiveConns[index];
@@ -516,6 +592,8 @@ void ConnectionEntry::MakeAllDontReuseExcept(HttpConnectionBase* conn) {
            "because new "
            "spdy connection (%p) takes precedence\n",
            otherConn, conn));
+      otherConn->SetCloseReason(
+          ConnectionCloseReason::CLOSE_EXISTING_CONN_FOR_COALESCING);
       otherConn->DontReuse();
     }
   }
@@ -661,6 +739,18 @@ void ConnectionEntry::CloseAllActiveConnsWithNullTransactcion(
   }
 }
 
+void ConnectionEntry::ClosePendingConnections() {
+  while (mPendingConns.Length()) {
+    RefPtr<HttpConnectionBase> conn(mPendingConns[0]);
+    mPendingConns.RemoveElementAt(0);
+
+    // Since HttpConnectionBase::Close doesn't break the bond with
+    // the connection's transaction, we must explicitely tell it
+    // to close its transaction and not just self.
+    conn->CloseTransaction(conn->Transaction(), NS_ERROR_ABORT, true);
+  }
+}
+
 void ConnectionEntry::PruneNoTraffic() {
   LOG(("  pruning no traffic [ci=%s]\n", mConnInfo->HashKey().get()));
   if (mConnInfo->IsHttp3()) {
@@ -800,6 +890,39 @@ HttpRetParams ConnectionEntry::GetConnectionData() {
   return data;
 }
 
+Http3ConnectionStatsParams ConnectionEntry::GetHttp3ConnectionStatsData() {
+  Http3ConnectionStatsParams data;
+  if (!mConnInfo->IsHttp3()) {
+    return data;
+  }
+  data.host = mConnInfo->Origin();
+  data.port = mConnInfo->OriginPort();
+
+  for (uint32_t i = 0; i < mActiveConns.Length(); i++) {
+    RefPtr<HttpConnectionUDP> connUDP = do_QueryObject(mActiveConns[i]);
+    if (!connUDP) {
+      continue;
+    }
+
+    Http3Stats stats = connUDP->GetStats();
+    Http3ConnStats res;
+    res.packetsRx = stats.packets_rx;
+    res.dupsRx = stats.dups_rx;
+    res.droppedRx = stats.dropped_rx;
+    res.savedDatagrams = stats.saved_datagrams;
+    res.packetsTx = stats.packets_tx;
+    res.lost = stats.lost;
+    res.lateAck = stats.late_ack;
+    res.ptoAck = stats.pto_ack;
+    res.wouldBlockRx = stats.would_block_rx;
+    res.wouldBlockTx = stats.would_block_tx;
+    res.ptoCounts.AppendElements(&stats.pto_counts[0], 16);
+
+    data.stats.AppendElement(std::move(res));
+  }
+  return data;
+}
+
 void ConnectionEntry::LogConnections() {
   if (!mConnInfo->IsHttp3()) {
     LOG(("active urgent conns ["));
@@ -897,18 +1020,16 @@ bool ConnectionEntry::MaybeProcessCoalescingKeys(nsIDNSAddrRecord* dnsRecord,
     return false;
   }
 
-  nsTArray<NetAddr> addressSet;
-  nsresult rv = dnsRecord->GetAddresses(addressSet);
-
-  if (NS_FAILED(rv) || addressSet.IsEmpty()) {
+  nsresult rv = dnsRecord->GetAddresses(mAddresses);
+  if (NS_FAILED(rv) || mAddresses.IsEmpty()) {
     return false;
   }
 
-  for (uint32_t i = 0; i < addressSet.Length(); ++i) {
-    if ((addressSet[i].raw.family == AF_INET && addressSet[i].inet.ip == 0) ||
-        (addressSet[i].raw.family == AF_INET6 &&
-         addressSet[i].inet6.ip.u64[0] == 0 &&
-         addressSet[i].inet6.ip.u64[1] == 0)) {
+  for (uint32_t i = 0; i < mAddresses.Length(); ++i) {
+    if ((mAddresses[i].raw.family == AF_INET && mAddresses[i].inet.ip == 0) ||
+        (mAddresses[i].raw.family == AF_INET6 &&
+         mAddresses[i].inet6.ip.u64[0] == 0 &&
+         mAddresses[i].inet6.ip.u64[1] == 0)) {
       // Bug 1680249 - Don't create the coalescing key if the ip address is
       // `0.0.0.0` or `::`.
       LOG(
@@ -919,7 +1040,7 @@ bool ConnectionEntry::MaybeProcessCoalescingKeys(nsIDNSAddrRecord* dnsRecord,
     }
     nsCString* newKey = mCoalescingKeys.AppendElement(nsCString());
     newKey->SetLength(kIPv6CStrBufSize + 26);
-    addressSet[i].ToStringBuffer(newKey->BeginWriting(), kIPv6CStrBufSize);
+    mAddresses[i].ToStringBuffer(newKey->BeginWriting(), kIPv6CStrBufSize);
     newKey->SetLength(strlen(newKey->BeginReading()));
     if (mConnInfo->GetAnonymous()) {
       newKey->AppendLiteral("~A:");
@@ -976,5 +1097,76 @@ nsresult ConnectionEntry::CreateDnsAndConnectSocket(
   return NS_OK;
 }
 
+bool ConnectionEntry::AllowToRetryDifferentIPFamilyForHttp3(nsresult aError) {
+  LOG(
+      ("ConnectionEntry::AllowToRetryDifferentIPFamilyForHttp3 %p "
+       "error=%" PRIx32,
+       this, static_cast<uint32_t>(aError)));
+  if (!IsHttp3()) {
+    MOZ_ASSERT(false, "Should not be called for non Http/3 connection");
+    return false;
+  }
+
+  if (!StaticPrefs::network_http_http3_retry_different_ip_family()) {
+    return false;
+  }
+
+  // Only allow to retry with these two errors.
+  if (aError != NS_ERROR_CONNECTION_REFUSED &&
+      aError != NS_ERROR_PROXY_CONNECTION_REFUSED) {
+    return false;
+  }
+
+  // Already retried once.
+  if (mRetriedDifferentIPFamilyForHttp3) {
+    return false;
+  }
+
+  return true;
+}
+
+void ConnectionEntry::SetRetryDifferentIPFamilyForHttp3(uint16_t aIPFamily) {
+  LOG(("ConnectionEntry::SetRetryDifferentIPFamilyForHttp3 %p, af=%u", this,
+       aIPFamily));
+
+  mPreferIPv4 = false;
+  mPreferIPv6 = false;
+
+  if (aIPFamily == AF_INET) {
+    mPreferIPv6 = true;
+  }
+
+  if (aIPFamily == AF_INET6) {
+    mPreferIPv4 = true;
+  }
+
+  mRetriedDifferentIPFamilyForHttp3 = true;
+
+  LOG(("  %p prefer ipv4=%d, ipv6=%d", this, (bool)mPreferIPv4,
+       (bool)mPreferIPv6));
+  MOZ_DIAGNOSTIC_ASSERT(mPreferIPv4 ^ mPreferIPv6);
+}
+
+void ConnectionEntry::SetServerCertHashes(
+    nsTArray<RefPtr<nsIWebTransportHash>>&& aHashes) {
+  mServerCertHashes = std::move(aHashes);
+}
+
+const nsTArray<RefPtr<nsIWebTransportHash>>&
+ConnectionEntry::GetServerCertHashes() {
+  MOZ_ASSERT(OnSocketThread(), "not on socket thread");
+  return mServerCertHashes;
+}
+
+const nsCString& ConnectionEntry::OriginFrameHashKey() {
+  MOZ_ASSERT(OnSocketThread(), "not on socket thread");
+  if (mOriginFrameHashKey.IsEmpty()) {
+    nsHttpConnectionInfo::BuildOriginFrameHashKey(
+        mOriginFrameHashKey, mConnInfo, mConnInfo->GetOrigin(),
+        mConnInfo->OriginPort());
+  }
+  return mOriginFrameHashKey;
+}
+
 }  // namespace net
 }  // namespace mozilla
