@@ -18,10 +18,13 @@
 #define TLS_EARLY_DATA_AVAILABLE_AND_USED 2
 
 #include "ASpdySession.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/StaticPrefs_network.h"
+#include "mozilla/glean/NetwerkMetrics.h"
 #include "HttpConnectionUDP.h"
 #include "nsHttpHandler.h"
 #include "Http3Session.h"
+#include "nsComponentManagerUtils.h"
+#include "nsIHttpChannelInternal.h"
 #include "nsISocketProvider.h"
 #include "nsNetAddr.h"
 #include "nsINetAddr.h"
@@ -53,7 +56,6 @@ nsresult HttpConnectionUDP::Init(nsHttpConnectionInfo* info,
   LOG1(("HttpConnectionUDP::Init this=%p", this));
   NS_ENSURE_ARG_POINTER(info);
   NS_ENSURE_TRUE(!mConnInfo, NS_ERROR_ALREADY_INITIALIZED);
-  MOZ_ASSERT(dnsRecord || NS_FAILED(status));
 
   mConnInfo = info;
   MOZ_ASSERT(mConnInfo);
@@ -66,15 +68,17 @@ nsresult HttpConnectionUDP::Init(nsHttpConnectionInfo* info,
     // SetSecurityCallbacks.
     mCallbacks = new nsMainThreadPtrHolder<nsIInterfaceRequestor>(
         "HttpConnectionUDP::mCallbacks", callbacks, false);
+    SetCloseReason(ToCloseReason(mErrorBeforeConnect));
     return mErrorBeforeConnect;
   }
 
   nsCOMPtr<nsIDNSAddrRecord> dnsAddrRecord = do_QueryInterface(dnsRecord);
-  MOZ_ASSERT(dnsAddrRecord);
   if (!dnsAddrRecord) {
     return NS_ERROR_FAILURE;
   }
   dnsAddrRecord->IsTRR(&mResolvedByTRR);
+  dnsAddrRecord->GetEffectiveTRRMode(&mEffectiveTRRMode);
+  dnsAddrRecord->GetTrrSkipReason(&mTRRSkipReason);
   NetAddr peerAddr;
   nsresult rv = dnsAddrRecord->GetNextAddr(mConnInfo->GetRoutedHost().IsEmpty()
                                                ? mConnInfo->OriginPort()
@@ -84,6 +88,22 @@ nsresult HttpConnectionUDP::Init(nsHttpConnectionInfo* info,
     return rv;
   }
 
+  // We are disabling 0.0.0.0 for non-test purposes.
+  // See https://github.com/whatwg/fetch/pull/1763 for context.
+  if (peerAddr.IsIPAddrAny()) {
+    if (StaticPrefs::network_socket_ip_addr_any_disabled()) {
+      mozilla::glean::networking::http_ip_addr_any_count
+          .Get("blocked_requests"_ns)
+          .Add(1);
+      LOG(("Connection refused because of 0.0.0.0 IP address\n"));
+      return NS_ERROR_CONNECTION_REFUSED;
+    }
+
+    mozilla::glean::networking::http_ip_addr_any_count
+        .Get("not_blocked_requests"_ns)
+        .Add(1);
+  }
+
   mSocket = do_CreateInstance("@mozilla.org/network/udp-socket;1", &rv);
   if (NS_FAILED(rv)) {
     return rv;
@@ -92,7 +112,6 @@ nsresult HttpConnectionUDP::Init(nsHttpConnectionInfo* info,
   // We need an address here so that we can convey the IP version of the
   // socket.
   NetAddr local;
-  memset(&local, 0, sizeof(local));
   local.raw.family = peerAddr.raw.family;
   rv = mSocket->InitWithAddress(&local, nullptr, false, 1);
   if (NS_FAILED(rv)) {
@@ -126,23 +145,32 @@ nsresult HttpConnectionUDP::Init(nsHttpConnectionInfo* info,
     return rv;
   }
 
-  uint32_t controlFlags = 0;
+  uint32_t providerFlags = 0;
   if (caps & NS_HTTP_LOAD_ANONYMOUS) {
-    controlFlags |= nsISocketProvider::ANONYMOUS_CONNECT;
+    providerFlags |= nsISocketProvider::ANONYMOUS_CONNECT;
   }
   if (mConnInfo->GetPrivate()) {
-    controlFlags |= nsISocketProvider::NO_PERMANENT_STORAGE;
+    providerFlags |= nsISocketProvider::NO_PERMANENT_STORAGE;
   }
   if (((caps & NS_HTTP_BE_CONSERVATIVE) || mConnInfo->GetBeConservative()) &&
       gHttpHandler->ConnMgr()->BeConservativeIfProxied(
           mConnInfo->ProxyInfo())) {
-    controlFlags |= nsISocketProvider::BE_CONSERVATIVE;
+    providerFlags |= nsISocketProvider::BE_CONSERVATIVE;
+  }
+  if ((caps & NS_HTTP_IS_RETRY) ||
+      (mConnInfo->GetTlsFlags() &
+       nsIHttpChannelInternal::TLS_FLAG_CONFIGURE_AS_RETRY)) {
+    providerFlags |= nsISocketProvider::IS_RETRY;
+  }
+
+  if (mResolvedByTRR) {
+    providerFlags |= nsISocketProvider::USED_PRIVATE_DNS;
   }
 
   mPeerAddr = new nsNetAddr(&peerAddr);
   mHttp3Session = new Http3Session();
-  rv = mHttp3Session->Init(mConnInfo, mSelfAddr, mPeerAddr, this, controlFlags,
-                           callbacks);
+  rv = mHttp3Session->Init(mConnInfo, mSelfAddr, mPeerAddr, this, providerFlags,
+                           callbacks, mSocket);
   if (NS_FAILED(rv)) {
     LOG(
         ("HttpConnectionUDP::Init mHttp3Session->Init failed "
@@ -154,6 +182,7 @@ nsresult HttpConnectionUDP::Init(nsHttpConnectionInfo* info,
     return rv;
   }
 
+  ChangeConnectionState(ConnectionState::INITED);
   // See explanation for non-strictness of this operation in
   // SetSecurityCallbacks.
   mCallbacks = new nsMainThreadPtrHolder<nsIInterfaceRequestor>(
@@ -179,6 +208,7 @@ nsresult HttpConnectionUDP::Activate(nsAHttpTransaction* trans, uint32_t caps,
         caps));
 
   if (!mExperienced && !trans->IsNullTransaction()) {
+    mHasFirstHttpTransaction = true;
     // For QUIC we have HttpConnecitonUDP before the actual connection
     // has been establish so wait for TLS handshake to be finished before
     // we mark the connection 'experienced'.
@@ -216,6 +246,11 @@ nsresult HttpConnectionUDP::Activate(nsAHttpTransaction* trans, uint32_t caps,
     return NS_ERROR_FAILURE;
   }
 
+  if (mHasFirstHttpTransaction && mExperienced) {
+    mHasFirstHttpTransaction = false;
+    mExperienceState |= ConnectionExperienceState::Experienced;
+  }
+
   Unused << ResumeSend();
   return NS_OK;
 }
@@ -226,6 +261,11 @@ void HttpConnectionUDP::Close(nsresult reason, bool aIsShutdown) {
 
   MOZ_ASSERT(OnSocketThread(), "not on socket thread");
 
+  if (mConnectionState != ConnectionState::CLOSED) {
+    RecordConnectionCloseTelemetry(reason);
+    ChangeConnectionState(ConnectionState::CLOSED);
+  }
+
   if (mForceSendTimer) {
     mForceSendTimer->Cancel();
     mForceSendTimer = nullptr;
@@ -242,6 +282,12 @@ void HttpConnectionUDP::Close(nsresult reason, bool aIsShutdown) {
     mSocket->Close();
     mSocket = nullptr;
   }
+
+  if (mHttp3Session) {
+    mHttp3Session->SetCleanShutdown(true);
+    mHttp3Session->Close(reason);
+    mHttp3Session = nullptr;
+  }
 }
 
 void HttpConnectionUDP::DontReuse() {
@@ -327,7 +373,7 @@ nsresult HttpConnectionUDP::OnHeadersAvailable(nsAHttpTransaction* trans,
     // response headers so that it will be ready to receive the new response.
     if (mIsReused &&
         ((PR_IntervalNow() - mHttp3Session->LastWriteTime()) < k1000ms)) {
-      Close(NS_ERROR_NET_RESET);
+      CloseTransaction(mHttp3Session, NS_ERROR_NET_RESET);
       *reset = true;
       return NS_OK;
     }
@@ -344,13 +390,13 @@ nsresult HttpConnectionUDP::TakeTransport(
   return NS_ERROR_FAILURE;
 }
 
-void HttpConnectionUDP::GetSecurityInfo(nsISupports** secinfo) {
+void HttpConnectionUDP::GetTLSSocketControl(nsITLSSocketControl** secinfo) {
   MOZ_ASSERT(OnSocketThread(), "not on socket thread");
-  LOG(("HttpConnectionUDP::GetSecurityInfo http3Session=%p\n",
+  LOG(("HttpConnectionUDP::GetTLSSocketControl http3Session=%p\n",
        mHttp3Session.get()));
 
   if (mHttp3Session &&
-      NS_SUCCEEDED(mHttp3Session->GetTransactionSecurityInfo(secinfo))) {
+      NS_SUCCEEDED(mHttp3Session->GetTransactionTLSSocketControl(secinfo))) {
     return;
   }
 
@@ -446,6 +492,10 @@ nsresult HttpConnectionUDP::ForceSend() {
 
 HttpVersion HttpConnectionUDP::Version() { return HttpVersion::v3_0; }
 
+PRIntervalTime HttpConnectionUDP::LastWriteTime() {
+  return mHttp3Session->LastWriteTime();
+}
+
 //-----------------------------------------------------------------------------
 // HttpConnectionUDP <private>
 //-----------------------------------------------------------------------------
@@ -497,15 +547,9 @@ void HttpConnectionUDP::CloseTransaction(nsAHttpTransaction* trans,
   mIsReused = true;
 }
 
-void HttpConnectionUDP::OnQuicTimeout(nsITimer* aTimer, void* aClosure) {
-  MOZ_ASSERT(OnSocketThread(), "not on socket thread");
-  LOG(("HttpConnectionUDP::OnQuicTimeout [this=%p]\n", aClosure));
-
-  HttpConnectionUDP* self = static_cast<HttpConnectionUDP*>(aClosure);
-  self->OnQuicTimeoutExpired();
-}
-
 void HttpConnectionUDP::OnQuicTimeoutExpired() {
+  MOZ_ASSERT(OnSocketThread(), "not on socket thread");
+  LOG(("HttpConnectionUDP::OnQuicTimeoutExpired [this=%p]\n", this));
   // if the transaction was dropped...
   if (!mHttp3Session) {
     LOG(("  no transaction; ignoring event\n"));
@@ -533,6 +577,14 @@ NS_INTERFACE_MAP_BEGIN(HttpConnectionUDP)
   NS_INTERFACE_MAP_ENTRY_CONCRETE(HttpConnectionUDP)
 NS_INTERFACE_MAP_END
 
+void HttpConnectionUDP::NotifyDataRead() {
+  mExperienceState |= ConnectionExperienceState::First_Response_Received;
+}
+
+void HttpConnectionUDP::NotifyDataWrite() {
+  mExperienceState |= ConnectionExperienceState::First_Request_Sent;
+}
+
 // called on the socket transport thread
 nsresult HttpConnectionUDP::RecvData() {
   MOZ_ASSERT(OnSocketThread(), "not on socket thread");
@@ -651,7 +703,9 @@ NS_IMETHODIMP HttpConnectionUDP::OnPacketReceived(nsIUDPSocket* aSocket) {
 
 NS_IMETHODIMP HttpConnectionUDP::OnStopListening(nsIUDPSocket* aSocket,
                                                  nsresult aStatus) {
-  CloseTransaction(mHttp3Session, aStatus);
+  // At this point, the UDP socket has already been closed. Set aIsShutdown to
+  // true to ensure that mHttp3Session is also closed.
+  CloseTransaction(mHttp3Session, aStatus, true);
   return NS_OK;
 }
 
@@ -671,5 +725,18 @@ nsresult HttpConnectionUDP::GetPeerAddr(NetAddr* addr) {
 
 bool HttpConnectionUDP::ResolvedByTRR() { return mResolvedByTRR; }
 
+nsIRequest::TRRMode HttpConnectionUDP::EffectiveTRRMode() {
+  return mEffectiveTRRMode;
+}
+
+TRRSkippedReason HttpConnectionUDP::TRRSkipReason() { return mTRRSkipReason; }
+
+Http3Stats HttpConnectionUDP::GetStats() {
+  if (!mHttp3Session) {
+    return Http3Stats();
+  }
+  return mHttp3Session->GetStats();
+}
+
 }  // namespace net
 }  // namespace mozilla
