@@ -10,7 +10,6 @@
 #include "HttpTransactionChild.h"
 
 #include "mozilla/ipc/IPCStreamUtils.h"
-#include "mozilla/ipc/BackgroundParent.h"
 #include "mozilla/net/BackgroundDataBridgeParent.h"
 #include "mozilla/net/ChannelEventQueue.h"
 #include "mozilla/net/InputChannelThrottleQueueChild.h"
@@ -25,15 +24,14 @@
 #include "nsProxyRelease.h"
 #include "nsQueryObject.h"
 #include "nsSerializationHelper.h"
+#include "OpaqueResponseUtils.h"
+#include "nsIRequestContext.h"
 
-using mozilla::ipc::BackgroundParent;
-
-namespace mozilla {
-namespace net {
+namespace mozilla::net {
 
 NS_IMPL_ISUPPORTS(HttpTransactionChild, nsIRequestObserver, nsIStreamListener,
                   nsITransportEventSink, nsIThrottledInputChannel,
-                  nsIThreadRetargetableStreamListener);
+                  nsIThreadRetargetableStreamListener, nsIEarlyHintObserver);
 
 //-----------------------------------------------------------------------------
 // HttpTransactionChild <public>
@@ -68,34 +66,17 @@ nsresult HttpTransactionChild::InitInternal(
     uint32_t caps, const HttpConnectionInfoCloneArgs& infoArgs,
     nsHttpRequestHead* requestHead, nsIInputStream* requestBody,
     uint64_t requestContentLength, bool requestBodyHasHeaders,
-    uint64_t topLevelOuterContentWindowId, uint8_t httpTrafficCategory,
-    uint64_t requestContextID, uint32_t classOfService, uint32_t initialRwin,
+    uint64_t browserId, uint8_t httpTrafficCategory, uint64_t requestContextID,
+    ClassOfService classOfService, uint32_t initialRwin,
     bool responseTimeoutEnabled, uint64_t channelId,
-    bool aHasTransactionObserver,
-    const Maybe<H2PushedStreamArg>& aPushedStreamArg) {
+    bool aHasTransactionObserver) {
   LOG(("HttpTransactionChild::InitInternal [this=%p caps=%x]\n", this, caps));
 
   RefPtr<nsHttpConnectionInfo> cinfo =
       nsHttpConnectionInfo::DeserializeHttpConnectionInfoCloneArgs(infoArgs);
   nsCOMPtr<nsIRequestContext> rc = CreateRequestContext(requestContextID);
 
-  HttpTransactionShell::OnPushCallback pushCallback = nullptr;
-  if (caps & NS_HTTP_ONPUSH_LISTENER) {
-    RefPtr<HttpTransactionChild> self = this;
-    pushCallback = [self](uint32_t aPushedStreamId, const nsACString& aUrl,
-                          const nsACString& aRequestString,
-                          HttpTransactionShell* aTransaction) {
-      bool res = false;
-      if (self->CanSend()) {
-        res =
-            self->SendOnH2PushStream(aPushedStreamId, PromiseFlatCString(aUrl),
-                                     PromiseFlatCString(aRequestString));
-      }
-      return res ? NS_OK : NS_ERROR_FAILURE;
-    };
-  }
-
-  std::function<void(TransactionObserverResult &&)> observer;
+  std::function<void(TransactionObserverResult&&)> observer;
   if (aHasTransactionObserver) {
     nsMainThreadPtrHandle<HttpTransactionChild> handle(
         new nsMainThreadPtrHolder<HttpTransactionChild>(
@@ -105,23 +86,13 @@ nsresult HttpTransactionChild::InitInternal(
     };
   }
 
-  RefPtr<nsHttpTransaction> transWithPushedStream;
-  uint32_t pushedStreamId = 0;
-  if (aPushedStreamArg) {
-    HttpTransactionChild* transChild = static_cast<HttpTransactionChild*>(
-        aPushedStreamArg.ref().transWithPushedStreamChild());
-    transWithPushedStream = transChild->GetHttpTransaction();
-    pushedStreamId = aPushedStreamArg.ref().pushedStreamId();
-  }
-
   nsresult rv = mTransaction->Init(
       caps, cinfo, requestHead, requestBody, requestContentLength,
-      requestBodyHasHeaders, GetCurrentEventTarget(),
+      requestBodyHasHeaders, GetCurrentSerialEventTarget(),
       nullptr,  // TODO: security callback, fix in bug 1512479.
-      this, topLevelOuterContentWindowId,
-      static_cast<HttpTrafficCategory>(httpTrafficCategory), rc, classOfService,
-      initialRwin, responseTimeoutEnabled, channelId, std::move(observer),
-      std::move(pushCallback), transWithPushedStream, pushedStreamId);
+      this, browserId, static_cast<HttpTrafficCategory>(httpTrafficCategory),
+      rc, classOfService, initialRwin, responseTimeoutEnabled, channelId,
+      std::move(observer));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     mTransaction = nullptr;
     return rv;
@@ -172,10 +143,9 @@ mozilla::ipc::IPCResult HttpTransactionChild::RecvInit(
     const uint64_t& aReqContentLength, const bool& aReqBodyIncludesHeaders,
     const uint64_t& aTopLevelOuterContentWindowId,
     const uint8_t& aHttpTrafficCategory, const uint64_t& aRequestContextID,
-    const uint32_t& aClassOfService, const uint32_t& aInitialRwin,
+    const ClassOfService& aClassOfService, const uint32_t& aInitialRwin,
     const bool& aResponseTimeoutEnabled, const uint64_t& aChannelId,
     const bool& aHasTransactionObserver,
-    const Maybe<H2PushedStreamArg>& aPushedStreamArg,
     const mozilla::Maybe<PInputChannelThrottleQueueChild*>& aThrottleQueue,
     const bool& aIsDocumentLoad, const TimeStamp& aRedirectStart,
     const TimeStamp& aRedirectEnd) {
@@ -199,8 +169,7 @@ mozilla::ipc::IPCResult HttpTransactionChild::RecvInit(
       aCaps, aArgs, &mRequestHead, mUploadStream, aReqContentLength,
       aReqBodyIncludesHeaders, aTopLevelOuterContentWindowId,
       aHttpTrafficCategory, aRequestContextID, aClassOfService, aInitialRwin,
-      aResponseTimeoutEnabled, aChannelId, aHasTransactionObserver,
-      aPushedStreamArg);
+      aResponseTimeoutEnabled, aChannelId, aHasTransactionObserver);
   if (NS_FAILED(rv)) {
     LOG(("HttpTransactionChild::RecvInit: [this=%p] InitInternal failed!\n",
          this));
@@ -278,7 +247,8 @@ HttpTransactionChild::OnDataAvailable(nsIRequest* aRequest,
     nsHttp::SendFunc<nsCString> sendFunc =
         [self = UnsafePtr<HttpTransactionChild>(this)](
             const nsCString& aData, uint64_t aOffset, uint32_t aCount) {
-          return self->SendOnDataAvailable(aData, aOffset, aCount);
+          return self->SendOnDataAvailable(aData, aOffset, aCount,
+                                           TimeStamp::Now());
         };
 
     LOG(("  ODA to parent process"));
@@ -288,7 +258,6 @@ HttpTransactionChild::OnDataAvailable(nsIRequest* aRequest,
     return NS_OK;
   }
 
-  ipc::AssertIsOnBackgroundThread();
   MOZ_ASSERT(mDataBridgeParent);
 
   if (!mDataBridgeParent->CanSend()) {
@@ -299,8 +268,8 @@ HttpTransactionChild::OnDataAvailable(nsIRequest* aRequest,
       [self = UnsafePtr<HttpTransactionChild>(this)](
           const nsDependentCSubstring& aData, uint64_t aOffset,
           uint32_t aCount) {
-        return self->mDataBridgeParent->SendOnTransportAndData(aOffset, aCount,
-                                                               aData);
+        return self->mDataBridgeParent->SendOnTransportAndData(
+            aOffset, aCount, aData, TimeStamp::Now());
       };
 
   LOG(("  ODA to content process"));
@@ -310,7 +279,7 @@ HttpTransactionChild::OnDataAvailable(nsIRequest* aRequest,
   }
 
   // We still need to send ODA to parent process, because the data needs to be
-  // saved in cache. Note that we set dataSentToChildProcess to true, to this
+  // saved in cache. Note that we set dataSentToChildProcess to true, so this
   // ODA will not be sent to child process.
   RefPtr<HttpTransactionChild> self = this;
   rv = NS_DispatchToMainThread(
@@ -320,7 +289,8 @@ HttpTransactionChild::OnDataAvailable(nsIRequest* aRequest,
             nsHttp::SendFunc<nsCString> sendFunc =
                 [self](const nsCString& aData, uint64_t aOffset,
                        uint32_t aCount) {
-                  return self->SendOnDataAvailable(aData, aOffset, aCount);
+                  return self->SendOnDataAvailable(aData, aOffset, aCount,
+                                                   TimeStamp::Now());
                 };
 
             if (!nsHttp::SendDataInChunks(data, offset, count, sendFunc)) {
@@ -344,6 +314,7 @@ static TimingStructArgs ToTimingStructArgs(TimingStruct aTiming) {
   args.requestStart() = aTiming.requestStart;
   args.responseStart() = aTiming.responseStart;
   args.responseEnd() = aTiming.responseEnd;
+  args.transactionPending() = aTiming.transactionPending;
   return args;
 }
 
@@ -408,22 +379,18 @@ HttpTransactionChild::OnStartRequest(nsIRequest* aRequest) {
 
   mProtocolVersion.Truncate();
 
-  nsCString serializedSecurityInfoOut;
-  nsCOMPtr<nsISupports> secInfoSupp = mTransaction->SecurityInfo();
-  if (secInfoSupp) {
-    nsCOMPtr<nsITransportSecurityInfo> info = do_QueryInterface(secInfoSupp);
+  nsCOMPtr<nsITransportSecurityInfo> securityInfo(mTransaction->SecurityInfo());
+  if (securityInfo) {
     nsAutoCString protocol;
-    if (info && NS_SUCCEEDED(info->GetNegotiatedNPN(protocol)) &&
+    if (NS_SUCCEEDED(securityInfo->GetNegotiatedNPN(protocol)) &&
         !protocol.IsEmpty()) {
       mProtocolVersion.Assign(protocol);
     }
-    nsCOMPtr<nsISerializable> secInfoSer = do_QueryInterface(secInfoSupp);
-    if (secInfoSer) {
-      NS_SerializeToString(secInfoSer, serializedSecurityInfoOut);
-    }
   }
 
-  UniquePtr<nsHttpResponseHead> head(mTransaction->TakeResponseHead());
+  RefPtr<nsHttpConnectionInfo> connInfo;
+  UniquePtr<nsHttpResponseHead> head(
+      mTransaction->TakeResponseHeadAndConnInfo(getter_AddRefs(connInfo)));
   Maybe<nsHttpResponseHead> optionalHead;
   nsTArray<uint8_t> dataForSniffer;
   if (head) {
@@ -432,13 +399,11 @@ HttpTransactionChild::OnStartRequest(nsIRequest* aRequest) {
       mProtocolVersion.Assign(nsHttp::GetProtocolVersion(version));
     }
     optionalHead = Some(*head);
-    if (mTransaction->Caps() & NS_HTTP_CALL_CONTENT_SNIFFER) {
-      nsAutoCString contentTypeOptionsHeader;
-      if (!(head->GetContentTypeOptionsHeader(contentTypeOptionsHeader) &&
-            contentTypeOptionsHeader.EqualsIgnoreCase("nosniff"))) {
-        RefPtr<nsInputStreamPump> pump = do_QueryObject(mTransactionPump);
-        pump->PeekStream(GetDataForSniffer, &dataForSniffer);
-      }
+
+    if (GetOpaqueResponseBlockedReason(*head) ==
+        OpaqueResponseBlockedReason::BLOCKED_SHOULD_SNIFF) {
+      RefPtr<nsInputStreamPump> pump = do_QueryObject(mTransactionPump);
+      pump->PeekStream(GetDataForSniffer, &dataForSniffer);
     }
   }
 
@@ -457,7 +422,7 @@ HttpTransactionChild::OnStartRequest(nsIRequest* aRequest) {
     if (dataBridgeParent) {
       mDataBridgeParent = std::move(dataBridgeParent.ref());
 
-      nsCOMPtr<nsIThread> backgroundThread =
+      nsCOMPtr<nsISerialEventTarget> backgroundThread =
           mDataBridgeParent->GetBackgroundThread();
       nsCOMPtr<nsIThreadRetargetableRequest> retargetableTransactionPump;
       retargetableTransactionPump = do_QueryObject(mTransactionPump);
@@ -478,13 +443,30 @@ HttpTransactionChild::OnStartRequest(nsIRequest* aRequest) {
   int32_t proxyConnectResponseCode =
       mTransaction->GetProxyConnectResponseCode();
 
+  nsIRequest::TRRMode mode = nsIRequest::TRR_DEFAULT_MODE;
+  TRRSkippedReason reason = nsITRRSkipReason::TRR_UNSET;
+  {
+    NetAddr selfAddr;
+    NetAddr peerAddr;
+    bool isTrr = false;
+    bool echConfigUsed = false;
+    if (mTransaction) {
+      mTransaction->GetNetworkAddresses(selfAddr, peerAddr, isTrr, mode, reason,
+                                        echConfigUsed);
+    }
+  }
+
+  HttpConnectionInfoCloneArgs infoArgs;
+  nsHttpConnectionInfo::SerializeHttpConnectionInfo(connInfo, infoArgs);
+
   Unused << SendOnStartRequest(
-      status, optionalHead, serializedSecurityInfoOut,
+      status, std::move(optionalHead), securityInfo,
       mTransaction->ProxyConnectFailed(),
       ToTimingStructArgs(mTransaction->Timings()), proxyConnectResponseCode,
       dataForSniffer, optionalAltSvcUsed, !!mDataBridgeParent,
       mTransaction->TakeRestartedState(), mTransaction->HTTPSSVCReceivedStage(),
-      mTransaction->GetSupportsHTTP3());
+      mTransaction->GetSupportsHTTP3(), mode, reason, mTransaction->Caps(),
+      TimeStamp::Now(), infoArgs);
   return NS_OK;
 }
 
@@ -505,7 +487,8 @@ ResourceTimingStructArgs HttpTransactionChild::GetTimingAttributes() {
   args.encodedBodySize() = mLogicalOffset;
   args.redirectStart() = mRedirectStart;
   args.redirectEnd() = mRedirectEnd;
-  args.protocolVersion() = mProtocolVersion;
+  args.transferSize() = mTransaction->GetTransferSize();
+  args.transactionPending() = mTransaction->GetPendingTime();
   return args;
 }
 
@@ -521,7 +504,8 @@ HttpTransactionChild::OnStopRequest(nsIRequest* aRequest, nsresult aStatus) {
     MOZ_ASSERT(NS_FAILED(mStatus), "This shoule be only called when failure");
     if (mDataBridgeParent) {
       mDataBridgeParent->OnStopRequest(mStatus, ResourceTimingStructArgs(),
-                                       TimeStamp(), nsHttpHeaderArray());
+                                       TimeStamp(), nsHttpHeaderArray(),
+                                       TimeStamp::Now());
       mDataBridgeParent = nullptr;
     }
   });
@@ -552,18 +536,16 @@ HttpTransactionChild::OnStopRequest(nsIRequest* aRequest, nsresult aStatus) {
   if (mDataBridgeParent) {
     mDataBridgeParent->OnStopRequest(
         aStatus, GetTimingAttributes(), lastActTabOpt,
-        responseTrailers ? *responseTrailers : nsHttpHeaderArray());
+        responseTrailers ? *responseTrailers : nsHttpHeaderArray(),
+        TimeStamp::Now());
     mDataBridgeParent = nullptr;
   }
 
-  RefPtr<nsHttpConnectionInfo> connInfo = mTransaction->GetConnInfo();
-  HttpConnectionInfoCloneArgs infoArgs;
-  nsHttpConnectionInfo::SerializeHttpConnectionInfo(connInfo, infoArgs);
   Unused << SendOnStopRequest(aStatus, mTransaction->ResponseIsComplete(),
                               mTransaction->GetTransferSize(),
                               ToTimingStructArgs(mTransaction->Timings()),
                               responseTrailers, mTransactionObserverResult,
-                              lastActTabOpt, mTransaction->Caps(), infoArgs);
+                              lastActTabOpt, TimeStamp::Now());
 
   return NS_OK;
 }
@@ -591,8 +573,10 @@ HttpTransactionChild::OnTransportStatus(nsITransport* aTransport,
     NetAddr peerAddr;
     bool isTrr = false;
     bool echConfigUsed = false;
+    nsIRequest::TRRMode mode = nsIRequest::TRR_DEFAULT_MODE;
+    TRRSkippedReason reason = nsITRRSkipReason::TRR_UNSET;
     if (mTransaction) {
-      mTransaction->GetNetworkAddresses(selfAddr, peerAddr, isTrr,
+      mTransaction->GetNetworkAddresses(selfAddr, peerAddr, isTrr, mode, reason,
                                         echConfigUsed);
     } else {
       nsCOMPtr<nsISocketTransport> socketTransport =
@@ -601,10 +585,12 @@ HttpTransactionChild::OnTransportStatus(nsITransport* aTransport,
         socketTransport->GetSelfAddr(&selfAddr);
         socketTransport->GetPeerAddr(&peerAddr);
         socketTransport->ResolvedByTRR(&isTrr);
+        socketTransport->GetEffectiveTRRMode(&mode);
+        socketTransport->GetTrrSkipReason(&reason);
         socketTransport->GetEchConfigUsed(&echConfigUsed);
       }
     }
-    arg.emplace(selfAddr, peerAddr, isTrr, echConfigUsed);
+    arg.emplace(selfAddr, peerAddr, isTrr, mode, reason, echConfigUsed);
   }
 
   Unused << SendOnTransportStatus(aStatus, aProgress, aProgressMax, arg);
@@ -637,5 +623,18 @@ HttpTransactionChild::CheckListenerChain() {
   return NS_OK;
 }
 
-}  // namespace net
-}  // namespace mozilla
+NS_IMETHODIMP
+HttpTransactionChild::OnDataFinished(nsresult aStatus) { return NS_OK; }
+
+NS_IMETHODIMP
+HttpTransactionChild::EarlyHint(const nsACString& aValue,
+                                const nsACString& aReferrerPolicy,
+                                const nsACString& aCSPHeader) {
+  LOG(("HttpTransactionChild::EarlyHint"));
+  if (CanSend()) {
+    Unused << SendEarlyHint(aValue, aReferrerPolicy, aCSPHeader);
+  }
+  return NS_OK;
+}
+
+}  // namespace mozilla::net
