@@ -4,7 +4,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "nsIThreadRetargetableStreamListener.h"
+#include "nsString.h"
 #include "mozilla/Assertions.h"
+#include "mozilla/Components.h"
 #include "mozilla/LinkedList.h"
 #include "mozilla/StaticPrefs_content.h"
 #include "mozilla/StoragePrincipalHelper.h"
@@ -17,6 +20,7 @@
 #include "nsError.h"
 #include "nsContentUtils.h"
 #include "nsNetUtil.h"
+#include "nsComponentManagerUtils.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "nsMimeTypes.h"
@@ -24,6 +28,7 @@
 #include "nsGkAtoms.h"
 #include "nsWhitespaceTokenizer.h"
 #include "nsIChannelEventSink.h"
+#include "nsIDNSService.h"
 #include "nsIAsyncVerifyRedirectCallback.h"
 #include "nsCharSeparatedTokenizer.h"
 #include "nsAsyncRedirectVerifyHelper.h"
@@ -35,11 +40,13 @@
 #include "nsILoadGroup.h"
 #include "nsILoadContext.h"
 #include "nsIConsoleService.h"
+#include "nsICORSPreflightCache.h"
 #include "nsINetworkInterceptController.h"
 #include "nsICorsPreflightCallback.h"
 #include "nsISupportsImpl.h"
 #include "nsHttpChannel.h"
 #include "mozilla/BasePrincipal.h"
+#include "mozilla/ExpandedPrincipal.h"
 #include "mozilla/LoadInfo.h"
 #include "mozilla/NullPrincipal.h"
 #include "nsIHttpHeaderVisitor.h"
@@ -47,23 +54,40 @@
 #include "mozilla/StaticPrefs_network.h"
 #include "mozilla/StaticPrefs_dom.h"
 #include "mozilla/dom/nsHTTPSOnlyUtils.h"
+#include "mozilla/dom/ReferrerInfo.h"
+#include "mozilla/dom/RequestBinding.h"
+#include "mozilla/glean/NetwerkProtocolHttpMetrics.h"
 #include <algorithm>
 
 using namespace mozilla;
 using namespace mozilla::net;
 
+struct CORSCacheEntry;
+
 #define PREFLIGHT_CACHE_SIZE 100
 // 5 seconds is chosen to be compatible with Chromium.
 #define PREFLIGHT_DEFAULT_EXPIRY_SECONDS 5
 
+static inline nsAutoString GetStatusCodeAsString(nsIHttpChannel* aHttp) {
+  nsAutoString result;
+  uint32_t code;
+  if (NS_SUCCEEDED(aHttp->GetResponseStatus(&code))) {
+    result.AppendInt(code);
+  }
+  return result;
+}
+
 static void LogBlockedRequest(nsIRequest* aRequest, const char* aProperty,
                               const char16_t* aParam, uint32_t aBlockingReason,
-                              nsIHttpChannel* aCreatingChannel) {
+                              nsIHttpChannel* aCreatingChannel,
+                              bool aIsWarning = false) {
   nsresult rv = NS_OK;
 
   nsCOMPtr<nsIChannel> channel = do_QueryInterface(aRequest);
 
-  NS_SetRequestBlockingReason(channel, aBlockingReason);
+  if (!aIsWarning) {
+    NS_SetRequestBlockingReason(channel, aBlockingReason);
+  }
 
   nsCOMPtr<nsIURI> aUri;
   channel->GetURI(getter_AddRefs(aUri));
@@ -93,7 +117,7 @@ static void LogBlockedRequest(nsIRequest* aRequest, const char* aProperty,
 
   if (XRE_IsParentProcess()) {
     if (aCreatingChannel) {
-      rv = aCreatingChannel->LogBlockedCORSRequest(msg, category);
+      rv = aCreatingChannel->LogBlockedCORSRequest(msg, category, aIsWarning);
       if (NS_SUCCEEDED(rv)) {
         return;
       }
@@ -130,74 +154,175 @@ static void LogBlockedRequest(nsIRequest* aRequest, const char* aProperty,
     }
   }
   nsCORSListenerProxy::LogBlockedCORSRequest(innerWindowID, privateBrowsing,
-                                             fromChromeContext, msg, category);
+                                             fromChromeContext, msg, category,
+                                             aIsWarning);
 }
 
 //////////////////////////////////////////////////////////////////////////
 // Preflight cache
 
-class nsPreflightCache {
+class nsPreflightCache : public nsICORSPreflightCache {
  public:
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSICORSPREFLIGHTCACHE
+
   struct TokenTime {
     nsCString token;
     TimeStamp expirationTime;
   };
 
-  struct CacheEntry : public LinkedListElement<CacheEntry> {
-    explicit CacheEntry(nsCString& aKey) : mKey(aKey) {
-      MOZ_COUNT_CTOR(nsPreflightCache::CacheEntry);
-    }
-
-    ~CacheEntry() { MOZ_COUNT_DTOR(nsPreflightCache::CacheEntry); }
-
-    void PurgeExpired(TimeStamp now);
-    bool CheckRequest(const nsCString& aMethod,
-                      const nsTArray<nsCString>& aHeaders);
-
-    nsCString mKey;
-    nsTArray<TokenTime> mMethods;
-    nsTArray<TokenTime> mHeaders;
-  };
-
-  MOZ_COUNTED_DEFAULT_CTOR(nsPreflightCache)
-
-  ~nsPreflightCache() {
-    Clear();
-    MOZ_COUNT_DTOR(nsPreflightCache);
-  }
-
-  bool Initialize() { return true; }
-
-  CacheEntry* GetEntry(nsIURI* aURI, nsIPrincipal* aPrincipal,
-                       bool aWithCredentials,
-                       const OriginAttributes& aOriginAttributes, bool aCreate);
+  already_AddRefed<CORSCacheEntry> GetEntry(
+      nsIURI* aURI, nsIPrincipal* aPrincipal, bool aWithCredentials,
+      const OriginAttributes& aOriginAttributes, bool aCreate);
   void RemoveEntries(nsIURI* aURI, nsIPrincipal* aPrincipal,
                      const OriginAttributes& aOriginAttributes);
+  void PurgePrivateBrowsingEntries();
 
   void Clear();
 
+ protected:
+  virtual ~nsPreflightCache() { Clear(); }
+
  private:
-  nsClassHashtable<nsCStringHashKey, CacheEntry> mTable;
-  LinkedList<CacheEntry> mList;
+  nsRefPtrHashtable<nsCStringHashKey, CORSCacheEntry> mTable;
+  LinkedList<CORSCacheEntry> mList;
 };
 
+struct CORSCacheEntry : public LinkedListElement<CORSCacheEntry>,
+                        public nsICORSPreflightCacheEntry {
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSICORSPREFLIGHTCACHEENTRY
+  explicit CORSCacheEntry(nsIURI* aUri,
+                          const OriginAttributes& aOriginAttributes,
+                          nsIPrincipal* aPrincipal, bool withCredentials,
+                          nsCString& aKey)
+      : mURI(aUri),
+        mOA(aOriginAttributes),
+        mPrincipal(aPrincipal),
+        mWithCredentials(withCredentials),
+        mKey(aKey) {}
+
+  void PurgeExpired(TimeStamp now);
+  bool CheckRequest(const nsCString& aMethod,
+                    const nsTArray<nsCString>& aHeaders);
+
+  nsCOMPtr<nsIURI> mURI;
+  const OriginAttributes mOA;
+  nsCOMPtr<nsIPrincipal> mPrincipal;
+  bool mWithCredentials;
+  nsCString mKey;  // serialized key
+  const TimeStamp mCreationTime{TimeStamp::NowLoRes()};
+  bool mDoomed{false};
+  bool mIsProxyUsed{false};
+
+  nsTArray<nsPreflightCache::TokenTime> mMethods;
+  nsTArray<nsPreflightCache::TokenTime> mHeaders;
+
+ private:
+  virtual ~CORSCacheEntry() = default;
+
+  bool CheckDNSCache();
+};
+
+NS_IMPL_ISUPPORTS(nsPreflightCache, nsICORSPreflightCache)
+
+NS_IMETHODIMP
+nsPreflightCache::GetEntries(
+    nsIPrincipal* aPrincipal,
+    nsTArray<RefPtr<nsICORSPreflightCacheEntry>>& aEntries) {
+  for (auto iter = mTable.Iter(); !iter.Done(); iter.Next()) {
+    RefPtr<nsIPrincipal> iterPrincipal;
+    iter.Data()->GetPrincipal(getter_AddRefs(iterPrincipal));
+    if (iterPrincipal->Equals(aPrincipal)) {
+      auto* entry = iter.UserData();
+      aEntries.AppendElement(entry);
+    }
+  }
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPreflightCache::ClearEntry(nsICORSPreflightCacheEntry* entry) {
+  nsCOMPtr<nsIURI> uri;
+  nsresult rv = entry->GetURI(getter_AddRefs(uri));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIPrincipal> principal;
+  rv = entry->GetPrincipal(getter_AddRefs(principal));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  const OriginAttributes oa = entry->OriginAttributesRef();
+
+  RemoveEntries(uri, principal, oa);
+  return NS_OK;
+}
+
 // Will be initialized in EnsurePreflightCache.
-static nsPreflightCache* sPreflightCache = nullptr;
+static StaticRefPtr<nsPreflightCache> sPreflightCache;
 
 static bool EnsurePreflightCache() {
   if (sPreflightCache) return true;
 
-  UniquePtr<nsPreflightCache> newCache(new nsPreflightCache());
-
-  if (newCache->Initialize()) {
-    sPreflightCache = newCache.release();
-    return true;
-  }
-
-  return false;
+  RefPtr<nsPreflightCache> newCache(new nsPreflightCache());
+  sPreflightCache = newCache;
+  return true;
 }
 
-void nsPreflightCache::CacheEntry::PurgeExpired(TimeStamp now) {
+void nsPreflightCache::PurgePrivateBrowsingEntries() {
+  for (auto iter = mTable.Iter(); !iter.Done(); iter.Next()) {
+    auto* entry = iter.UserData();
+    if (entry->mOA.IsPrivateBrowsing()) {
+      // last private browsing window closed, remove preflight cache entries
+      entry->removeFrom(sPreflightCache->mList);
+      iter.Remove();
+    }
+  }
+}
+
+NS_IMPL_ISUPPORTS(CORSCacheEntry, nsICORSPreflightCacheEntry)
+
+NS_IMETHODIMP
+CORSCacheEntry::GetKey(nsACString& aKey) {
+  aKey = mKey;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+CORSCacheEntry::GetURI(nsIURI** aURI) {
+  *aURI = do_AddRef(mURI).take();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+CORSCacheEntry::GetOriginAttributes(JSContext* aCx,
+                                    JS::MutableHandle<JS::Value> aVal) {
+  if (NS_WARN_IF(!ToJSValue(aCx, mOA, aVal))) {
+    return NS_ERROR_FAILURE;
+  }
+  return NS_OK;
+}
+
+const OriginAttributes& CORSCacheEntry::OriginAttributesRef() { return mOA; }
+
+NS_IMETHODIMP
+CORSCacheEntry::GetPrincipal(nsIPrincipal** aPrincipal) {
+  *aPrincipal = do_AddRef(mPrincipal).take();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+CORSCacheEntry::GetPrivateBrowsing(bool* aPrivateBrowsing) {
+  *aPrivateBrowsing = mOA.IsPrivateBrowsing();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+CORSCacheEntry::GetWithCredentials(bool* aWithCredentials) {
+  *aWithCredentials = mWithCredentials;
+  return NS_OK;
+}
+
+void CORSCacheEntry::PurgeExpired(TimeStamp now) {
   for (uint32_t i = 0, len = mMethods.Length(); i < len; ++i) {
     if (now >= mMethods[i].expirationTime) {
       mMethods.UnorderedRemoveElementAt(i);
@@ -214,13 +339,60 @@ void nsPreflightCache::CacheEntry::PurgeExpired(TimeStamp now) {
   }
 }
 
-bool nsPreflightCache::CacheEntry::CheckRequest(
-    const nsCString& aMethod, const nsTArray<nsCString>& aHeaders) {
+bool CORSCacheEntry::CheckDNSCache() {
+  // When proxy is used, the DNS lookup is done by proxy, so we skip this check.
+  if (mIsProxyUsed) {
+    return true;
+  }
+
+  nsCOMPtr<nsIDNSService> dns;
+  dns = mozilla::components::DNS::Service();
+  if (!dns) {
+    return false;
+  }
+
+  nsAutoCString host;
+  if (NS_FAILED(mURI->GetAsciiHost(host))) {
+    return false;
+  }
+
+  nsCOMPtr<nsIDNSRecord> record;
+  nsresult rv = dns->ResolveNative(host, nsIDNSService::RESOLVE_OFFLINE, mOA,
+                                   getter_AddRefs(record));
+  if (NS_FAILED(rv) || !record) {
+    return false;
+  }
+
+  nsCOMPtr<nsIDNSAddrRecord> addrRec = do_QueryInterface(record);
+  if (!addrRec) {
+    return false;
+  }
+
+  TimeStamp lastUpdate;
+  Unused << addrRec->GetLastUpdate(&lastUpdate);
+
+  if (lastUpdate > mCreationTime) {
+    return false;
+  }
+
+  return true;
+}
+
+bool CORSCacheEntry::CheckRequest(const nsCString& aMethod,
+                                  const nsTArray<nsCString>& aHeaders) {
   PurgeExpired(TimeStamp::NowLoRes());
 
+  if (!CheckDNSCache()) {
+    mMethods.Clear();
+    mHeaders.Clear();
+    mDoomed = true;
+    return false;
+  }
+
   if (!aMethod.EqualsLiteral("GET") && !aMethod.EqualsLiteral("POST")) {
     struct CheckToken {
-      bool Equals(const TokenTime& e, const nsCString& method) const {
+      bool Equals(const nsPreflightCache::TokenTime& e,
+                  const nsCString& method) const {
         return e.token.Equals(method);
       }
     };
@@ -231,7 +403,8 @@ bool nsPreflightCache::CacheEntry::CheckRequest(
   }
 
   struct CheckHeaderToken {
-    bool Equals(const TokenTime& e, const nsCString& header) const {
+    bool Equals(const nsPreflightCache::TokenTime& e,
+                const nsCString& header) const {
       return e.token.Equals(header, nsCaseInsensitiveCStringComparator);
     }
   } checker;
@@ -244,26 +417,28 @@ bool nsPreflightCache::CacheEntry::CheckRequest(
   return true;
 }
 
-nsPreflightCache::CacheEntry* nsPreflightCache::GetEntry(
+already_AddRefed<CORSCacheEntry> nsPreflightCache::GetEntry(
     nsIURI* aURI, nsIPrincipal* aPrincipal, bool aWithCredentials,
     const OriginAttributes& aOriginAttributes, bool aCreate) {
-  nsCString key;
+  nsAutoCString key;
   if (NS_FAILED(aPrincipal->GetPrefLightCacheKey(aURI, aWithCredentials,
                                                  aOriginAttributes, key))) {
     NS_WARNING("Invalid cache key!");
     return nullptr;
   }
 
-  CacheEntry* existingEntry = nullptr;
-
-  if (mTable.Get(key, &existingEntry)) {
-    // Entry already existed so just return it. Also update the LRU list.
-
-    // Move to the head of the list.
-    existingEntry->removeFrom(mList);
-    mList.insertFront(existingEntry);
-
-    return existingEntry;
+  RefPtr<CORSCacheEntry> existingEntry = nullptr;
+  if ((existingEntry = mTable.Get(key))) {
+    if (existingEntry->mDoomed) {
+      existingEntry->removeFrom(mList);
+      mTable.Remove(key);
+    } else {
+      // Entry already existed so just return it. Also update the LRU list.
+      // Move to the head of the list.
+      existingEntry->removeFrom(mList);
+      mList.insertFront(existingEntry);
+      return existingEntry.forget();
+    }
   }
 
   if (!aCreate) {
@@ -272,7 +447,8 @@ nsPreflightCache::CacheEntry* nsPreflightCache::GetEntry(
 
   // This is a new entry, allocate and insert into the table now so that any
   // failures don't cause items to be removed from a full cache.
-  auto newEntry = MakeUnique<CacheEntry>(key);
+  RefPtr<CORSCacheEntry> newEntry = new CORSCacheEntry(
+      aURI, aOriginAttributes, aPrincipal, aWithCredentials, key);
 
   NS_ASSERTION(mTable.Count() <= PREFLIGHT_CACHE_SIZE,
                "Something is borked, too many entries in the cache!");
@@ -295,40 +471,43 @@ nsPreflightCache::CacheEntry* nsPreflightCache::GetEntry(
     // If that didn't remove anything then kick out the least recently used
     // entry.
     if (mTable.Count() == PREFLIGHT_CACHE_SIZE) {
-      CacheEntry* lruEntry = static_cast<CacheEntry*>(mList.popLast());
+      CORSCacheEntry* lruEntry = static_cast<CORSCacheEntry*>(mList.popLast());
       MOZ_ASSERT(lruEntry);
 
       // This will delete 'lruEntry'.
-      mTable.Remove(lruEntry->mKey);
+      nsAutoCString lruKey;
+      lruEntry->GetKey(lruKey);
+      mTable.Remove(lruKey);
 
       NS_ASSERTION(mTable.Count() == PREFLIGHT_CACHE_SIZE - 1,
                    "Somehow tried to remove an entry that was never added!");
     }
   }
-
-  auto* newEntryWeakRef = mTable.InsertOrUpdate(key, std::move(newEntry)).get();
-  mList.insertFront(newEntryWeakRef);
-
-  return newEntryWeakRef;
+  CORSCacheEntry* newEntryWeak = newEntry.get();
+  mTable.InsertOrUpdate(key, newEntry);
+  mList.insertFront(newEntryWeak);
+  return newEntry.forget();
 }
 
 void nsPreflightCache::RemoveEntries(
     nsIURI* aURI, nsIPrincipal* aPrincipal,
     const OriginAttributes& aOriginAttributes) {
-  CacheEntry* entry;
+  RefPtr<CORSCacheEntry> entry;
   nsCString key;
   if (NS_SUCCEEDED(aPrincipal->GetPrefLightCacheKey(aURI, true,
-                                                    aOriginAttributes, key)) &&
-      mTable.Get(key, &entry)) {
-    entry->removeFrom(mList);
-    mTable.Remove(key);
+                                                    aOriginAttributes, key))) {
+    if ((entry = mTable.Get(key))) {
+      entry->removeFrom(mList);
+      mTable.Remove(key);
+    }
   }
 
   if (NS_SUCCEEDED(aPrincipal->GetPrefLightCacheKey(aURI, false,
-                                                    aOriginAttributes, key)) &&
-      mTable.Get(key, &entry)) {
-    entry->removeFrom(mList);
-    mTable.Remove(key);
+                                                    aOriginAttributes, key))) {
+    if ((entry = mTable.Get(key))) {
+      entry->removeFrom(mList);
+      mTable.Remove(key);
+    }
   }
 }
 
@@ -345,11 +524,19 @@ NS_IMPL_ISUPPORTS(nsCORSListenerProxy, nsIStreamListener, nsIRequestObserver,
                   nsIThreadRetargetableStreamListener)
 
 /* static */
-void nsCORSListenerProxy::Shutdown() {
-  delete sPreflightCache;
-  sPreflightCache = nullptr;
+already_AddRefed<nsICORSPreflightCache>
+nsCORSListenerProxy::GetCORSPreflightSingleton() {
+  NS_ASSERTION(!IsNeckoChild(), "not a parent process");
+
+  if (!EnsurePreflightCache()) {
+    NS_ASSERTION(false, "Failed to get the preflightCache");
+  }
+  return do_AddRef(sPreflightCache);
 }
 
+/* static */
+void nsCORSListenerProxy::Shutdown() { sPreflightCache = nullptr; }
+
 /* static */
 void nsCORSListenerProxy::ClearCache() {
   if (!sPreflightCache) {
@@ -358,12 +545,39 @@ void nsCORSListenerProxy::ClearCache() {
   sPreflightCache->Clear();
 }
 
+// static
+void nsCORSListenerProxy::ClearPrivateBrowsingCache() {
+  if (!sPreflightCache) {
+    return;
+  }
+  sPreflightCache->PurgePrivateBrowsingEntries();
+}
+
+// Usually, when using an expanded principal, there's no particularly good
+// origin to do the request with. However if the expanded principal only wraps
+// one principal, we can use that one instead.
+//
+// This is needed so that DevTools can still do CORS-enabled requests (since
+// DevTools uses a triggering principal expanding the node principal to bypass
+// CSP checks, see Element::CreateDevToolsPrincipal(), bug 1604562, and bug
+// 1391994).
+static nsIPrincipal* GetOriginHeaderPrincipal(nsIPrincipal* aPrincipal) {
+  while (aPrincipal && aPrincipal->GetIsExpandedPrincipal()) {
+    auto* ep = BasePrincipal::Cast(aPrincipal)->As<ExpandedPrincipal>();
+    if (ep->AllowList().Length() != 1) {
+      break;
+    }
+    aPrincipal = ep->AllowList()[0];
+  }
+  return aPrincipal;
+}
+
 nsCORSListenerProxy::nsCORSListenerProxy(nsIStreamListener* aOuter,
                                          nsIPrincipal* aRequestingPrincipal,
                                          bool aWithCredentials)
     : mOuterListener(aOuter),
       mRequestingPrincipal(aRequestingPrincipal),
-      mOriginHeaderPrincipal(aRequestingPrincipal),
+      mOriginHeaderPrincipal(GetOriginHeaderPrincipal(aRequestingPrincipal)),
       mWithCredentials(aWithCredentials),
       mRequestApproved(false),
       mHasBeenCrossSite(false),
@@ -379,7 +593,8 @@ nsresult nsCORSListenerProxy::Init(nsIChannel* aChannel,
       getter_AddRefs(mOuterNotificationCallbacks));
   aChannel->SetNotificationCallbacks(this);
 
-  nsresult rv = UpdateChannel(aChannel, aAllowDataURI, UpdateType::Default);
+  nsresult rv =
+      UpdateChannel(aChannel, aAllowDataURI, UpdateType::Default, false);
   if (NS_FAILED(rv)) {
     {
       MutexAutoLock lock(mMutex);
@@ -497,7 +712,7 @@ nsresult nsCORSListenerProxy::CheckRequestApproved(nsIRequest* aRequest) {
   nsresult status;
   nsresult rv = aRequest->GetStatus(&status);
   if (NS_FAILED(rv)) {
-    LogBlockedRequest(aRequest, "CORSDidNotSucceed", nullptr,
+    LogBlockedRequest(aRequest, "CORSDidNotSucceed2", nullptr,
                       nsILoadInfo::BLOCKING_REASON_CORSDIDNOTSUCCEED,
                       topChannel);
     return rv;
@@ -506,7 +721,7 @@ nsresult nsCORSListenerProxy::CheckRequestApproved(nsIRequest* aRequest) {
   if (NS_FAILED(status)) {
     if (NS_BINDING_ABORTED != status) {
       // Don't want to log mere cancellation as an error.
-      LogBlockedRequest(aRequest, "CORSDidNotSucceed", nullptr,
+      LogBlockedRequest(aRequest, "CORSDidNotSucceed2", nullptr,
                         nsILoadInfo::BLOCKING_REASON_CORSDIDNOTSUCCEED,
                         topChannel);
     }
@@ -556,7 +771,8 @@ nsresult nsCORSListenerProxy::CheckRequestApproved(nsIRequest* aRequest) {
   rv = http->GetResponseHeader("Access-Control-Allow-Origin"_ns,
                                allowedOriginHeader);
   if (NS_FAILED(rv)) {
-    LogBlockedRequest(aRequest, "CORSMissingAllowOrigin", nullptr,
+    auto statusCode = GetStatusCodeAsString(http);
+    LogBlockedRequest(aRequest, "CORSMissingAllowOrigin2", statusCode.get(),
                       nsILoadInfo::BLOCKING_REASON_CORSMISSINGALLOWORIGIN,
                       topChannel);
     return rv;
@@ -578,9 +794,9 @@ nsresult nsCORSListenerProxy::CheckRequestApproved(nsIRequest* aRequest) {
   }
 
   if (mWithCredentials || !allowedOriginHeader.EqualsLiteral("*")) {
-    MOZ_ASSERT(!nsContentUtils::IsExpandedPrincipal(mOriginHeaderPrincipal));
+    MOZ_ASSERT(!mOriginHeaderPrincipal->GetIsExpandedPrincipal());
     nsAutoCString origin;
-    mOriginHeaderPrincipal->GetAsciiOrigin(origin);
+    mOriginHeaderPrincipal->GetWebExposedOriginSerialization(origin);
 
     if (!allowedOriginHeader.Equals(origin)) {
       LogBlockedRequest(
@@ -644,6 +860,25 @@ nsCORSListenerProxy::OnDataAvailable(nsIRequest* aRequest,
   return listener->OnDataAvailable(aRequest, aInputStream, aOffset, aCount);
 }
 
+NS_IMETHODIMP
+nsCORSListenerProxy::OnDataFinished(nsresult aStatus) {
+  nsCOMPtr<nsIStreamListener> listener;
+  {
+    MutexAutoLock lock(mMutex);
+    listener = mOuterListener;
+  }
+  if (!listener) {
+    return NS_ERROR_FAILURE;
+  }
+  nsCOMPtr<nsIThreadRetargetableStreamListener> retargetableListener =
+      do_QueryInterface(listener);
+  if (retargetableListener) {
+    return retargetableListener->OnDataFinished(aStatus);
+  }
+
+  return NS_OK;
+}
+
 void nsCORSListenerProxy::SetInterceptController(
     nsINetworkInterceptController* aInterceptController) {
   mInterceptController = aInterceptController;
@@ -683,7 +918,7 @@ nsCORSListenerProxy::AsyncOnChannelRedirect(
     // data URIs should have been blocked before we got to the internal
     // redirect.
     rv = UpdateChannel(aNewChannel, DataURIHandling::Allow,
-                       UpdateType::InternalOrHSTSRedirect);
+                       UpdateType::InternalOrHSTSRedirect, false);
     if (NS_FAILED(rv)) {
       NS_WARNING(
           "nsCORSListenerProxy::AsyncOnChannelRedirect: "
@@ -692,6 +927,7 @@ nsCORSListenerProxy::AsyncOnChannelRedirect(
       return rv;
     }
   } else {
+    mIsRedirect = true;
     // A real, external redirect.  Perform CORS checking on new URL.
     rv = CheckRequestApproved(aOldChannel);
     if (NS_FAILED(rv)) {
@@ -739,23 +975,25 @@ nsCORSListenerProxy::AsyncOnChannelRedirect(
         rv = NS_ERROR_OUT_OF_MEMORY;
       }
 
-      if (NS_SUCCEEDED(rv)) {
-        bool equal;
-        rv = oldChannelPrincipal->Equals(newChannelPrincipal, &equal);
-        if (NS_SUCCEEDED(rv) && !equal) {
-          // Spec says to set our source origin to a unique origin.
-          mOriginHeaderPrincipal =
-              NullPrincipal::CreateWithInheritedAttributes(oldChannelPrincipal);
-        }
-      }
-
       if (NS_FAILED(rv)) {
         aOldChannel->Cancel(rv);
         return rv;
       }
+
+      if (!oldChannelPrincipal->Equals(newChannelPrincipal)) {
+        // Spec says to set our source origin to a unique origin.
+        mOriginHeaderPrincipal =
+            NullPrincipal::CreateWithInheritedAttributes(oldChannelPrincipal);
+      }
     }
 
     bool rewriteToGET = false;
+    // We need to strip auth header from preflight request for
+    // cross-origin redirects.
+    // See Bug 1874132
+    bool stripAuthHeader =
+        NS_ShouldRemoveAuthHeaderOnRedirect(aOldChannel, aNewChannel, aFlags);
+
     nsCOMPtr<nsIHttpChannel> oldHttpChannel = do_QueryInterface(aOldChannel);
     if (oldHttpChannel) {
       nsAutoCString method;
@@ -764,9 +1002,10 @@ nsCORSListenerProxy::AsyncOnChannelRedirect(
                                                              &rewriteToGET);
     }
 
-    rv = UpdateChannel(aNewChannel, DataURIHandling::Disallow,
-                       rewriteToGET ? UpdateType::StripRequestBodyHeader
-                                    : UpdateType::Default);
+    rv = UpdateChannel(
+        aNewChannel, DataURIHandling::Disallow,
+        rewriteToGET ? UpdateType::StripRequestBodyHeader : UpdateType::Default,
+        stripAuthHeader);
     if (NS_FAILED(rv)) {
       NS_WARNING(
           "nsCORSListenerProxy::AsyncOnChannelRedirect: "
@@ -851,7 +1090,10 @@ bool CheckInsecureUpgradePreventsCORS(nsIPrincipal* aRequestingPrincipal,
 
 nsresult nsCORSListenerProxy::UpdateChannel(nsIChannel* aChannel,
                                             DataURIHandling aAllowDataURI,
-                                            UpdateType aUpdateType) {
+                                            UpdateType aUpdateType,
+                                            bool aStripAuthHeader) {
+  MOZ_ASSERT_IF(aUpdateType == UpdateType::InternalOrHSTSRedirect,
+                !aStripAuthHeader);
   nsCOMPtr<nsIURI> uri, originalURI;
   nsresult rv = NS_GetFinalChannelURI(aChannel, getter_AddRefs(uri));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -860,6 +1102,12 @@ nsresult nsCORSListenerProxy::UpdateChannel(nsIChannel* aChannel,
 
   nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
 
+  // Introduced for DevTools in order to allow overriding some requests
+  // with the content of data: URIs.
+  if (loadInfo->GetAllowInsecureRedirectToDataURI() && uri->SchemeIs("data")) {
+    return NS_OK;
+  }
+
   // exempt data URIs from the same origin check.
   if (aAllowDataURI == DataURIHandling::Allow && originalURI == uri) {
     if (uri->SchemeIs("data")) {
@@ -876,7 +1124,7 @@ nsresult nsCORSListenerProxy::UpdateChannel(nsIChannel* aChannel,
   // can't return early on failure.
   nsCOMPtr<nsIHttpChannelInternal> internal = do_QueryInterface(aChannel);
   if (internal) {
-    rv = internal->SetCorsMode(nsIHttpChannelInternal::CORS_MODE_CORS);
+    rv = internal->SetRequestMode(dom::RequestMode::Cors);
     NS_ENSURE_SUCCESS(rv, rv);
     rv = internal->SetCorsIncludeCredentials(mWithCredentials);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -935,15 +1183,22 @@ nsresult nsCORSListenerProxy::UpdateChannel(nsIChannel* aChannel,
 
   // Check if we need to do a preflight, and if so set one up. This must be
   // called once we know that the request is going, or has gone, cross-origin.
-  rv = CheckPreflightNeeded(aChannel, aUpdateType);
+  rv = CheckPreflightNeeded(aChannel, aUpdateType, aStripAuthHeader);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // It's a cross site load
   mHasBeenCrossSite = true;
 
-  nsCString userpass;
-  uri->GetUserPass(userpass);
-  NS_ENSURE_TRUE(userpass.IsEmpty(), NS_ERROR_DOM_BAD_URI);
+  if (mIsRedirect) {
+    // https://fetch.spec.whatwg.org/#http-redirect-fetch
+    // Step 9. If request’s mode is "cors", locationURL includes credentials,
+    // and request’s origin is not same origin with locationURL’s origin,
+    // then return a network error.
+
+    nsAutoCString userpass;
+    uri->GetUserPass(userpass);
+    NS_ENSURE_TRUE(userpass.IsEmpty(), NS_ERROR_DOM_BAD_URI);
+  }
 
   // If we have an expanded principal here, we'll reject the CORS request,
   // because we can't send a useful Origin header which is required for CORS.
@@ -957,7 +1212,7 @@ nsresult nsCORSListenerProxy::UpdateChannel(nsIChannel* aChannel,
 
   // Add the Origin header
   nsAutoCString origin;
-  rv = mOriginHeaderPrincipal->GetAsciiOrigin(origin);
+  rv = mOriginHeaderPrincipal->GetWebExposedOriginSerialization(origin);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIHttpChannel> http = do_QueryInterface(aChannel);
@@ -970,8 +1225,7 @@ nsresult nsCORSListenerProxy::UpdateChannel(nsIChannel* aChannel,
     }
   }
 
-  rv = http->SetRequestHeader(nsDependentCString(net::nsHttp::Origin), origin,
-                              false);
+  rv = http->SetRequestHeader(net::nsHttp::Origin.val(), origin, false);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Make cookie-less if needed. We don't need to do anything here if the
@@ -996,7 +1250,8 @@ nsresult nsCORSListenerProxy::UpdateChannel(nsIChannel* aChannel,
 }
 
 nsresult nsCORSListenerProxy::CheckPreflightNeeded(nsIChannel* aChannel,
-                                                   UpdateType aUpdateType) {
+                                                   UpdateType aUpdateType,
+                                                   bool aStripAuthHeader) {
   // If this caller isn't using AsyncOpen, or if this *is* a preflight channel,
   // then we shouldn't initiate preflight for this channel.
   nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
@@ -1054,7 +1309,8 @@ nsresult nsCORSListenerProxy::CheckPreflightNeeded(nsIChannel* aChannel,
 
   nsCOMPtr<nsIHttpChannelInternal> internal = do_QueryInterface(http);
   if (!internal) {
-    LogBlockedRequest(aChannel, "CORSDidNotSucceed", nullptr,
+    auto statusCode = GetStatusCodeAsString(http);
+    LogBlockedRequest(aChannel, "CORSDidNotSucceed2", statusCode.get(),
                       nsILoadInfo::BLOCKING_REASON_CORSDIDNOTSUCCEED,
                       mHttpChannel);
     return NS_ERROR_DOM_BAD_URI;
@@ -1062,7 +1318,7 @@ nsresult nsCORSListenerProxy::CheckPreflightNeeded(nsIChannel* aChannel,
 
   internal->SetCorsPreflightParameters(
       headers.IsEmpty() ? loadInfoHeaders : headers,
-      aUpdateType == UpdateType::StripRequestBodyHeader);
+      aUpdateType == UpdateType::StripRequestBodyHeader, aStripAuthHeader);
 
   return NS_OK;
 }
@@ -1158,12 +1414,18 @@ void nsCORSPreflightListener::AddResultToCache(nsIRequest* aRequest) {
   OriginAttributes attrs;
   StoragePrincipalHelper::GetOriginAttributesForNetworkState(http, attrs);
 
-  nsPreflightCache::CacheEntry* entry = sPreflightCache->GetEntry(
+  RefPtr<CORSCacheEntry> entry = sPreflightCache->GetEntry(
       uri, mReferrerPrincipal, mWithCredentials, attrs, true);
   if (!entry) {
     return;
   }
 
+  nsCOMPtr<nsIHttpChannelInternal> httpChannelInternal(
+      do_QueryInterface(aRequest));
+  if (httpChannelInternal) {
+    Unused << httpChannelInternal->GetIsProxyUsed(&entry->mIsProxyUsed);
+  }
+
   // The "Access-Control-Allow-Methods" header contains a comma separated
   // list of method names.
   Unused << http->GetResponseHeader("Access-Control-Allow-Methods"_ns,
@@ -1297,7 +1559,8 @@ nsresult nsCORSPreflightListener::CheckPreflightRequestApproved(
   bool succeedded;
   rv = http->GetRequestSucceeded(&succeedded);
   if (NS_FAILED(rv) || !succeedded) {
-    LogBlockedRequest(aRequest, "CORSPreflightDidNotSucceed2", nullptr,
+    auto statusCode = GetStatusCodeAsString(http);
+    LogBlockedRequest(aRequest, "CORSPreflightDidNotSucceed3", statusCode.get(),
                       nsILoadInfo::BLOCKING_REASON_CORSPREFLIGHTDIDNOTSUCCEED,
                       parentHttpChannel);
     return NS_ERROR_DOM_BAD_URI;
@@ -1342,7 +1605,8 @@ nsresult nsCORSPreflightListener::CheckPreflightRequestApproved(
   Unused << http->GetResponseHeader("Access-Control-Allow-Headers"_ns,
                                     headerVal);
   nsTArray<nsCString> headers;
-  bool allowAllHeaders = false;
+  bool wildcard = false;
+  bool hasAuthorizationHeader = false;
   for (const nsACString& header :
        nsCCharSeparatedTokenizer(headerVal, ',').ToRange()) {
     if (header.IsEmpty()) {
@@ -1356,24 +1620,60 @@ nsresult nsCORSPreflightListener::CheckPreflightRequestApproved(
       return NS_ERROR_DOM_BAD_URI;
     }
     if (header.EqualsLiteral("*") && !mWithCredentials) {
-      allowAllHeaders = true;
+      wildcard = true;
     } else {
       headers.AppendElement(header);
     }
+
+    if (header.LowerCaseEqualsASCII("authorization")) {
+      hasAuthorizationHeader = true;
+    }
   }
 
-  if (!allowAllHeaders) {
-    for (uint32_t i = 0; i < mPreflightHeaders.Length(); ++i) {
-      const auto& comparator = nsCaseInsensitiveCStringArrayComparator();
-      if (!headers.Contains(mPreflightHeaders[i], comparator)) {
-        LogBlockedRequest(
-            aRequest, "CORSMissingAllowHeaderFromPreflight2",
-            NS_ConvertUTF8toUTF16(mPreflightHeaders[i]).get(),
-            nsILoadInfo::BLOCKING_REASON_CORSMISSINGALLOWHEADERFROMPREFLIGHT,
-            parentHttpChannel);
-        return NS_ERROR_DOM_BAD_URI;
+  bool authorizationInPreflightHeaders = false;
+  bool authorizationCoveredByWildcard = false;
+  for (uint32_t i = 0; i < mPreflightHeaders.Length(); ++i) {
+    // Cache the result of the authorization header.
+    bool isAuthorization =
+        mPreflightHeaders[i].LowerCaseEqualsASCII("authorization");
+    if (wildcard) {
+      if (!isAuthorization) {
+        continue;
+      } else {
+        authorizationInPreflightHeaders = true;
+        if (StaticPrefs::
+                network_cors_preflight_authorization_covered_by_wildcard() &&
+            !hasAuthorizationHeader) {
+          // When `Access-Control-Allow-Headers` is `*` and there is no
+          // `Authorization` header listed, we send a deprecation warning to the
+          // console.
+          LogBlockedRequest(aRequest, "CORSAllowHeaderFromPreflightDeprecation",
+                            nullptr, 0, parentHttpChannel, true);
+          glean::network::cors_authorization_header
+              .Get("covered_by_wildcard"_ns)
+              .Add(1);
+          authorizationCoveredByWildcard = true;
+          continue;
+        }
       }
     }
+
+    const auto& comparator = nsCaseInsensitiveCStringArrayComparator();
+    if (!headers.Contains(mPreflightHeaders[i], comparator)) {
+      LogBlockedRequest(
+          aRequest, "CORSMissingAllowHeaderFromPreflight2",
+          NS_ConvertUTF8toUTF16(mPreflightHeaders[i]).get(),
+          nsILoadInfo::BLOCKING_REASON_CORSMISSINGALLOWHEADERFROMPREFLIGHT,
+          parentHttpChannel);
+      if (isAuthorization) {
+        glean::network::cors_authorization_header.Get("disallowed"_ns).Add(1);
+      }
+      return NS_ERROR_DOM_BAD_URI;
+    }
+  }
+
+  if (authorizationInPreflightHeaders && !authorizationCoveredByWildcard) {
+    glean::network::cors_authorization_header.Get("allowed"_ns).Add(1);
   }
 
   return NS_OK;
@@ -1435,10 +1735,14 @@ nsresult nsCORSListenerProxy::StartCORSPreflight(
   bool withCredentials =
       originalLoadInfo->GetCookiePolicy() == nsILoadInfo::SEC_COOKIES_INCLUDE;
 
-  nsPreflightCache::CacheEntry* entry = nullptr;
+  RefPtr<CORSCacheEntry> entry;
 
-  // Disable cache if devtools says so.
-  bool disableCache = Preferences::GetBool("devtools.cache.disabled");
+  nsLoadFlags loadFlags;
+  rv = aRequestChannel->GetLoadFlags(&loadFlags);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Disable preflight cache if devtools says so or on other force reloads
+  bool disableCache = (loadFlags & nsIRequest::LOAD_BYPASS_CACHE);
 
   if (sPreflightCache && !disableCache) {
     OriginAttributes attrs;
@@ -1474,10 +1778,6 @@ nsresult nsCORSListenerProxy::StartCORSPreflight(
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsILoadContext> loadContext = do_GetInterface(callbacks);
 
-  nsLoadFlags loadFlags;
-  rv = aRequestChannel->GetLoadFlags(&loadFlags);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   // Preflight requests should never be intercepted by service workers and
   // are always anonymous.
   // NOTE: We ignore CORS checks on synthesized responses (see the CORS
@@ -1572,16 +1872,14 @@ nsresult nsCORSListenerProxy::StartCORSPreflight(
 }
 
 // static
-void nsCORSListenerProxy::LogBlockedCORSRequest(uint64_t aInnerWindowID,
-                                                bool aPrivateBrowsing,
-                                                bool aFromChromeContext,
-                                                const nsAString& aMessage,
-                                                const nsACString& aCategory) {
+void nsCORSListenerProxy::LogBlockedCORSRequest(
+    uint64_t aInnerWindowID, bool aPrivateBrowsing, bool aFromChromeContext,
+    const nsAString& aMessage, const nsACString& aCategory, bool aIsWarning) {
   nsresult rv = NS_OK;
 
   // Build the error object and log it to the console
   nsCOMPtr<nsIConsoleService> console(
-      do_GetService(NS_CONSOLESERVICE_CONTRACTID, &rv));
+      mozilla::components::Console::Service(&rv));
   if (NS_FAILED(rv)) {
     NS_WARNING("Failed to log blocked cross-site request (no console)");
     return;
@@ -1594,25 +1892,24 @@ void nsCORSListenerProxy::LogBlockedCORSRequest(uint64_t aInnerWindowID,
     return;
   }
 
+  uint32_t errorFlag =
+      aIsWarning ? nsIScriptError::warningFlag : nsIScriptError::errorFlag;
+
   // query innerWindowID and log to web console, otherwise log to
   // the error to the browser console.
   if (aInnerWindowID > 0) {
     rv = scriptError->InitWithSanitizedSource(aMessage,
-                                              u""_ns,  // sourceName
-                                              u""_ns,  // sourceLine
-                                              0,       // lineNumber
-                                              0,       // columnNumber
-                                              nsIScriptError::errorFlag,
-                                              aCategory, aInnerWindowID);
+                                              ""_ns,  // sourceName
+                                              0,      // lineNumber
+                                              0,      // columnNumber
+                                              errorFlag, aCategory,
+                                              aInnerWindowID);
   } else {
-    nsCString category = PromiseFlatCString(aCategory);
     rv = scriptError->Init(aMessage,
-                           u""_ns,  // sourceName
-                           u""_ns,  // sourceLine
-                           0,       // lineNumber
-                           0,       // columnNumber
-                           nsIScriptError::errorFlag, category.get(),
-                           aPrivateBrowsing,
+                           ""_ns,  // sourceName
+                           0,      // lineNumber
+                           0,      // columnNumber
+                           errorFlag, aCategory, aPrivateBrowsing,
                            aFromChromeContext);  // From chrome context
   }
   if (NS_FAILED(rv)) {
