@@ -27,11 +27,11 @@
 
 class nsIHttpUpgradeListener;
 
-namespace mozilla {
-namespace net {
+namespace mozilla::net {
 class EventTokenBucket;
 class NullHttpTransaction;
 struct HttpRetParams;
+struct Http3ConnectionStatsParams;
 
 //-----------------------------------------------------------------------------
 
@@ -40,11 +40,13 @@ class nsHttpConnectionMgr;
 using nsConnEventHandler = void (nsHttpConnectionMgr::*)(int32_t, ARefBase*);
 
 class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
-                                  public nsIObserver {
+                                  public nsIObserver,
+                                  nsINamed {
  public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_HTTPCONNECTIONMGRSHELL
   NS_DECL_NSIOBSERVER
+  NS_DECL_NSINAMED
 
   //-------------------------------------------------------------------------
   // NOTE: functions below may only be called on the main thread.
@@ -59,6 +61,12 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   [[nodiscard]] nsresult CancelTransactions(nsHttpConnectionInfo*,
                                             nsresult code);
 
+  // The connection manager needs to know the hashes used for a WebTransport
+  // connection authenticated with serverCertHashes
+  nsresult StoreServerCertHashes(
+      nsHttpConnectionInfo* aConnInfo, bool aNoSpdy, bool aNoHttp3,
+      nsTArray<RefPtr<nsIWebTransportHash>>&& aServerCertHashes);
+
   //-------------------------------------------------------------------------
   // NOTE: functions below may be called only on the socket thread.
   //-------------------------------------------------------------------------
@@ -71,7 +79,8 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
 
   // Remove a transaction from the pendingQ of it's connection entry. Returns
   // true if the transaction is removed successfully, otherwise returns false.
-  bool RemoveTransFromConnEntry(nsHttpTransaction* aTrans);
+  bool RemoveTransFromConnEntry(nsHttpTransaction* aTrans,
+                                const nsACString& aHashKey);
 
   // Directly dispatch the transaction or insert it in to the pendingQ.
   [[nodiscard]] nsresult ProcessNewTransaction(nsHttpTransaction* aTrans);
@@ -82,14 +91,19 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   [[nodiscard]] nsresult CloseIdleConnection(nsHttpConnection*);
   [[nodiscard]] nsresult RemoveIdleConnection(nsHttpConnection*);
 
+  // Close a single connection and prevent it from being reused.
+  [[nodiscard]] nsresult DoSingleConnectionCleanup(nsHttpConnectionInfo*);
+
   // The connection manager needs to know when a normal HTTP connection has been
   // upgraded to SPDY because the dispatch and idle semantics are a little
   // bit different.
-  void ReportSpdyConnection(nsHttpConnection*, bool usingSpdy);
+  void ReportSpdyConnection(nsHttpConnection*, bool usingSpdy,
+                            bool disallowHttp3);
 
   void ReportHttp3Connection(HttpConnectionBase*);
 
   bool GetConnectionData(nsTArray<HttpRetParams>*);
+  bool GetHttp3ConnectionStatsData(nsTArray<Http3ConnectionStatsParams>*);
 
   void ResetIPFamilyPreference(nsHttpConnectionInfo*);
 
@@ -120,9 +134,7 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   // NOTE: relatively expensive to call, there are two hashtable lookups.
   bool IsConnEntryUnderPressure(nsHttpConnectionInfo*);
 
-  uint64_t CurrentTopBrowsingContextId() {
-    return mCurrentTopBrowsingContextId;
-  }
+  uint64_t CurrentBrowserId() { return mCurrentBrowserId; }
 
   void DoFallbackConnection(SpeculativeTransaction* aTrans, bool aFetchHTTPSRR);
   void DoSpeculativeConnection(SpeculativeTransaction* aTrans,
@@ -139,6 +151,9 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   void NewIdleConnectionAdded(uint32_t timeToLive);
   void DecrementNumIdleConns();
 
+  const nsTArray<RefPtr<nsIWebTransportHash>>* GetServerCertHashes(
+      nsHttpConnectionInfo* aConnInfo);
+
  private:
   virtual ~nsHttpConnectionMgr();
 
@@ -181,13 +196,17 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
       const nsHttpConnectionInfo* ci);
 
  public:
-  static nsAHttpConnection* MakeConnectionHandle(HttpConnectionBase* aWrapped);
   void RegisterOriginCoalescingKey(HttpConnectionBase*, const nsACString& host,
                                    int32_t port);
   // A test if be-conservative should be used when proxy is setup for the
   // connection
   bool BeConservativeIfProxied(nsIProxyInfo* proxy);
 
+  bool AllowToRetryDifferentIPFamilyForHttp3(nsHttpConnectionInfo* ci,
+                                             nsresult aError);
+  void SetRetryDifferentIPFamilyForHttp3(nsHttpConnectionInfo* ci,
+                                         uint16_t aIPFamily);
+
  protected:
   friend class ConnectionEntry;
   void IncrementActiveConnCount();
@@ -202,8 +221,16 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   //-------------------------------------------------------------------------
 
   ReentrantMonitor mReentrantMonitor{"nsHttpConnectionMgr.mReentrantMonitor"};
-  nsCOMPtr<nsIEventTarget> mSocketThreadTarget;
+  // This is used as a flag that we're shut down, and no new events should be
+  // dispatched.
+  nsCOMPtr<nsIEventTarget> mSocketThreadTarget
+      MOZ_GUARDED_BY(mReentrantMonitor);
 
+  Atomic<bool, mozilla::Relaxed> mIsShuttingDown{false};
+
+  //-------------------------------------------------------------------------
+  // NOTE: these members are only accessed on the socket transport thread
+  //-------------------------------------------------------------------------
   // connection limits
   uint16_t mMaxUrgentExcessiveConns{0};
   uint16_t mMaxConns{0};
@@ -211,19 +238,11 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   uint16_t mMaxPersistConnsPerProxy{0};
   uint16_t mMaxRequestDelay{0};  // in seconds
   bool mThrottleEnabled{false};
-  uint32_t mThrottleVersion{2};
   uint32_t mThrottleSuspendFor{0};
   uint32_t mThrottleResumeFor{0};
-  uint32_t mThrottleReadLimit{0};
-  uint32_t mThrottleReadInterval{0};
   uint32_t mThrottleHoldTime{0};
   TimeDuration mThrottleMaxTime;
   bool mBeConservativeForProxy{true};
-  Atomic<bool, mozilla::Relaxed> mIsShuttingDown{false};
-
-  //-------------------------------------------------------------------------
-  // NOTE: these members are only accessed on the socket transport thread
-  //-------------------------------------------------------------------------
 
   [[nodiscard]] bool ProcessPendingQForEntry(ConnectionEntry*,
                                              bool considerAll);
@@ -232,8 +251,8 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
 
   // This function selects transactions from mPendingTransactionTable to
   // dispatch according to the following conditions:
-  // 1. When ActiveTabPriority() is false, only get transactions from the
-  //    queue whose window id is 0.
+  // 1. When network.http.active_tab_priority is false, only get transactions
+  //    from the queue whose window id is 0.
   // 2. If |considerAll| is false, either get transactions from the focused
   //    window queue or non-focused ones.
   // 3. If |considerAll| is true, fill the |pendingQ| with the transactions from
@@ -268,7 +287,8 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
 
   ConnectionEntry* GetOrCreateConnectionEntry(
       nsHttpConnectionInfo*, bool prohibitWildCard, bool aNoHttp2,
-      bool aNoHttp3, bool* aAvailableForDispatchNow = nullptr);
+      bool aNoHttp3, bool* aIsWildcard,
+      bool* aAvailableForDispatchNow = nullptr);
 
   [[nodiscard]] nsresult MakeNewConnection(
       ConnectionEntry* ent, PendingTransactionInfo* pendingTransInfo);
@@ -286,7 +306,7 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
                                                          bool aNoHttp2,
                                                          bool aNoHttp3);
   void UpdateCoalescingForNewConn(HttpConnectionBase* conn,
-                                  ConnectionEntry* ent);
+                                  ConnectionEntry* ent, bool aNoHttp3);
 
   void ProcessSpdyPendingQ(ConnectionEntry* ent);
   void DispatchSpdyPendingQ(nsTArray<RefPtr<PendingTransactionInfo>>& pendingQ,
@@ -305,7 +325,7 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   void OnMsgNewTransaction(int32_t, ARefBase*);
   void OnMsgNewTransactionWithStickyConn(int32_t, ARefBase*);
   void OnMsgReschedTransaction(int32_t, ARefBase*);
-  void OnMsgUpdateClassOfServiceOnTransaction(int32_t, ARefBase*);
+  void OnMsgUpdateClassOfServiceOnTransaction(ClassOfService, ARefBase*);
   void OnMsgCancelTransaction(int32_t, ARefBase*);
   void OnMsgCancelTransactions(int32_t, ARefBase*);
   void OnMsgProcessPendingQ(int32_t, ARefBase*);
@@ -314,13 +334,15 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   void OnMsgCompleteUpgrade(int32_t, ARefBase*);
   void OnMsgUpdateParam(int32_t, ARefBase*);
   void OnMsgDoShiftReloadConnectionCleanup(int32_t, ARefBase*);
+  void OnMsgDoSingleConnectionCleanup(int32_t, ARefBase*);
   void OnMsgProcessFeedback(int32_t, ARefBase*);
   void OnMsgProcessAllSpdyPendingQ(int32_t, ARefBase*);
   void OnMsgUpdateRequestTokenBucket(int32_t, ARefBase*);
   void OnMsgVerifyTraffic(int32_t, ARefBase*);
   void OnMsgPruneNoTraffic(int32_t, ARefBase*);
-  void OnMsgUpdateCurrentTopBrowsingContextId(int32_t, ARefBase*);
+  void OnMsgUpdateCurrentBrowserId(int32_t, ARefBase*);
   void OnMsgClearConnectionHistory(int32_t, ARefBase*);
+  void OnMsgStoreServerCertHashes(int32_t, ARefBase*);
 
   // Total number of active connections in all of the ConnectionEntry objects
   // that are accessed from mCT connection table.
@@ -366,7 +388,7 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   void OnMsgPrintDiagnostics(int32_t, ARefBase*);
 
   nsCString mLogData;
-  uint64_t mCurrentTopBrowsingContextId{0};
+  uint64_t mCurrentBrowserId{0};
 
   // Called on a pref change
   void SetThrottlingEnabled(bool aEnable);
@@ -377,7 +399,7 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
   // these methods track this time.
   bool InThrottlingTimeWindow();
 
-  // Two hashtalbes keeping track of active transactions regarding window id and
+  // Two hashtables keeping track of active transactions regarding window id and
   // throttling. Used by the throttling algorithm to obtain number of
   // transactions for the active tab and for inactive tabs according their
   // throttle status. mActiveTransactions[0] are all unthrottled transactions,
@@ -443,13 +465,14 @@ class nsHttpConnectionMgr final : public HttpConnectionMgrShell,
 
   // When current active tab is changed, this function uses
   // |previousId| to select background transactions and
-  // |mCurrentTopBrowsingContextId| to select foreground transactions.
+  // |mCurrentBrowserId| to select foreground transactions.
   // Then, it notifies selected transactions' connection of the new active tab
   // id.
-  void NotifyConnectionOfBrowsingContextIdChange(uint64_t previousId);
+  void NotifyConnectionOfBrowserIdChange(uint64_t previousId);
+
+  void CheckTransInPendingQueue(nsHttpTransaction* aTrans);
 };
 
-}  // namespace net
-}  // namespace mozilla
+}  // namespace mozilla::net
 
 #endif  // !nsHttpConnectionMgr_h__
