@@ -7,12 +7,14 @@
 // HttpLog.h should generally be included first
 #include "HttpLog.h"
 
+#include "mozilla/dom/MimeType.h"
+#include "mozilla/StaticPrefs_network.h"
+#include "mozilla/TextUtils.h"
 #include "mozilla/Unused.h"
 #include "nsHttpResponseHead.h"
 #include "nsIHttpHeaderVisitor.h"
 #include "nsPrintfCString.h"
 #include "prtime.h"
-#include "plstr.h"
 #include "nsCRT.h"
 #include "nsURLHelper.h"
 #include "CacheControlParser.h"
@@ -81,7 +83,34 @@ nsHttpResponseHead& nsHttpResponseHead::operator=(
   return *this;
 }
 
-HttpVersion nsHttpResponseHead::Version() {
+nsHttpResponseHead::nsHttpResponseHead(nsHttpResponseHead&& aOther) {
+  nsHttpResponseHead& other = aOther;
+  RecursiveMutexAutoLock monitor(other.mRecursiveMutex);
+
+  mHeaders = std::move(other.mHeaders);
+  mVersion = std::move(other.mVersion);
+  mStatus = std::move(other.mStatus);
+  mStatusText = std::move(other.mStatusText);
+  mContentLength = std::move(other.mContentLength);
+  mContentType = std::move(other.mContentType);
+  mContentCharset = std::move(other.mContentCharset);
+  mHasCacheControl = std::move(other.mHasCacheControl);
+  mCacheControlPublic = std::move(other.mCacheControlPublic);
+  mCacheControlPrivate = std::move(other.mCacheControlPrivate);
+  mCacheControlNoStore = std::move(other.mCacheControlNoStore);
+  mCacheControlNoCache = std::move(other.mCacheControlNoCache);
+  mCacheControlImmutable = std::move(other.mCacheControlImmutable);
+  mCacheControlStaleWhileRevalidateSet =
+      std::move(other.mCacheControlStaleWhileRevalidateSet);
+  mCacheControlStaleWhileRevalidate =
+      std::move(other.mCacheControlStaleWhileRevalidate);
+  mCacheControlMaxAgeSet = std::move(other.mCacheControlMaxAgeSet);
+  mCacheControlMaxAge = std::move(other.mCacheControlMaxAge);
+  mPragmaNoCache = std::move(other.mPragmaNoCache);
+  mInVisitHeaders = false;
+}
+
+HttpVersion nsHttpResponseHead::Version() const {
   RecursiveMutexAutoLock monitor(mRecursiveMutex);
   return mVersion;
 }
@@ -145,7 +174,7 @@ nsresult nsHttpResponseHead::SetHeader(const nsACString& hdr,
   }
 
   nsHttpAtom atom = nsHttp::ResolveAtom(hdr);
-  if (!atom) {
+  if (!atom.get()) {
     NS_WARNING("failed to resolve atom");
     return NS_ERROR_NOT_AVAILABLE;
   }
@@ -183,7 +212,8 @@ nsresult nsHttpResponseHead::SetHeader_locked(const nsHttpAtom& atom,
   return NS_OK;
 }
 
-nsresult nsHttpResponseHead::GetHeader(const nsHttpAtom& h, nsACString& v) {
+nsresult nsHttpResponseHead::GetHeader(const nsHttpAtom& h,
+                                       nsACString& v) const {
   v.Truncate();
   RecursiveMutexAutoLock monitor(mRecursiveMutex);
   return mHeaders.GetHeader(h, v);
@@ -270,7 +300,7 @@ nsresult nsHttpResponseHead::ParseCachedHead(const char* block) {
   // this command works on a buffer as prepared by Flatten, as such it is
   // not very forgiving ;-)
 
-  char* p = PL_strstr(block, "\r\n");
+  const char* p = strstr(block, "\r\n");
   if (!p) return NS_ERROR_UNEXPECTED;
 
   ParseStatusLine_locked(nsDependentCSubstring(block, p - block));
@@ -280,7 +310,7 @@ nsresult nsHttpResponseHead::ParseCachedHead(const char* block) {
 
     if (*block == 0) break;
 
-    p = PL_strstr(block, "\r\n");
+    p = strstr(block, "\r\n");
     if (!p) return NS_ERROR_UNEXPECTED;
 
     Unused << ParseHeaderLine_locked(nsDependentCSubstring(block, p - block),
@@ -313,7 +343,7 @@ nsresult nsHttpResponseHead::ParseCachedOriginalHeaders(char* block) {
 
     if (*block == 0) break;
 
-    p = PL_strstr(block, "\r\n");
+    p = strstr(block, "\r\n");
     if (!p) return NS_ERROR_UNEXPECTED;
 
     *p = 0;
@@ -347,236 +377,66 @@ void nsHttpResponseHead::AssignDefaultStatusText() {
   // In particular, HTTP/2 does not use reason phrases at all so they need to
   // always be injected.
 
-  switch (mStatus) {
-      // start with the most common
-    case 200:
-      mStatusText.AssignLiteral("OK");
-      break;
-    case 404:
-      mStatusText.AssignLiteral("Not Found");
-      break;
-    case 301:
-      mStatusText.AssignLiteral("Moved Permanently");
-      break;
-    case 304:
-      mStatusText.AssignLiteral("Not Modified");
-      break;
-    case 307:
-      mStatusText.AssignLiteral("Temporary Redirect");
-      break;
-    case 500:
-      mStatusText.AssignLiteral("Internal Server Error");
-      break;
-
-      // also well known
-    case 100:
-      mStatusText.AssignLiteral("Continue");
-      break;
-    case 101:
-      mStatusText.AssignLiteral("Switching Protocols");
-      break;
-    case 201:
-      mStatusText.AssignLiteral("Created");
-      break;
-    case 202:
-      mStatusText.AssignLiteral("Accepted");
-      break;
-    case 203:
-      mStatusText.AssignLiteral("Non Authoritative");
-      break;
-    case 204:
-      mStatusText.AssignLiteral("No Content");
-      break;
-    case 205:
-      mStatusText.AssignLiteral("Reset Content");
-      break;
-    case 206:
-      mStatusText.AssignLiteral("Partial Content");
-      break;
-    case 207:
-      mStatusText.AssignLiteral("Multi-Status");
-      break;
-    case 208:
-      mStatusText.AssignLiteral("Already Reported");
-      break;
-    case 300:
-      mStatusText.AssignLiteral("Multiple Choices");
-      break;
-    case 302:
-      mStatusText.AssignLiteral("Found");
-      break;
-    case 303:
-      mStatusText.AssignLiteral("See Other");
-      break;
-    case 305:
-      mStatusText.AssignLiteral("Use Proxy");
-      break;
-    case 308:
-      mStatusText.AssignLiteral("Permanent Redirect");
-      break;
-    case 400:
-      mStatusText.AssignLiteral("Bad Request");
-      break;
-    case 401:
-      mStatusText.AssignLiteral("Unauthorized");
-      break;
-    case 402:
-      mStatusText.AssignLiteral("Payment Required");
-      break;
-    case 403:
-      mStatusText.AssignLiteral("Forbidden");
-      break;
-    case 405:
-      mStatusText.AssignLiteral("Method Not Allowed");
-      break;
-    case 406:
-      mStatusText.AssignLiteral("Not Acceptable");
-      break;
-    case 407:
-      mStatusText.AssignLiteral("Proxy Authentication Required");
-      break;
-    case 408:
-      mStatusText.AssignLiteral("Request Timeout");
-      break;
-    case 409:
-      mStatusText.AssignLiteral("Conflict");
-      break;
-    case 410:
-      mStatusText.AssignLiteral("Gone");
-      break;
-    case 411:
-      mStatusText.AssignLiteral("Length Required");
-      break;
-    case 412:
-      mStatusText.AssignLiteral("Precondition Failed");
-      break;
-    case 413:
-      mStatusText.AssignLiteral("Request Entity Too Large");
-      break;
-    case 414:
-      mStatusText.AssignLiteral("Request URI Too Long");
-      break;
-    case 415:
-      mStatusText.AssignLiteral("Unsupported Media Type");
-      break;
-    case 416:
-      mStatusText.AssignLiteral("Requested Range Not Satisfiable");
-      break;
-    case 417:
-      mStatusText.AssignLiteral("Expectation Failed");
-      break;
-    case 418:
-      mStatusText.AssignLiteral("I'm a teapot");
-      break;
-    case 421:
-      mStatusText.AssignLiteral("Misdirected Request");
-      break;
-    case 422:
-      mStatusText.AssignLiteral("Unprocessable Entity");
-      break;
-    case 423:
-      mStatusText.AssignLiteral("Locked");
-      break;
-    case 424:
-      mStatusText.AssignLiteral("Failed Dependency");
-      break;
-    case 425:
-      mStatusText.AssignLiteral("Too Early");
-      break;
-    case 426:
-      mStatusText.AssignLiteral("Upgrade Required");
-      break;
-    case 428:
-      mStatusText.AssignLiteral("Precondition Required");
-      break;
-    case 429:
-      mStatusText.AssignLiteral("Too Many Requests");
-      break;
-    case 431:
-      mStatusText.AssignLiteral("Request Header Fields Too Large");
-      break;
-    case 451:
-      mStatusText.AssignLiteral("Unavailable For Legal Reasons");
-      break;
-    case 501:
-      mStatusText.AssignLiteral("Not Implemented");
-      break;
-    case 502:
-      mStatusText.AssignLiteral("Bad Gateway");
-      break;
-    case 503:
-      mStatusText.AssignLiteral("Service Unavailable");
-      break;
-    case 504:
-      mStatusText.AssignLiteral("Gateway Timeout");
-      break;
-    case 505:
-      mStatusText.AssignLiteral("HTTP Version Unsupported");
-      break;
-    case 506:
-      mStatusText.AssignLiteral("Variant Also Negotiates");
-      break;
-    case 507:
-      mStatusText.AssignLiteral("Insufficient Storage ");
-      break;
-    case 508:
-      mStatusText.AssignLiteral("Loop Detected");
-      break;
-    case 510:
-      mStatusText.AssignLiteral("Not Extended");
-      break;
-    case 511:
-      mStatusText.AssignLiteral("Network Authentication Required");
-      break;
-    default:
-      mStatusText.AssignLiteral("No Reason Phrase");
-      break;
-  }
+  net_GetDefaultStatusTextForCode(mStatus, mStatusText);
 }
 
-void nsHttpResponseHead::ParseStatusLine(const nsACString& line) {
+nsresult nsHttpResponseHead::ParseStatusLine(const nsACString& line) {
   RecursiveMutexAutoLock monitor(mRecursiveMutex);
-  ParseStatusLine_locked(line);
+  return ParseStatusLine_locked(line);
 }
 
-void nsHttpResponseHead::ParseStatusLine_locked(const nsACString& line) {
+nsresult nsHttpResponseHead::ParseStatusLine_locked(const nsACString& line) {
   //
-  // Parse Status-Line:: HTTP-Version SP Status-Code SP Reason-Phrase CRLF
+  // Parse Status-Line: HTTP-Version SP Status-Code SP Reason-Phrase CRLF
   //
 
   const char* start = line.BeginReading();
   const char* end = line.EndReading();
-  const char* p = start;
 
   // HTTP-Version
   ParseVersion(start);
 
   int32_t index = line.FindChar(' ');
 
-  if ((mVersion == HttpVersion::v0_9) || (index == -1)) {
+  if (mVersion == HttpVersion::v0_9 || index == -1) {
     mStatus = 200;
     AssignDefaultStatusText();
-  } else {
-    // Status-Code
-    p += index + 1;
-    mStatus = (uint16_t)atoi(p);
-    if (mStatus == 0) {
-      LOG(("mal-formed response status; assuming status = 200\n"));
-      mStatus = 200;
-    }
-
-    // Reason-Phrase is whatever is remaining of the line
-    index = line.FindChar(' ', p - start);
-    if (index == -1) {
-      AssignDefaultStatusText();
-    } else {
-      p = start + index + 1;
-      mStatusText = nsDependentCSubstring(p, end - p);
-    }
+    LOG1(("Have status line [version=%u status=%u statusText=%s]\n",
+          unsigned(mVersion), unsigned(mStatus), mStatusText.get()));
+    return NS_OK;
   }
 
+  // Status-Code: all ASCII digits after any whitespace
+  const char* p = start + index + 1;
+  while (p < end && NS_IsHTTPWhitespace(*p)) ++p;
+  if (p == end || !mozilla::IsAsciiDigit(*p)) {
+    return NS_ERROR_PARSING_HTTP_STATUS_LINE;
+  }
+  const char* codeStart = p;
+  while (p < end && mozilla::IsAsciiDigit(*p)) ++p;
+
+  // Only accept 3 digits (including all leading zeros)
+  // Also if next char isn't whitespace, fail (ie, code is 0x2)
+  if (p - codeStart > 3 || (p < end && !NS_IsHTTPWhitespace(*p))) {
+    return NS_ERROR_PARSING_HTTP_STATUS_LINE;
+  }
+
+  // At this point the code is between 0 and 999 inclusive
+  nsDependentCSubstring strCode(codeStart, p - codeStart);
+  nsresult rv;
+  mStatus = strCode.ToInteger(&rv);
+  if (NS_FAILED(rv)) {
+    return NS_ERROR_PARSING_HTTP_STATUS_LINE;
+  }
+
+  // Reason-Phrase: whatever remains after any whitespace (even empty)
+  while (p < end && NS_IsHTTPWhitespace(*p)) ++p;
+  if (p != end) {
+    mStatusText = nsDependentCSubstring(p, end - p);
+  }
   LOG1(("Have status line [version=%u status=%u statusText=%s]\n",
         unsigned(mVersion), unsigned(mStatus), mStatusText.get()));
+  return NS_OK;
 }
 
 nsresult nsHttpResponseHead::ParseHeaderLine(const nsACString& line) {
@@ -594,6 +454,13 @@ nsresult nsHttpResponseHead::ParseHeaderLine_locked(
           line, &hdr, &headerNameOriginal, &val))) {
     return NS_OK;
   }
+
+  // reject the header if there are 0x00 bytes in the value.
+  // (see https://github.com/httpwg/http-core/issues/215 for details).
+  if (val.FindChar('\0') >= 0) {
+    return NS_ERROR_DOM_INVALID_HEADER_VALUE;
+  }
+
   nsresult rv;
   if (originalFromNetHeaders) {
     rv = mHeaders.SetHeaderFromNet(hdr, headerNameOriginal, val, true);
@@ -609,21 +476,27 @@ nsresult nsHttpResponseHead::ParseHeaderLine_locked(
 
   // handle some special case headers...
   if (hdr == nsHttp::Content_Length) {
-    int64_t len;
-    const char* ignored;
-    // permit only a single value here.
-    if (nsHttp::ParseInt64(val.get(), &ignored, &len)) {
-      mContentLength = len;
-    } else {
-      // If this is a negative content length then just ignore it
-      LOG(("invalid content-length! %s\n", val.get()));
+    rv = ParseResponseContentLength(val);
+    if (rv == NS_ERROR_ILLEGAL_VALUE) {
+      LOG(("illegal content-length! %s\n", val.get()));
+      return rv;
     }
+
+    if (rv == NS_ERROR_NOT_AVAILABLE) {
+      LOG(("content-length value ignored! %s\n", val.get()));
+    }
+
   } else if (hdr == nsHttp::Content_Type) {
-    LOG(("ParseContentType [type=%s]\n", val.get()));
-    bool dummy;
-    net_ParseContentType(val, mContentType, mContentCharset, &dummy);
+    if (StaticPrefs::network_standard_content_type_parsing_response_headers() &&
+        CMimeType::Parse(val, mContentType, mContentCharset)) {
+    } else {
+      bool dummy;
+      net_ParseContentType(val, mContentType, mContentCharset, &dummy);
+    }
+    LOG(("ParseContentType [input=%s, type=%s, charset=%s]\n", val.get(),
+         mContentType.get(), mContentCharset.get()));
   } else if (hdr == nsHttp::Cache_Control) {
-    ParseCacheControl(val.get());
+    ParseCacheControl(mHeaders.PeekHeader(hdr));
   } else if (hdr == nsHttp::Pragma) {
     ParsePragma(val.get());
   }
@@ -712,8 +585,7 @@ nsresult nsHttpResponseHead::ComputeFreshnessLifetime(uint32_t* result) {
   }
 
   // These responses can be cached indefinitely.
-  if ((mStatus == 300) || (mStatus == 410) ||
-      nsHttp::IsPermanentRedirect(mStatus)) {
+  if (mStatus == 410) {
     LOG(
         ("nsHttpResponseHead::ComputeFreshnessLifetime [this = %p] "
          "Assign an infinite heuristic lifetime\n",
@@ -903,10 +775,13 @@ void nsHttpResponseHead::UpdateHeaders(nsHttpResponseHead* aOther) {
   for (i = 0; i < count; ++i) {
     nsHttpAtom header;
     nsAutoCString headerNameOriginal;
-    const char* val =
-        aOther->mHeaders.PeekHeaderAt(i, header, headerNameOriginal);
 
-    if (!val) {
+    if (!aOther->mHeaders.PeekHeaderAt(i, header, headerNameOriginal)) {
+      continue;
+    }
+
+    nsAutoCString val;
+    if (NS_FAILED(aOther->GetHeader(header, val))) {
       continue;
     }
 
@@ -926,13 +801,13 @@ void nsHttpResponseHead::UpdateHeaders(nsHttpResponseHead* aOther) {
         // this one is for MS servers that send "Content-Length: 0"
         // on 304 responses
         header == nsHttp::Content_Length) {
-      LOG(("ignoring response header [%s: %s]\n", header.get(), val));
+      LOG(("ignoring response header [%s: %s]\n", header.get(), val.get()));
     } else {
-      LOG(("new response header [%s: %s]\n", header.get(), val));
+      LOG(("new response header [%s: %s]\n", header.get(), val.get()));
 
       // overwrite the current header value with the new value...
       DebugOnly<nsresult> rv =
-          SetHeader_locked(header, headerNameOriginal, nsDependentCString(val));
+          SetHeader_locked(header, headerNameOriginal, val);
       MOZ_ASSERT(NS_SUCCEEDED(rv));
     }
   }
@@ -1042,7 +917,8 @@ nsresult nsHttpResponseHead::GetLastModifiedValue(uint32_t* result) {
   return ParseDateHeader(nsHttp::Last_Modified, result);
 }
 
-bool nsHttpResponseHead::operator==(const nsHttpResponseHead& aOther) const {
+bool nsHttpResponseHead::operator==(const nsHttpResponseHead& aOther) const
+    MOZ_NO_THREAD_SAFETY_ANALYSIS {
   nsHttpResponseHead& curr = const_cast<nsHttpResponseHead&>(*this);
   nsHttpResponseHead& other = const_cast<nsHttpResponseHead&>(aOther);
   RecursiveMutexAutoLock monitorOther(other.mRecursiveMutex);
@@ -1209,6 +1085,57 @@ void nsHttpResponseHead::ParsePragma(const char* val) {
   mPragmaNoCache = nsHttp::FindToken(val, "no-cache", HTTP_HEADER_VALUE_SEPS);
 }
 
+nsresult nsHttpResponseHead::ParseResponseContentLength(
+    const nsACString& aHeaderStr) {
+  int64_t contentLength = 0;
+  // Ref: https://fetch.spec.whatwg.org/#content-length-header
+  // Step 1. Let values be the result of getting, decoding, and splitting
+  // `Content - Length` from headers.
+  //  Step 1 is done by the caller
+  //  Step 2. If values is null, then return null.
+  if (aHeaderStr.IsEmpty()) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  // Step 3 Let candidateValue be null.
+  Maybe<nsAutoCString> candidateValue;
+  // Step 4 For each value of values
+  for (const nsACString& token :
+       nsCCharSeparatedTokenizerTemplate<
+           NS_IsAsciiWhitespace, nsTokenizerFlags::IncludeEmptyTokenAtEnd>(
+           aHeaderStr, ',')
+           .ToRange()) {
+    // Step 4.1 If candidateValue is null, then set candidateValue to value.
+    if (candidateValue.isNothing()) {
+      candidateValue.emplace(token);
+    }
+    // Step 4.2 Otherwise, if value is not candidateValue, return failure.
+    if (candidateValue.value() != token) {
+      return NS_ERROR_ILLEGAL_VALUE;
+    }
+  }
+  // Step 5 If candidateValue is the empty string or has a code point that is
+  // not an ASCII digit, then return null.
+  if (candidateValue.isNothing()) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  // Step 6 Return candidateValue, interpreted as decimal number contentLength
+  const char* end = nullptr;
+  if (!net::nsHttp::ParseInt64(candidateValue->get(), &end, &contentLength)) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  if (*end != '\0') {
+    // a number was parsed by ParseInt64 but candidateValue contains non-numeric
+    // characters
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  mContentLength = contentLength;
+  return NS_OK;
+}
+
 nsresult nsHttpResponseHead::VisitHeaders(
     nsIHttpHeaderVisitor* visitor, nsHttpHeaderArray::VisitorFilter filter) {
   RecursiveMutexAutoLock monitor(mRecursiveMutex);
@@ -1218,6 +1145,38 @@ nsresult nsHttpResponseHead::VisitHeaders(
   return rv;
 }
 
+namespace {
+class ContentTypeOptionsVisitor final : public nsIHttpHeaderVisitor {
+ public:
+  NS_DECL_ISUPPORTS
+
+  ContentTypeOptionsVisitor() = default;
+
+  NS_IMETHOD
+  VisitHeader(const nsACString& aHeader, const nsACString& aValue) override {
+    if (!mHeaderPresent) {
+      mHeaderPresent = true;
+    } else {
+      // multiple XCTO headers in response, merge them
+      mContentTypeOptionsHeader.Append(", "_ns);
+    }
+    mContentTypeOptionsHeader.Append(aValue);
+    return NS_OK;
+  }
+
+  void GetMergedHeader(nsACString& aValue) {
+    aValue = mContentTypeOptionsHeader;
+  }
+
+ private:
+  ~ContentTypeOptionsVisitor() = default;
+  bool mHeaderPresent{false};
+  nsAutoCString mContentTypeOptionsHeader;
+};
+
+NS_IMPL_ISUPPORTS(ContentTypeOptionsVisitor, nsIHttpHeaderVisitor)
+}  // namespace
+
 nsresult nsHttpResponseHead::GetOriginalHeader(const nsHttpAtom& aHeader,
                                                nsIHttpHeaderVisitor* aVisitor) {
   RecursiveMutexAutoLock monitor(mRecursiveMutex);
@@ -1241,7 +1200,11 @@ bool nsHttpResponseHead::GetContentTypeOptionsHeader(nsACString& aOutput) {
   aOutput.Truncate();
 
   nsAutoCString contentTypeOptionsHeader;
-  Unused << GetHeader(nsHttp::X_Content_Type_Options, contentTypeOptionsHeader);
+  // We need to fetch original headers and manually merge them because empty
+  // header values are not retrieved with GetHeader. Ref - Bug 1819642
+  RefPtr<ContentTypeOptionsVisitor> visitor = new ContentTypeOptionsVisitor();
+  Unused << GetOriginalHeader(nsHttp::X_Content_Type_Options, visitor);
+  visitor->GetMergedHeader(contentTypeOptionsHeader);
   if (contentTypeOptionsHeader.IsEmpty()) {
     // if there is no XCTO header, then there is nothing to do.
     return false;
@@ -1251,7 +1214,7 @@ bool nsHttpResponseHead::GetContentTypeOptionsHeader(nsACString& aOutput) {
   // a) let's skip all subsequent values
   //     e.g. "   NoSniFF   , foo " will be "   NoSniFF   "
   int32_t idx = contentTypeOptionsHeader.Find(",");
-  if (idx > 0) {
+  if (idx >= 0) {
     contentTypeOptionsHeader = Substring(contentTypeOptionsHeader, 0, idx);
   }
   // b) let's trim all surrounding whitespace
