@@ -41,12 +41,18 @@ class nsHttpResponseHead {
   nsHttpResponseHead() = default;
 
   nsHttpResponseHead(const nsHttpResponseHead& aOther);
+  nsHttpResponseHead(nsHttpResponseHead&& aOther);
   nsHttpResponseHead& operator=(const nsHttpResponseHead& aOther);
 
-  void Enter() { mRecursiveMutex.Lock(); }
-  void Exit() { mRecursiveMutex.Unlock(); }
+  void Enter() const MOZ_CAPABILITY_ACQUIRE(mRecursiveMutex) {
+    mRecursiveMutex.Lock();
+  }
+  void Exit() const MOZ_CAPABILITY_RELEASE(mRecursiveMutex) {
+    mRecursiveMutex.Unlock();
+  }
+  void AssertMutexOwned() const { mRecursiveMutex.AssertCurrentThreadIn(); }
 
-  HttpVersion Version();
+  HttpVersion Version() const;
   uint16_t Status() const;
   void StatusText(nsACString& aStatusText);
   int64_t ContentLength();
@@ -68,7 +74,7 @@ class nsHttpResponseHead {
                                    bool m = false);
   [[nodiscard]] nsresult SetHeader(const nsHttpAtom& h, const nsACString& v,
                                    bool m = false);
-  [[nodiscard]] nsresult GetHeader(const nsHttpAtom& h, nsACString& v);
+  [[nodiscard]] nsresult GetHeader(const nsHttpAtom& h, nsACString& v) const;
   void ClearHeader(const nsHttpAtom& h);
   void ClearHeaders();
   bool HasHeaderValue(const nsHttpAtom& h, const char* v);
@@ -96,7 +102,7 @@ class nsHttpResponseHead {
   [[nodiscard]] nsresult ParseCachedOriginalHeaders(char* block);
 
   // parse the status line.
-  void ParseStatusLine(const nsACString& line);
+  nsresult ParseStatusLine(const nsACString& line);
 
   // parse a header line.
   [[nodiscard]] nsresult ParseHeaderLine(const nsACString& line);
@@ -126,11 +132,6 @@ class nsHttpResponseHead {
   // reset the response head to it's initial state
   void Reset();
 
-  [[nodiscard]] nsresult GetAgeValue(uint32_t* result);
-  [[nodiscard]] nsresult GetMaxAgeValue(uint32_t* result);
-  [[nodiscard]] nsresult GetStaleWhileRevalidateValue(uint32_t* result);
-  [[nodiscard]] nsresult GetDateValue(uint32_t* result);
-  [[nodiscard]] nsresult GetExpiresValue(uint32_t* result);
   [[nodiscard]] nsresult GetLastModifiedValue(uint32_t* result);
 
   bool operator==(const nsHttpResponseHead& aOther) const;
@@ -149,36 +150,54 @@ class nsHttpResponseHead {
  private:
   [[nodiscard]] nsresult SetHeader_locked(const nsHttpAtom& atom,
                                           const nsACString& h,
-                                          const nsACString& v, bool m = false);
-  void AssignDefaultStatusText();
-  void ParseVersion(const char*);
-  void ParseCacheControl(const char*);
-  void ParsePragma(const char*);
+                                          const nsACString& v, bool m = false)
+      MOZ_REQUIRES(mRecursiveMutex);
+  void AssignDefaultStatusText() MOZ_REQUIRES(mRecursiveMutex);
+  void ParseVersion(const char*) MOZ_REQUIRES(mRecursiveMutex);
+  void ParseCacheControl(const char*) MOZ_REQUIRES(mRecursiveMutex);
+  void ParsePragma(const char*) MOZ_REQUIRES(mRecursiveMutex);
+  // Parses a content-length header-value as described in
+  // https://fetch.spec.whatwg.org/#content-length-header
+  nsresult ParseResponseContentLength(const nsACString& aHeaderStr)
+      MOZ_REQUIRES(mRecursiveMutex);
 
-  void ParseStatusLine_locked(const nsACString& line);
+  nsresult ParseStatusLine_locked(const nsACString& line)
+      MOZ_REQUIRES(mRecursiveMutex);
   [[nodiscard]] nsresult ParseHeaderLine_locked(const nsACString& line,
-                                                bool originalFromNetHeaders);
+                                                bool originalFromNetHeaders)
+      MOZ_REQUIRES(mRecursiveMutex);
 
   // these return failure if the header does not exist.
   [[nodiscard]] nsresult ParseDateHeader(const nsHttpAtom& header,
-                                         uint32_t* result) const;
+                                         uint32_t* result) const
+      MOZ_REQUIRES(mRecursiveMutex);
+  [[nodiscard]] nsresult GetAgeValue(uint32_t* result);
+  [[nodiscard]] nsresult GetMaxAgeValue(uint32_t* result);
+  [[nodiscard]] nsresult GetStaleWhileRevalidateValue(uint32_t* result);
+  [[nodiscard]] nsresult GetDateValue(uint32_t* result);
+  [[nodiscard]] nsresult GetExpiresValue(uint32_t* result);
 
-  bool ExpiresInPast_locked() const;
-  [[nodiscard]] nsresult GetAgeValue_locked(uint32_t* result) const;
-  [[nodiscard]] nsresult GetExpiresValue_locked(uint32_t* result) const;
-  [[nodiscard]] nsresult GetMaxAgeValue_locked(uint32_t* result) const;
+  bool ExpiresInPast_locked() const MOZ_REQUIRES(mRecursiveMutex);
+  [[nodiscard]] nsresult GetAgeValue_locked(uint32_t* result) const
+      MOZ_REQUIRES(mRecursiveMutex);
+  [[nodiscard]] nsresult GetExpiresValue_locked(uint32_t* result) const
+      MOZ_REQUIRES(mRecursiveMutex);
+  [[nodiscard]] nsresult GetMaxAgeValue_locked(uint32_t* result) const
+      MOZ_REQUIRES(mRecursiveMutex);
   [[nodiscard]] nsresult GetStaleWhileRevalidateValue_locked(
-      uint32_t* result) const;
+      uint32_t* result) const MOZ_REQUIRES(mRecursiveMutex);
 
-  [[nodiscard]] nsresult GetDateValue_locked(uint32_t* result) const {
+  [[nodiscard]] nsresult GetDateValue_locked(uint32_t* result) const
+      MOZ_REQUIRES(mRecursiveMutex) {
     return ParseDateHeader(nsHttp::Date, result);
   }
 
-  [[nodiscard]] nsresult GetLastModifiedValue_locked(uint32_t* result) const {
+  [[nodiscard]] nsresult GetLastModifiedValue_locked(uint32_t* result) const
+      MOZ_REQUIRES(mRecursiveMutex) {
     return ParseDateHeader(nsHttp::Last_Modified, result);
   }
 
-  bool NoCache_locked() const {
+  bool NoCache_locked() const MOZ_REQUIRES(mRecursiveMutex) {
     // We ignore Pragma: no-cache if Cache-Control is set.
     MOZ_ASSERT_IF(mCacheControlNoCache, mHasCacheControl);
     return mHasCacheControl ? mCacheControlNoCache : mPragmaNoCache;
@@ -186,30 +205,31 @@ class nsHttpResponseHead {
 
  private:
   // All members must be copy-constructable and assignable
-  nsHttpHeaderArray mHeaders;
-  HttpVersion mVersion{HttpVersion::v1_1};
-  uint16_t mStatus{200};
-  nsCString mStatusText;
-  int64_t mContentLength{-1};
-  nsCString mContentType;
-  nsCString mContentCharset;
-  bool mHasCacheControl{false};
-  bool mCacheControlPublic{false};
-  bool mCacheControlPrivate{false};
-  bool mCacheControlNoStore{false};
-  bool mCacheControlNoCache{false};
-  bool mCacheControlImmutable{false};
-  bool mCacheControlStaleWhileRevalidateSet{false};
-  uint32_t mCacheControlStaleWhileRevalidate{0};
-  bool mCacheControlMaxAgeSet{false};
-  uint32_t mCacheControlMaxAge{0};
-  bool mPragmaNoCache{false};
+  nsHttpHeaderArray mHeaders MOZ_GUARDED_BY(mRecursiveMutex);
+  HttpVersion mVersion MOZ_GUARDED_BY(mRecursiveMutex){HttpVersion::v1_1};
+  uint16_t mStatus MOZ_GUARDED_BY(mRecursiveMutex){200};
+  nsCString mStatusText MOZ_GUARDED_BY(mRecursiveMutex);
+  int64_t mContentLength MOZ_GUARDED_BY(mRecursiveMutex){-1};
+  nsCString mContentType MOZ_GUARDED_BY(mRecursiveMutex);
+  nsCString mContentCharset MOZ_GUARDED_BY(mRecursiveMutex);
+  bool mHasCacheControl MOZ_GUARDED_BY(mRecursiveMutex){false};
+  bool mCacheControlPublic MOZ_GUARDED_BY(mRecursiveMutex){false};
+  bool mCacheControlPrivate MOZ_GUARDED_BY(mRecursiveMutex){false};
+  bool mCacheControlNoStore MOZ_GUARDED_BY(mRecursiveMutex){false};
+  bool mCacheControlNoCache MOZ_GUARDED_BY(mRecursiveMutex){false};
+  bool mCacheControlImmutable MOZ_GUARDED_BY(mRecursiveMutex){false};
+  bool mCacheControlStaleWhileRevalidateSet MOZ_GUARDED_BY(mRecursiveMutex){
+      false};
+  uint32_t mCacheControlStaleWhileRevalidate MOZ_GUARDED_BY(mRecursiveMutex){0};
+  bool mCacheControlMaxAgeSet MOZ_GUARDED_BY(mRecursiveMutex){false};
+  uint32_t mCacheControlMaxAge MOZ_GUARDED_BY(mRecursiveMutex){0};
+  bool mPragmaNoCache MOZ_GUARDED_BY(mRecursiveMutex){false};
 
   // We are using RecursiveMutex instead of a Mutex because VisitHeader
   // function calls nsIHttpHeaderVisitor::VisitHeader while under lock.
   mutable RecursiveMutex mRecursiveMutex{"nsHttpResponseHead.mRecursiveMutex"};
-  // During VisitHeader we sould not allow cal to SetHeader.
-  bool mInVisitHeaders{false};
+  // During VisitHeader we sould not allow call to SetHeader.
+  bool mInVisitHeaders MOZ_GUARDED_BY(mRecursiveMutex){false};
 
   friend struct IPC::ParamTraits<nsHttpResponseHead>;
 };
