@@ -14,13 +14,14 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/IncrementalTokenizer.h"
 #include "nsHttpResponseHead.h"
+#include "mozilla/UniquePtr.h"
 
-#define NS_MULTIMIXEDCONVERTER_CID                 \
-  { /* 7584CE90-5B25-11d3-A175-0050041CAF44 */     \
-    0x7584ce90, 0x5b25, 0x11d3, {                  \
-      0xa1, 0x75, 0x0, 0x50, 0x4, 0x1c, 0xaf, 0x44 \
-    }                                              \
-  }
+#define NS_MULTIMIXEDCONVERTER_CID            \
+  {/* 7584CE90-5B25-11d3-A175-0050041CAF44 */ \
+   0x7584ce90,                                \
+   0x5b25,                                    \
+   0x11d3,                                    \
+   {0xa1, 0x75, 0x0, 0x50, 0x4, 0x1c, 0xaf, 0x44}}
 
 //
 // nsPartChannel is a "dummy" channel which represents an individual part of
@@ -34,7 +35,7 @@ class nsPartChannel final : public nsIChannel,
                             public nsIMultiPartChannel {
  public:
   nsPartChannel(nsIChannel* aMultipartChannel, uint32_t aPartID,
-                nsIStreamListener* aListener);
+                bool aIsFirstPart, nsIStreamListener* aListener);
 
   void InitializeByteRange(int64_t aStart, int64_t aEnd);
   void SetIsLastPart() { mIsLastPart = true; }
@@ -62,7 +63,7 @@ class nsPartChannel final : public nsIChannel,
  protected:
   nsCOMPtr<nsIChannel> mMultipartChannel;
   nsCOMPtr<nsIStreamListener> mListener;
-  UniquePtr<mozilla::net::nsHttpResponseHead> mResponseHead;
+  mozilla::UniquePtr<mozilla::net::nsHttpResponseHead> mResponseHead;
 
   nsresult mStatus{NS_OK};
   nsLoadFlags mLoadFlags{0};
@@ -82,6 +83,7 @@ class nsPartChannel final : public nsIChannel,
 
   uint32_t mPartID;  // unique ID that can be used to identify
                      // this part of the multipart document
+  bool mIsFirstPart;
   bool mIsLastPart{false};
 };
 
@@ -130,6 +132,7 @@ class nsMultiMixedConv : public nsIStreamConverter {
   NS_DECL_ISUPPORTS
   NS_DECL_NSISTREAMCONVERTER
   NS_DECL_NSISTREAMLISTENER
+  NS_DECL_NSITHREADRETARGETABLESTREAMLISTENER
   NS_DECL_NSIREQUESTOBSERVER
 
   explicit nsMultiMixedConv();
@@ -148,15 +151,17 @@ class nsMultiMixedConv : public nsIStreamConverter {
   nsCOMPtr<nsIStreamListener> mFinalListener;  // this guy gets the converted
                                                // data via his OnDataAvailable()
 
-  nsCOMPtr<nsIChannel>
-      mChannel;  // The channel as we get in in OnStartRequest call
-  RefPtr<nsPartChannel> mPartChannel;  // the channel for the given part we're
-                                       // processing. one channel per part.
+  // The channel as we get it in OnStartRequest call
+  nsCOMPtr<nsIChannel> mChannel;
+  // the channel for the given part we're
+  // processing. one channel per part.
+  RefPtr<nsPartChannel> mPartChannel;
   nsCOMPtr<nsISupports> mContext;
   nsCString mContentType;
   nsCString mContentDisposition;
   nsCString mContentSecurityPolicy;
   nsCString mRootContentSecurityPolicy;
+  nsCString mRootContentDisposition;
   uint64_t mContentLength{UINT64_MAX};
   uint64_t mTotalSent{0};
 
