@@ -1,3 +1,5 @@
+"use strict";
+
 function run_test() {
   do_get_profile();
 
@@ -7,16 +9,16 @@ function run_test() {
     "pin",
     Ci.nsICacheStorage.OPEN_TRUNCATE,
     Services.loadContextInfo.default,
-    new OpenCallback(NEW | WAITFORWRITE, "a1m", "a1d", function(entry) {
+    new OpenCallback(NEW | WAITFORWRITE, "a1m", "a1d", function () {
       // Open for read and check
       asyncOpenCacheEntry(
         "http://a/",
         "disk",
         Ci.nsICacheStorage.OPEN_NORMALLY,
         Services.loadContextInfo.default,
-        new OpenCallback(NORMAL, "a1m", "a1d", function(entry) {
+        new OpenCallback(NORMAL, "a1m", "a1d", function () {
           // Now clear the whole cache
-          get_cache_service().clear();
+          Services.cache2.clear();
 
           // The pinned entry should be intact
           asyncOpenCacheEntry(
@@ -24,7 +26,7 @@ function run_test() {
             "disk",
             Ci.nsICacheStorage.OPEN_NORMALLY,
             Services.loadContextInfo.default,
-            new OpenCallback(NORMAL, "a1m", "a1d", function(entry) {
+            new OpenCallback(NORMAL, "a1m", "a1d", function () {
               finish_cache2_test();
             })
           );
