@@ -2,7 +2,9 @@
 
 "use strict";
 
-const { HttpServer } = ChromeUtils.import("resource://testing-common/httpd.js");
+const { HttpServer } = ChromeUtils.importESModule(
+  "resource://testing-common/httpd.sys.mjs"
+);
 
 var httpserver = new HttpServer();
 httpserver.start(-1);
@@ -30,7 +32,7 @@ CheckGetHostListener.prototype = {
     }
   },
 
-  onStopRequest(request, statusCode) {
+  onStopRequest() {
     dump("*** listener onStopRequest\n");
 
     Assert.equal(gotOnStartRequest, true);
