@@ -4,63 +4,72 @@
 
 "use strict";
 
-ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
-var { setTimeout } = ChromeUtils.import("resource://gre/modules/Timer.jsm");
+const { TestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TestUtils.sys.mjs"
+);
 
 let h2Port;
 let h3Port;
 let h3NoResponsePort;
 let trrServer;
 
-const dns = Cc["@mozilla.org/network/dns-service;1"].getService(
-  Ci.nsIDNSService
-);
 const certOverrideService = Cc[
   "@mozilla.org/security/certoverride;1"
 ].getService(Ci.nsICertOverrideService);
 
-function setup() {
+add_setup(async function setup() {
   trr_test_setup();
 
-  let env = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
-  h2Port = env.get("MOZHTTP2_PORT");
+  h2Port = Services.env.get("MOZHTTP2_PORT");
   Assert.notEqual(h2Port, null);
   Assert.notEqual(h2Port, "");
 
-  h3Port = env.get("MOZHTTP3_PORT");
+  h3Port = Services.env.get("MOZHTTP3_PORT");
   Assert.notEqual(h3Port, null);
   Assert.notEqual(h3Port, "");
 
-  h3NoResponsePort = env.get("MOZHTTP3_PORT_NO_RESPONSE");
+  h3NoResponsePort = Services.env.get("MOZHTTP3_PORT_NO_RESPONSE");
   Assert.notEqual(h3NoResponsePort, null);
   Assert.notEqual(h3NoResponsePort, "");
 
-  Services.prefs.setIntPref("network.trr.mode", Ci.nsIDNSService.MODE_TRRFIRST);
-
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
   Services.prefs.setBoolPref("network.dns.echconfig.enabled", true);
-}
-
-setup();
-registerCleanupFunction(async () => {
-  trr_clear_prefs();
-  Services.prefs.clearUserPref("network.dns.upgrade_with_https_rr");
-  Services.prefs.clearUserPref("network.dns.use_https_rr_as_altsvc");
-  Services.prefs.clearUserPref("network.dns.echconfig.enabled");
-  Services.prefs.clearUserPref("network.dns.echconfig.fallback_to_origin");
-  Services.prefs.clearUserPref("network.dns.httpssvc.reset_exclustion_list");
-  Services.prefs.clearUserPref("network.http.http3.enabled");
-  Services.prefs.clearUserPref(
-    "network.dns.httpssvc.http3_fast_fallback_timeout"
+  Services.prefs.setBoolPref(
+    "network.dns.https_rr.check_record_with_cname",
+    false
   );
-  Services.prefs.clearUserPref("network.http.speculative-parallel-limit");
-  Services.prefs.clearUserPref("network.dns.localDomains");
-  if (trrServer) {
-    await trrServer.stop();
+
+  registerCleanupFunction(async () => {
+    trr_clear_prefs();
+    Services.prefs.clearUserPref("network.dns.upgrade_with_https_rr");
+    Services.prefs.clearUserPref("network.dns.use_https_rr_as_altsvc");
+    Services.prefs.clearUserPref("network.dns.echconfig.enabled");
+    Services.prefs.clearUserPref(
+      "network.dns.echconfig.fallback_to_origin_when_all_failed"
+    );
+    Services.prefs.clearUserPref("network.dns.httpssvc.reset_exclustion_list");
+    Services.prefs.clearUserPref("network.http.http3.enable");
+    Services.prefs.clearUserPref(
+      "network.dns.httpssvc.http3_fast_fallback_timeout"
+    );
+    Services.prefs.clearUserPref("network.http.speculative-parallel-limit");
+    Services.prefs.clearUserPref("network.dns.localDomains");
+    Services.prefs.clearUserPref("network.dns.http3_echconfig.enabled");
+    Services.prefs.clearUserPref(
+      "network.dns.https_rr.check_record_with_cname"
+    );
+    if (trrServer) {
+      await trrServer.stop();
+    }
+  });
+
+  if (mozinfo.socketprocess_networking) {
+    Services.dns; // Needed to trigger socket process.
+    await TestUtils.waitForCondition(() => Services.io.socketProcessLaunched);
   }
+
+  Services.prefs.setIntPref("network.trr.mode", Ci.nsIDNSService.MODE_TRRFIRST);
 });
 
 function makeChan(url) {
@@ -80,8 +89,6 @@ function channelOpenPromise(chan, flags) {
         false
       );
     }
-    let internal = chan.QueryInterface(Ci.nsIHttpChannelInternal);
-    internal.setWaitForHTTPSSVCRecord();
     certOverrideService.setDisableAllSecurityChecksAndLetAttackersInterceptMyData(
       true
     );
@@ -97,74 +104,79 @@ add_task(async function testFallbackToTheLastRecord() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
 
   // Only the last record is valid to use.
-  await trrServer.registerDoHAnswers("test.fallback.com", "HTTPS", {
-    answers: [
-      {
-        name: "test.fallback.com",
-        ttl: 55,
-        type: "HTTPS",
-        flush: false,
-        data: {
-          priority: 1,
-          name: "test.fallback1.com",
-          values: [
-            { key: "alpn", value: ["h2", "h3-26"] },
-            { key: "echconfig", value: "123..." },
-          ],
+  await trrServer.registerDoHAnswers(
+    `_${h2Port}._https.test.fallback.com`,
+    "HTTPS",
+    {
+      answers: [
+        {
+          name: `_${h2Port}._https.test.fallback.com`,
+          ttl: 55,
+          type: "HTTPS",
+          flush: false,
+          data: {
+            priority: 1,
+            name: "test.fallback1.com",
+            values: [
+              { key: "alpn", value: ["h2", "h3-26"] },
+              { key: "echconfig", value: "123..." },
+            ],
+          },
         },
-      },
-      {
-        name: "test.fallback.com",
-        ttl: 55,
-        type: "HTTPS",
-        flush: false,
-        data: {
-          priority: 4,
-          name: "foo.example.com",
-          values: [
-            { key: "alpn", value: ["h2", "h3-26"] },
-            { key: "port", value: h2Port },
-            { key: "echconfig", value: "456..." },
-          ],
+        {
+          name: `_${h2Port}._https.test.fallback.com`,
+          ttl: 55,
+          type: "HTTPS",
+          flush: false,
+          data: {
+            priority: 4,
+            name: "foo.example.com",
+            values: [
+              { key: "alpn", value: ["h2", "h3-26"] },
+              { key: "port", value: h2Port },
+              { key: "echconfig", value: "456..." },
+            ],
+          },
         },
-      },
-      {
-        name: "test.fallback.com",
-        ttl: 55,
-        type: "HTTPS",
-        flush: false,
-        data: {
-          priority: 3,
-          name: "test.fallback3.com",
-          values: [
-            { key: "alpn", value: ["h2", "h3-26"] },
-            { key: "echconfig", value: "456..." },
-          ],
+        {
+          name: `_${h2Port}._https.test.fallback.com`,
+          ttl: 55,
+          type: "HTTPS",
+          flush: false,
+          data: {
+            priority: 3,
+            name: "test.fallback3.com",
+            values: [
+              { key: "alpn", value: ["h2", "h3-26"] },
+              { key: "echconfig", value: "456..." },
+            ],
+          },
         },
-      },
-      {
-        name: "test.fallback.com",
-        ttl: 55,
-        type: "HTTPS",
-        flush: false,
-        data: {
-          priority: 2,
-          name: "test.fallback2.com",
-          values: [
-            { key: "alpn", value: ["h2", "h3-26"] },
-            { key: "echconfig", value: "456..." },
-          ],
+        {
+          name: `_${h2Port}._https.test.fallback.com`,
+          ttl: 55,
+          type: "HTTPS",
+          flush: false,
+          data: {
+            priority: 2,
+            name: "test.fallback2.com",
+            values: [
+              { key: "alpn", value: ["h2", "h3-26"] },
+              { key: "echconfig", value: "456..." },
+            ],
+          },
         },
-      },
-    ],
-  });
+      ],
+    }
+  );
 
   await new TRRDNSListener("test.fallback.com", {
     type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
+    port: h2Port,
   });
 
   let chan = makeChan(`https://test.fallback.com:${h2Port}/server-timing`);
@@ -179,9 +191,13 @@ add_task(async function testFallbackToTheOrigin() {
   trrServer = new TRRServer();
   await trrServer.start();
   Services.prefs.setIntPref("network.trr.mode", 3);
+  Services.prefs.setBoolPref(
+    "network.dns.echconfig.fallback_to_origin_when_all_failed",
+    true
+  );
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
 
   // All records are not able to use to connect, so we fallback to the origin
@@ -265,9 +281,12 @@ add_task(async function testAllRecordsFailed() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
+  );
+  Services.prefs.setBoolPref(
+    "network.dns.echconfig.fallback_to_origin_when_all_failed",
+    false
   );
-  Services.prefs.setBoolPref("network.dns.echconfig.fallback_to_origin", false);
 
   await trrServer.registerDoHAnswers("test.bar.com", "HTTPS", {
     answers: [
@@ -335,7 +354,7 @@ add_task(async function testFallbackToTheOrigin2() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
 
   await trrServer.registerDoHAnswers("test.example.com", "HTTPS", {
@@ -393,14 +412,14 @@ add_task(async function testFallbackToTheOrigin2() {
 // Test when some records have echConfig and some not, we directly fallback to
 // the origin one.
 add_task(async function testFallbackToTheOrigin3() {
-  dns.clearCache(true);
+  Services.dns.clearCache(true);
 
   trrServer = new TRRServer();
   await trrServer.start();
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
 
   await trrServer.registerDoHAnswers("vulnerable.com", "A", {
@@ -475,49 +494,54 @@ add_task(async function testResetExclusionList() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref(
     "network.dns.httpssvc.reset_exclustion_list",
     false
   );
 
-  await trrServer.registerDoHAnswers("test.reset.com", "HTTPS", {
-    answers: [
-      {
-        name: "test.reset.com",
-        ttl: 55,
-        type: "HTTPS",
-        flush: false,
-        data: {
-          priority: 1,
-          name: "test.reset1.com",
-          values: [
-            { key: "alpn", value: ["h2", "h3-26"] },
-            { key: "port", value: h2Port },
-            { key: "echconfig", value: "456..." },
-          ],
+  await trrServer.registerDoHAnswers(
+    `_${h2Port}._https.test.reset.com`,
+    "HTTPS",
+    {
+      answers: [
+        {
+          name: `_${h2Port}._https.test.reset.com`,
+          ttl: 55,
+          type: "HTTPS",
+          flush: false,
+          data: {
+            priority: 1,
+            name: "test.reset1.com",
+            values: [
+              { key: "alpn", value: ["h2", "h3-26"] },
+              { key: "port", value: h2Port },
+              { key: "echconfig", value: "456..." },
+            ],
+          },
         },
-      },
-      {
-        name: "test.reset.com",
-        ttl: 55,
-        type: "HTTPS",
-        flush: false,
-        data: {
-          priority: 2,
-          name: "test.reset2.com",
-          values: [
-            { key: "alpn", value: ["h2", "h3-26"] },
-            { key: "echconfig", value: "456..." },
-          ],
+        {
+          name: `_${h2Port}._https.test.reset.com`,
+          ttl: 55,
+          type: "HTTPS",
+          flush: false,
+          data: {
+            priority: 2,
+            name: "test.reset2.com",
+            values: [
+              { key: "alpn", value: ["h2", "h3-26"] },
+              { key: "echconfig", value: "456..." },
+            ],
+          },
         },
-      },
-    ],
-  });
+      ],
+    }
+  );
 
   await new TRRDNSListener("test.reset.com", {
     type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
+    port: h2Port,
   });
 
   // After this request, test.reset1.com and test.reset2.com should be both in
@@ -561,9 +585,9 @@ add_task(async function testH3Connection() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
-  Services.prefs.setBoolPref("network.http.http3.enabled", true);
+  Services.prefs.setBoolPref("network.http.http3.enable", true);
 
   Services.prefs.setIntPref(
     "network.dns.httpssvc.http3_fast_fallback_timeout",
@@ -581,7 +605,7 @@ add_task(async function testH3Connection() {
           priority: 1,
           name: "www.h3.com",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
             { key: "port", value: h3Port },
             { key: "echconfig", value: "456..." },
           ],
@@ -608,7 +632,7 @@ add_task(async function testH3Connection() {
 
   let chan = makeChan(`https://test.h3.com`);
   let [req] = await channelOpenPromise(chan);
-  Assert.equal(req.protocolVersion, "h3-29");
+  Assert.equal(req.protocolVersion, "h3");
   let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
   Assert.equal(internal.remotePort, h3Port);
 
@@ -621,9 +645,9 @@ add_task(async function testFastfallbackToH2() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
-  Services.prefs.setBoolPref("network.http.http3.enabled", true);
+  Services.prefs.setBoolPref("network.http.http3.enable", true);
   // Use a short timeout to make sure the fast fallback timer will be triggered.
   Services.prefs.setIntPref(
     "network.dns.httpssvc.http3_fast_fallback_timeout",
@@ -645,7 +669,7 @@ add_task(async function testFastfallbackToH2() {
           priority: 1,
           name: "test.fastfallback1.com",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
             { key: "port", value: h3NoResponsePort },
             { key: "echconfig", value: "456..." },
           ],
@@ -710,13 +734,13 @@ add_task(async function testFastfallbackToH2() {
 add_task(async function testFailedH3Connection() {
   trrServer = new TRRServer();
   await trrServer.start();
-  dns.clearCache(true);
+  Services.dns.clearCache(true);
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
-  Services.prefs.setBoolPref("network.http.http3.enabled", true);
+  Services.prefs.setBoolPref("network.http.http3.enable", true);
   Services.prefs.setIntPref(
     "network.dns.httpssvc.http3_fast_fallback_timeout",
     0
@@ -733,7 +757,7 @@ add_task(async function testFailedH3Connection() {
           priority: 1,
           name: "www.h3.org",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
             { key: "port", value: h3Port },
             { key: "echconfig", value: "456..." },
           ],
@@ -757,13 +781,13 @@ add_task(async function testFailedH3Connection() {
 add_task(async function testHttp3ExcludedList() {
   trrServer = new TRRServer();
   await trrServer.start();
-  dns.clearCache(true);
+  Services.dns.clearCache(true);
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
-  Services.prefs.setBoolPref("network.http.http3.enabled", true);
+  Services.prefs.setBoolPref("network.http.http3.enable", true);
   Services.prefs.setIntPref(
     "network.dns.httpssvc.http3_fast_fallback_timeout",
     0
@@ -771,7 +795,7 @@ add_task(async function testHttp3ExcludedList() {
 
   Services.prefs.setCharPref(
     "network.http.http3.alt-svc-mapping-for-testing",
-    "www.h3_fail.org;h3-29=:" + h3Port
+    "www.h3_fail.org;h3=:" + h3Port
   );
 
   // This will fail because there is no address record for www.h3_fail.org.
@@ -791,7 +815,8 @@ add_task(async function testHttp3ExcludedList() {
           priority: 1,
           name: "www.h3_fail.org",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
+            { key: "no-default-alpn" },
             { key: "port", value: h3Port },
           ],
         },
@@ -805,7 +830,7 @@ add_task(async function testHttp3ExcludedList() {
           priority: 2,
           name: "foo.example.com",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
             { key: "port", value: h3Port },
           ],
         },
@@ -819,7 +844,7 @@ add_task(async function testHttp3ExcludedList() {
 
   chan = makeChan(`https://test.h3_excluded.org`);
   let [req] = await channelOpenPromise(chan);
-  Assert.equal(req.protocolVersion, "h3-29");
+  Assert.equal(req.protocolVersion, "h3");
   let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
   Assert.equal(internal.remotePort, h3Port);
 
@@ -829,13 +854,14 @@ add_task(async function testHttp3ExcludedList() {
 add_task(async function testAllRecordsInHttp3ExcludedList() {
   trrServer = new TRRServer();
   await trrServer.start();
-  dns.clearCache(true);
+  Services.dns.clearCache(true);
   Services.prefs.setIntPref("network.trr.mode", 3);
+  Services.prefs.setBoolPref("network.dns.http3_echconfig.enabled", true);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
-  Services.prefs.setBoolPref("network.http.http3.enabled", true);
+  Services.prefs.setBoolPref("network.http.http3.enable", true);
   Services.prefs.setIntPref(
     "network.dns.httpssvc.http3_fast_fallback_timeout",
     0
@@ -843,13 +869,13 @@ add_task(async function testAllRecordsInHttp3ExcludedList() {
 
   Services.prefs.setCharPref(
     "network.http.http3.alt-svc-mapping-for-testing",
-    "www.h3_fail1.org;h3-29=:" + h3Port
+    "_${h2Port}._https.www.h3_fail1.org;h3=:" + h3Port
   );
 
-  await trrServer.registerDoHAnswers("www.h3_all_excluded.org", "A", {
+  await trrServer.registerDoHAnswers(`www.h3_all_excluded.org`, "A", {
     answers: [
       {
-        name: "www.h3_all_excluded.org",
+        name: `www.h3_all_excluded.org`,
         ttl: 55,
         type: "A",
         flush: false,
@@ -874,53 +900,59 @@ add_task(async function testAllRecordsInHttp3ExcludedList() {
 
   Services.prefs.setCharPref(
     "network.http.http3.alt-svc-mapping-for-testing",
-    "www.h3_fail2.org;h3-29=:" + h3Port
+    "_${h2Port}._https.www.h3_fail2.org;h3=:" + h3Port
   );
 
   // This will fail because there is no address record for www.h3_fail2.org.
   chan = makeChan(`https://www.h3_fail2.org`);
   await channelOpenPromise(chan, CL_EXPECT_LATE_FAILURE | CL_ALLOW_UNKNOWN_CL);
 
-  await trrServer.registerDoHAnswers("www.h3_all_excluded.org", "HTTPS", {
-    answers: [
-      {
-        name: "www.h3_all_excluded.org",
-        ttl: 55,
-        type: "HTTPS",
-        flush: false,
-        data: {
-          priority: 1,
-          name: "www.h3_fail1.org",
-          values: [
-            { key: "alpn", value: "h3-29" },
-            { key: "port", value: h3Port },
-            { key: "echconfig", value: "456..." },
-          ],
+  await trrServer.registerDoHAnswers(
+    `_${h2Port}._https.www.h3_all_excluded.org`,
+    "HTTPS",
+    {
+      answers: [
+        {
+          name: `_${h2Port}._https.www.h3_all_excluded.org`,
+          ttl: 55,
+          type: "HTTPS",
+          flush: false,
+          data: {
+            priority: 1,
+            name: "www.h3_fail1.org",
+            values: [
+              { key: "alpn", value: "h3" },
+              { key: "no-default-alpn" },
+              { key: "port", value: h3Port },
+              { key: "echconfig", value: "456..." },
+            ],
+          },
         },
-      },
-      {
-        name: "www.h3_all_excluded.org",
-        ttl: 55,
-        type: "HTTPS",
-        flush: false,
-        data: {
-          priority: 2,
-          name: "www.h3_fail2.org",
-          values: [
-            { key: "alpn", value: "h3-29" },
-            { key: "port", value: h3Port },
-            { key: "echconfig", value: "456..." },
-          ],
+        {
+          name: `_${h2Port}._https.www.h3_all_excluded.org`,
+          ttl: 55,
+          type: "HTTPS",
+          flush: false,
+          data: {
+            priority: 2,
+            name: "www.h3_fail2.org",
+            values: [
+              { key: "alpn", value: "h3" },
+              { key: "port", value: h3Port },
+              { key: "echconfig", value: "456..." },
+            ],
+          },
         },
-      },
-    ],
-  });
+      ],
+    }
+  );
 
   await new TRRDNSListener("www.h3_all_excluded.org", {
     type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
+    port: h2Port,
   });
 
-  dns.clearCache(true);
+  Services.dns.clearCache(true);
   Services.prefs.setIntPref("network.http.speculative-parallel-limit", 0);
   Services.obs.notifyObservers(null, "net:prune-all-connections");
 
@@ -943,11 +975,153 @@ add_task(async function testAllRecordsInHttp3ExcludedList() {
 
   // The the case that when all records are in http3 excluded list, we still
   // give the first record one more shot.
-  chan = makeChan(`https://www.h3_all_excluded.org`);
+  chan = makeChan(`https://www.h3_all_excluded.org:${h2Port}`);
   [req] = await channelOpenPromise(chan);
-  Assert.equal(req.protocolVersion, "h3-29");
+  Assert.equal(req.protocolVersion, "h3");
   let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
   Assert.equal(internal.remotePort, h3Port);
 
   await trrServer.stop();
 });
+
+WebSocketListener.prototype = {
+  onAcknowledge() {},
+  onBinaryMessageAvailable() {},
+  onMessageAvailable() {},
+  onServerClose() {},
+  onStart() {
+    this.finish();
+  },
+  onStop() {},
+};
+
+add_task(async function testUpgradeNotUsingHTTPSRR() {
+  trrServer = new TRRServer();
+  await trrServer.start();
+  Services.prefs.setIntPref("network.trr.mode", 3);
+  Services.prefs.setCharPref(
+    "network.trr.uri",
+    `https://foo.example.com:${trrServer.port()}/dns-query`
+  );
+
+  await trrServer.registerDoHAnswers("test.ws.com", "HTTPS", {
+    answers: [
+      {
+        name: "test.ws.com",
+        ttl: 55,
+        type: "HTTPS",
+        flush: false,
+        data: {
+          priority: 1,
+          name: "test.ws1.com",
+          values: [{ key: "port", value: ["8888"] }],
+        },
+      },
+    ],
+  });
+
+  await new TRRDNSListener("test.ws.com", {
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
+  });
+
+  await trrServer.registerDoHAnswers("test.ws.com", "A", {
+    answers: [
+      {
+        name: "test.ws.com",
+        ttl: 55,
+        type: "A",
+        flush: false,
+        data: "127.0.0.1",
+      },
+    ],
+  });
+
+  let wssUri = "wss://test.ws.com:" + h2Port + "/websocket";
+  let chan = Cc["@mozilla.org/network/protocol;1?name=wss"].createInstance(
+    Ci.nsIWebSocketChannel
+  );
+  chan.initLoadInfo(
+    null, // aLoadingNode
+    Services.scriptSecurityManager.getSystemPrincipal(),
+    null, // aTriggeringPrincipal
+    Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
+    Ci.nsIContentPolicy.TYPE_DOCUMENT
+  );
+
+  var uri = Services.io.newURI(wssUri);
+  var wsListener = new WebSocketListener();
+  certOverrideService.setDisableAllSecurityChecksAndLetAttackersInterceptMyData(
+    false
+  );
+  await new Promise(resolve => {
+    wsListener.finish = resolve;
+    chan.asyncOpen(uri, wssUri, {}, 0, wsListener, null);
+    certOverrideService.setDisableAllSecurityChecksAndLetAttackersInterceptMyData(
+      true
+    );
+  });
+
+  await trrServer.stop();
+});
+
+// Test if we fallback to h2 with echConfig.
+add_task(async function testFallbackToH2WithEchConfig() {
+  trrServer = new TRRServer();
+  await trrServer.start();
+  Services.dns.clearCache(true);
+  Services.prefs.setIntPref("network.trr.mode", 3);
+  Services.prefs.setCharPref(
+    "network.trr.uri",
+    `https://foo.example.com:${trrServer.port()}/dns-query`
+  );
+  Services.prefs.setBoolPref("network.http.http3.enable", true);
+  Services.prefs.setIntPref(
+    "network.dns.httpssvc.http3_fast_fallback_timeout",
+    0
+  );
+
+  await trrServer.registerDoHAnswers("test.fallback.org", "HTTPS", {
+    answers: [
+      {
+        name: "test.fallback.org",
+        ttl: 55,
+        type: "HTTPS",
+        flush: false,
+        data: {
+          priority: 1,
+          name: "test.fallback.org",
+          values: [
+            { key: "alpn", value: ["h2", "h3"] },
+            { key: "port", value: h2Port },
+            { key: "echconfig", value: "456..." },
+          ],
+        },
+      },
+    ],
+  });
+
+  await trrServer.registerDoHAnswers("test.fallback.org", "A", {
+    answers: [
+      {
+        name: "test.fallback.org",
+        ttl: 55,
+        type: "A",
+        flush: false,
+        data: "127.0.0.1",
+      },
+    ],
+  });
+
+  await new TRRDNSListener("test.fallback.org", {
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
+  });
+
+  await new TRRDNSListener("test.fallback.org", "127.0.0.1");
+
+  let chan = makeChan(`https://test.fallback.org/server-timing`);
+  let [req] = await channelOpenPromise(chan);
+  // Test if this request is done by h2.
+  Assert.equal(req.getResponseHeader("x-connection-http2"), "yes");
+
+  await trrServer.stop();
+});
