@@ -3,6 +3,8 @@
 
 "use strict";
 
+requestLongerTimeout(2);
+
 const TEST_PATH = getRootDirectory(gTestPath).replace(
   "chrome://mochitests/content",
   "https://example.com"
@@ -88,12 +90,12 @@ add_task(async function test_closed_by_tab_navigation() {
   info(
     "Set up unload handler to ensure we don't break when the window global gets cleared"
   );
-  await SpecialPowers.spawn(tab.linkedBrowser, [], async function() {
-    content.addEventListener("unload", function() {});
+  await SpecialPowers.spawn(tab.linkedBrowser, [], async function () {
+    content.addEventListener("unload", function () {});
   });
 
   info("Navigating tab to a different but same origin page.");
-  BrowserTestUtils.loadURI(tab.linkedBrowser, TEST_PATH);
+  BrowserTestUtils.startLoadingURIString(tab.linkedBrowser, TEST_PATH);
   await BrowserTestUtils.browserLoaded(tab.linkedBrowser, false, TEST_PATH);
   ok(dialog._frame.contentWindow, "Dialog should stay open.");
 
@@ -109,7 +111,10 @@ add_task(async function test_closed_by_tab_navigation() {
 
   info("Now navigate to a cross-origin page.");
   const CROSS_ORIGIN_TEST_PATH = TEST_PATH.replace(".com", ".org");
-  BrowserTestUtils.loadURI(tab.linkedBrowser, CROSS_ORIGIN_TEST_PATH);
+  BrowserTestUtils.startLoadingURIString(
+    tab.linkedBrowser,
+    CROSS_ORIGIN_TEST_PATH
+  );
   let loadPromise = BrowserTestUtils.browserLoaded(
     tab.linkedBrowser,
     false,
@@ -220,7 +225,7 @@ add_task(async function invisible_iframes() {
     tab.linkedBrowser,
     true
   );
-  await SpecialPowers.spawn(tab.linkedBrowser, [], function() {
+  await SpecialPowers.spawn(tab.linkedBrowser, [], function () {
     let frame = content.document.createElement("iframe");
     frame.style.display = "none";
     frame.src = "mailto:help@example.com";
@@ -263,7 +268,7 @@ add_task(async function nested_iframes() {
     "https://example.org/"
   );
   info("Constructing top frame");
-  await SpecialPowers.spawn(tab.linkedBrowser, [], function() {
+  await SpecialPowers.spawn(tab.linkedBrowser, [], function () {
     let frame = content.document.createElement("iframe");
     frame.src = "https://example.org/"; // cross-origin frame.
     content.document.body.prepend(frame);
@@ -277,7 +282,7 @@ add_task(async function nested_iframes() {
   let parentBC = tab.linkedBrowser.browsingContext;
 
   info("Creating innermost frame");
-  await SpecialPowers.spawn(parentBC.children[0], [], async function() {
+  await SpecialPowers.spawn(parentBC.children[0], [], async function () {
     let innerFrame = content.document.createElement("iframe");
     let frameLoaded = ContentTaskUtils.waitForEvent(innerFrame, "load", true);
     content.document.body.prepend(innerFrame);
@@ -288,7 +293,7 @@ add_task(async function nested_iframes() {
   await SpecialPowers.spawn(
     parentBC.children[0].children[0],
     [],
-    async function() {
+    async function () {
       // Top browsing context needs reference to the innermost, which is cross origin.
       content.eval("top.postMessage('hello', '*')");
     }
@@ -367,7 +372,7 @@ add_task(async function xorigin_iframe_can_navigate_top() {
     "https://example.org/"
   );
   info("Constructing frame");
-  await SpecialPowers.spawn(tab.linkedBrowser, [], function() {
+  await SpecialPowers.spawn(tab.linkedBrowser, [], function () {
     let frame = content.document.createElement("iframe");
     frame.src = "https://example.org/"; // cross-origin frame.
     content.document.body.prepend(frame);
@@ -376,7 +381,7 @@ add_task(async function xorigin_iframe_can_navigate_top() {
 
   info("Navigating top bc from frame");
   let parentBC = tab.linkedBrowser.browsingContext;
-  await SpecialPowers.spawn(parentBC.children[0], [], async function() {
+  await SpecialPowers.spawn(parentBC.children[0], [], async function () {
     content.eval("window.top.location.href = 'mailto:example@example.com';");
   });
 
@@ -396,3 +401,195 @@ add_task(async function xorigin_iframe_can_navigate_top() {
   await dialogClosedPromise;
   gBrowser.removeTab(tab);
 });
+
+/**
+ * Check that when navigating to an external protocol from an iframe in a
+ * background tab, we show the dialog in the correct tab.
+ */
+add_task(async function iframe_background_tab() {
+  let tab = await BrowserTestUtils.openNewForegroundTab(
+    gBrowser,
+    "https://example.com/"
+  );
+
+  let innerLoaded = BrowserTestUtils.browserLoaded(
+    tab.linkedBrowser,
+    true,
+    "https://example.org/"
+  );
+  info("Constructing frame");
+  await SpecialPowers.spawn(tab.linkedBrowser, [], function () {
+    let frame = content.document.createElement("iframe");
+    frame.src = "https://example.org/";
+    content.document.body.prepend(frame);
+  });
+  await innerLoaded;
+
+  info("Switching to new tab");
+  let newTab = await BrowserTestUtils.openNewForegroundTab(
+    gBrowser,
+    "https://example.net/"
+  );
+
+  // Wait for the chooser dialog to open in the background tab. It should not
+  // open in the foreground tab which is unrelated to the external protocol
+  // navigation.
+  let dialogWindowPromise = waitForProtocolAppChooserDialog(gBrowser, true);
+
+  info("Navigating to external proto from frame in background tab");
+  let parentBC = tab.linkedBrowser.browsingContext;
+  await SpecialPowers.spawn(parentBC.children[0], [], async function () {
+    content.eval("location.href = 'mailto:example@example.com';");
+  });
+
+  // Wait for dialog to open in one of the tabs.
+  let dialog = await dialogWindowPromise;
+
+  is(
+    gBrowser.getTabDialogBox(tab.linkedBrowser)._tabDialogManager._topDialog,
+    dialog,
+    "Dialog opened in the background tab"
+  );
+
+  is(
+    dialog._frame.contentDocument.location.href,
+    CONTENT_HANDLING_URL,
+    "Opened dialog is appChooser dialog."
+  );
+
+  // Close the dialog:
+  let dialogClosedPromise = waitForProtocolAppChooserDialog(gBrowser, false);
+  dialog.close();
+  await dialogClosedPromise;
+
+  gBrowser.removeTab(tab);
+  gBrowser.removeTab(newTab);
+});
+
+/**
+ * Check that when navigating to an external protocol in a noopener pop-up
+ * window, we show the dialog in the correct tab.
+ */
+add_task(async function iframe_popup_tab() {
+  let tab = await BrowserTestUtils.openNewForegroundTab(
+    gBrowser,
+    "https://example.com/"
+  );
+
+  // Wait for the chooser dialog to open in the background tab. It should not
+  // open in the foreground tab which is unrelated to the external protocol
+  // navigation.
+  let dialogWindowPromise = waitForProtocolAppChooserDialog(gBrowser, true);
+
+  // Wait for the new tab to appear. The URI in this tab will never change from
+  // `about:blank` as we're going to just end up opening a dialog, so we can't
+  // use `waitForNewTab`, as that will wait for the tab to actually load
+  // something.
+  let newTabPromise = new Promise(resolve => {
+    gBrowser.tabContainer.addEventListener(
+      "TabOpen",
+      openEvent => resolve(openEvent.target),
+      { once: true }
+    );
+  });
+
+  info("Navigating to external proto in pop-up");
+  await SpecialPowers.spawn(tab.linkedBrowser, [], async function () {
+    content.eval(
+      "window.open('mailto:example@example.com', '_blank', 'noopener');"
+    );
+  });
+
+  // Wait for the new tab to be opened.
+  info("Waiting for new tab to appear");
+  let newTab = await newTabPromise;
+
+  // Wait for dialog to open in one of the tabs.
+  info("Waiting for dialog to appear");
+  let dialog = await dialogWindowPromise;
+
+  is(
+    gBrowser.getTabDialogBox(newTab.linkedBrowser)._tabDialogManager._topDialog,
+    dialog,
+    "Dialog opened in the background tab"
+  );
+
+  is(
+    dialog._frame.contentDocument.location.href,
+    CONTENT_HANDLING_URL,
+    "Opened dialog is appChooser dialog."
+  );
+
+  // Close the dialog:
+  let dialogClosedPromise = waitForProtocolAppChooserDialog(gBrowser, false);
+  dialog.close();
+  await dialogClosedPromise;
+
+  gBrowser.removeTab(tab);
+  gBrowser.removeTab(newTab);
+});
+
+/**
+ * Check that when navigating to a http channel which redirects to a external
+ * protocol in a noopener pop-up window, we show the dialog in the correct tab.
+ */
+add_task(async function redirect_popup_tab() {
+  let tab = await BrowserTestUtils.openNewForegroundTab(
+    gBrowser,
+    "https://example.com/"
+  );
+
+  // Wait for the chooser dialog to open in the background tab. It should not
+  // open in the foreground tab which is unrelated to the external protocol
+  // navigation.
+  let dialogWindowPromise = waitForProtocolAppChooserDialog(gBrowser, true);
+
+  // Wait for the new tab to appear. The URI in this tab will never change from
+  // `about:blank` as we're going to just end up opening a dialog, so we can't
+  // use `waitForNewTab`, as that will wait for the tab to actually load
+  // something.
+  let newTabPromise = new Promise(resolve => {
+    gBrowser.tabContainer.addEventListener(
+      "TabOpen",
+      openEvent => resolve(openEvent.target),
+      { once: true }
+    );
+  });
+
+  info("Navigating to redirect to external proto in pop-up");
+  await SpecialPowers.spawn(
+    tab.linkedBrowser,
+    [TEST_PATH + "redirect_helper.sjs?uri=mailto:example@example.com"],
+    async function (popupUri) {
+      content.eval("window.open('" + popupUri + "', '_blank', 'noopener');");
+    }
+  );
+
+  // Wait for the new tab to be opened.
+  info("Waiting for new tab to appear");
+  let newTab = await newTabPromise;
+
+  // Wait for dialog to open in one of the tabs.
+  info("Waiting for dialog to appear");
+  let dialog = await dialogWindowPromise;
+
+  is(
+    gBrowser.getTabDialogBox(newTab.linkedBrowser)._tabDialogManager._topDialog,
+    dialog,
+    "Dialog opened in the background tab"
+  );
+
+  is(
+    dialog._frame.contentDocument.location.href,
+    CONTENT_HANDLING_URL,
+    "Opened dialog is appChooser dialog."
+  );
+
+  // Close the dialog:
+  let dialogClosedPromise = waitForProtocolAppChooserDialog(gBrowser, false);
+  dialog.close();
+  await dialogClosedPromise;
+
+  gBrowser.removeTab(tab);
+  gBrowser.removeTab(newTab);
+});
