@@ -15,6 +15,7 @@
 #include "nsIWidgetListener.h"
 #include "Units.h"
 #include "mozilla/Attributes.h"
+#include "mozilla/CallState.h"
 #include "mozilla/EventForwards.h"
 #include "mozilla/UniquePtr.h"
 
@@ -24,6 +25,9 @@ class nsIFrame;
 
 namespace mozilla {
 class PresShell;
+namespace dom {
+class BrowserParent;
+}  // namespace dom
 namespace widget {
 struct InitData;
 enum class TransparencyMode : uint8_t;
@@ -96,14 +100,6 @@ enum class WindowType : uint8_t;
 //        the layer's parent.
 enum class ViewVisibility : uint8_t { Hide = 0, Show = 1 };
 
-// Public view flags
-
-// Indicates that the view is using auto z-indexing
-#define NS_VIEW_FLAG_AUTO_ZINDEX 0x0004
-
-// Indicates that the view is a floating view.
-#define NS_VIEW_FLAG_FLOATING 0x0008
-
 //----------------------------------------------------------------------
 
 /**
@@ -224,16 +220,6 @@ class nsView final : public nsIWidgetListener {
    */
   ViewVisibility GetVisibility() const { return mVis; }
 
-  /**
-   * Get whether the view "floats" above all other views,
-   * which tells the compositor not to consider higher views in
-   * the view hierarchy that would geometrically intersect with
-   * this view. This is a hack, but it fixes some problems with
-   * views that need to be drawn in front of all other views.
-   * @result true if the view floats, false otherwise.
-   */
-  bool GetFloating() const { return (mVFlags & NS_VIEW_FLAG_FLOATING) != 0; }
-
   /**
    * Called to query the parent of the view.
    * @result view's parent
@@ -279,24 +265,11 @@ class nsView final : public nsIWidgetListener {
    * CreateWidget*() will look around in the view hierarchy for an
    * appropriate parent widget for the view.
    *
-   * @param aWidgetInitData data used to initialize this view's widget before
-   *        its create is called.
    * @return error status
    */
-  nsresult CreateWidget(mozilla::widget::InitData* aWidgetInitData = nullptr,
-                        bool aEnableDragDrop = true,
+  nsresult CreateWidget(nsIWidget* aParent, bool aEnableDragDrop = true,
                         bool aResetVisibility = true);
 
-  /**
-   * Create a widget for this view with an explicit parent widget.
-   * |aParentWidget| must be nonnull.  The other params are the same
-   * as for |CreateWidget()|.
-   */
-  nsresult CreateWidgetForParent(nsIWidget* aParentWidget,
-                                 mozilla::widget::InitData* = nullptr,
-                                 bool aEnableDragDrop = true,
-                                 bool aResetVisibility = true);
-
   /**
    * Create a popup widget for this view.  Pass |aParentWidget| to
    * explicitly set the popup's parent.  If it's not passed, the view
@@ -304,8 +277,7 @@ class nsView final : public nsIWidgetListener {
    * other params are the same as for |CreateWidget()|, except that
    * |aWidgetInitData| must be nonnull.
    */
-  nsresult CreateWidgetForPopup(mozilla::widget::InitData*,
-                                nsIWidget* aParentWidget = nullptr);
+  nsresult CreateWidgetForPopup(mozilla::widget::InitData*, nsIWidget* aParent);
 
   /**
    * Destroys the associated widget for this view.  If this method is
@@ -405,68 +377,41 @@ class nsView final : public nsIWidgetListener {
    * @param y new y position
    */
   void SetPosition(nscoord aX, nscoord aY);
-
-  /**
-   * Called to indicate that the z-index of a view has been changed.
-   * The z-index is relative to all siblings of the view.
-   * @param aAuto Indicate that the z-index of a view is "auto". An "auto"
-   *              z-index means that the view does not define a new stacking
-   *              context, which means that the z-indicies of the view's
-   *              children are relative to the view's siblings.
-   * @param zindex new z depth
-   */
-  void SetZIndex(bool aAuto, int32_t aZIndex);
-  bool GetZIndexIsAuto() const {
-    return (mVFlags & NS_VIEW_FLAG_AUTO_ZINDEX) != 0;
-  }
-  int32_t GetZIndex() const { return mZIndex; }
-
   void SetParent(nsView* aParent) { mParent = aParent; }
   void SetNextSibling(nsView* aSibling) {
     NS_ASSERTION(aSibling != this, "Can't be our own sibling!");
     mNextSibling = aSibling;
   }
 
-  nsRegion& GetDirtyRegion() {
-    if (!mDirtyRegion) {
-      NS_ASSERTION(!mParent || GetFloating(),
-                   "Only display roots should have dirty regions");
-      mDirtyRegion = mozilla::MakeUnique<nsRegion>();
-    }
-    return *mDirtyRegion;
-  }
-
   // nsIWidgetListener
-  virtual mozilla::PresShell* GetPresShell() override;
-  virtual nsView* GetView() override { return this; }
-  virtual bool WindowMoved(nsIWidget* aWidget, int32_t x, int32_t y,
-                           ByMoveToRect) override;
-  virtual bool WindowResized(nsIWidget* aWidget, int32_t aWidth,
-                             int32_t aHeight) override;
-#if defined(MOZ_WIDGET_ANDROID)
-  virtual void DynamicToolbarMaxHeightChanged(
-      mozilla::ScreenIntCoord aHeight) override;
-  virtual void DynamicToolbarOffsetChanged(
-      mozilla::ScreenIntCoord aOffset) override;
+  mozilla::PresShell* GetPresShell() override;
+  nsView* GetView() override { return this; }
+  bool WindowMoved(nsIWidget* aWidget, int32_t x, int32_t y,
+                   ByMoveToRect) override;
+  bool WindowResized(nsIWidget* aWidget, int32_t aWidth,
+                     int32_t aHeight) override;
+#ifdef MOZ_WIDGET_ANDROID
+  void DynamicToolbarMaxHeightChanged(mozilla::ScreenIntCoord aHeight) override;
+  void DynamicToolbarOffsetChanged(mozilla::ScreenIntCoord aOffset) override;
+  void KeyboardHeightChanged(mozilla::ScreenIntCoord aHeight) override;
+  void AndroidPipModeChanged(bool) override;
 #endif
-  virtual bool RequestWindowClose(nsIWidget* aWidget) override;
+  bool RequestWindowClose(nsIWidget* aWidget) override;
   MOZ_CAN_RUN_SCRIPT_BOUNDARY
-  virtual void WillPaintWindow(nsIWidget* aWidget) override;
+  void WillPaintWindow(nsIWidget* aWidget) override;
   MOZ_CAN_RUN_SCRIPT_BOUNDARY
-  virtual bool PaintWindow(nsIWidget* aWidget,
-                           LayoutDeviceIntRegion aRegion) override;
+  bool PaintWindow(nsIWidget* aWidget, LayoutDeviceIntRegion aRegion) override;
   MOZ_CAN_RUN_SCRIPT_BOUNDARY
-  virtual void DidPaintWindow() override;
-  virtual void DidCompositeWindow(
-      mozilla::layers::TransactionId aTransactionId,
-      const mozilla::TimeStamp& aCompositeStart,
-      const mozilla::TimeStamp& aCompositeEnd) override;
-  virtual void RequestRepaint() override;
-  virtual bool ShouldNotBeVisible() override;
+  void DidPaintWindow() override;
+  void DidCompositeWindow(mozilla::layers::TransactionId aTransactionId,
+                          const mozilla::TimeStamp& aCompositeStart,
+                          const mozilla::TimeStamp& aCompositeEnd) override;
+  void RequestRepaint() override;
+  bool ShouldNotBeVisible() override;
   MOZ_CAN_RUN_SCRIPT_BOUNDARY
-  virtual nsEventStatus HandleEvent(mozilla::WidgetGUIEvent* aEvent,
-                                    bool aUseAttachedEvents) override;
-  virtual void SafeAreaInsetsChanged(const mozilla::ScreenIntMargin&) override;
+  nsEventStatus HandleEvent(mozilla::WidgetGUIEvent* aEvent,
+                            bool aUseAttachedEvents) override;
+  void SafeAreaInsetsChanged(const mozilla::LayoutDeviceIntMargin&) override;
 
   virtual ~nsView();
 
@@ -504,23 +449,12 @@ class nsView final : public nsIWidgetListener {
    */
   void SetVisibility(ViewVisibility visibility);
 
-  /**
-   * Set/Get whether the view "floats" above all other views,
-   * which tells the compositor not to consider higher views in
-   * the view hierarchy that would geometrically intersect with
-   * this view. This is a hack, but it fixes some problems with
-   * views that need to be drawn in front of all other views.
-   * @result true if the view floats, false otherwise.
-   */
-  void SetFloating(bool aFloatingView);
-
   // Helper function to get mouse grabbing off this view (by moving it to the
   // parent, if we can)
   void DropMouseGrabbing();
 
-  bool HasNonEmptyDirtyRegion() {
-    return mDirtyRegion && !mDirtyRegion->IsEmpty();
-  }
+  bool IsDirty() const { return mIsDirty; }
+  void SetIsDirty(bool aDirty) { mIsDirty = aDirty; }
 
   void InsertChild(nsView* aChild, nsView* aSibling);
   void RemoveChild(nsView* aChild);
@@ -533,6 +467,10 @@ class nsView final : public nsIWidgetListener {
   // Update the cached RootViewManager for all view manager descendents.
   void InvalidateHierarchy();
 
+  void CallOnAllRemoteChildren(
+      const std::function<mozilla::CallState(mozilla::dom::BrowserParent*)>&
+          aCallback);
+
   nsViewManager* mViewManager;
   nsView* mParent;
   nsCOMPtr<nsIWidget> mWindow;
@@ -540,8 +478,6 @@ class nsView final : public nsIWidgetListener {
   nsView* mNextSibling;
   nsView* mFirstChild;
   nsIFrame* mFrame;
-  mozilla::UniquePtr<nsRegion> mDirtyRegion;
-  int32_t mZIndex;
   ViewVisibility mVis;
   // position relative our parent view origin but in our appunits
   nscoord mPosX, mPosY;
@@ -549,10 +485,10 @@ class nsView final : public nsIWidgetListener {
   nsRect mDimBounds;
   // in our appunits
   nsPoint mViewToWidgetOffset;
-  uint32_t mVFlags;
   bool mWidgetIsTopLevel;
   bool mForcedRepaint;
   bool mNeedsWindowPropertiesSync;
+  bool mIsDirty = false;
 };
 
 #endif
